package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.auth.client.LogoutClient;
import com.cusc.nirvana.user.auth.common.dto.LogoutDTO;
import com.cusc.nirvana.user.auth.identification.dto.Oauth2Token;
import com.cusc.nirvana.user.auth.identification.dto.RandomIdResp;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.enterprise.dto.CaptchaResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginSmsRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginUserNameReqDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IAuthService;
import com.cusc.nirvana.user.rnr.enterprise.util.ValidationUtil;
import com.cusc.nirvana.user.util.CuscStringUtils;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * @author stayAnd
 * @date 2022/4/12
 */
@RestController
@RequestMapping("/userEnterprise/auth")
@ApiModel("B端用户登录相关(车企、经销商)")
public class AuthController {

    @Resource
    private IAuthService userService;

    @Autowired
    private LogoutClient logoutClient;

    @GetMapping("/getGraphic")
    @ApiOperation("获取图片验证码")
    public Response<CaptchaResponseDTO> getGraphic(
            @RequestParam(value = "platformSource", required = false, defaultValue = "4") String platformSource) {
        return Response.createSuccess(userService.getGraphic(platformSource));
    }


    @PostMapping("/getSms")
    @ApiOperation("获取短信验证码")
    public Response getSms(@RequestBody LoginSmsRequestDTO mobileLoginReq) {
		//检查手机
		Response checkPhone = ValidationUtil.checkPhone(mobileLoginReq.getPhone());
		if (!checkPhone.isSuccess()) {
			throw new CuscUserException(checkPhone.getCode(),checkPhone.getMsg());
		}
        userService.getSms(mobileLoginReq);
        return Response.createSuccess();
    }

    @PostMapping("/login")
    @ApiOperation("登录-手机号+验证码登录")
	@SysLog(sysModule = SysModuleEnum.USER_SMS_LOGIN)
    public Response<LoginResponseDTO> login(@RequestBody LoginRequestDTO loginRequestDTO) {
		//检查手机
		Response checkPhone = ValidationUtil.checkPhone(loginRequestDTO.getPhone());
		if (!checkPhone.isSuccess()) {
			throw new CuscUserException(checkPhone.getCode(),checkPhone.getMsg());
		}
        return Response.createSuccess(userService.login(loginRequestDTO));
    }

    @GetMapping("/getLoginSecretKey")
    @ApiOperation("获取随机请求id-登录密码加密")
    public Response<RandomIdResp> getLoginSecretKey(
            @RequestParam(value = "platformSource", required = false, defaultValue = "4") String platformSource) {
        if (CuscStringUtils.isEmpty(platformSource)) {
            return Response.createSuccess(ResponseCode.PARAMETER_NULL.getMsg(), ResponseCode.PARAMETER_NULL.getCode());
        }
        return userService.getLoginSecretKey(platformSource);
    }

    @PostMapping("/login/userName")
    @ApiOperation(value = "登录-用户名登录", notes = "登录-用户名登录")
	@SysLog(sysModule = SysModuleEnum.USER_PWD_LOGIN)
    public Response<Oauth2Token> userNameLogin(@RequestBody LoginUserNameReqDTO bean) {
        if (CuscStringUtils.isEmpty(bean.getPlatformSource())
                || CuscStringUtils.isEmpty(bean.getUserName()) || CuscStringUtils.isEmpty(bean.getPassword())
                || CuscStringUtils.isEmpty(bean.getRequestId())|| CuscStringUtils.isEmpty(bean.getCaptchaCode())|| CuscStringUtils.isEmpty(bean.getCaptchaId())) {
            return Response.createError(ResponseCode.PARAMETER_NULL.getMsg(), ResponseCode.PARAMETER_NULL.getCode());
        }
        return Response.createSuccess(userService.userNameLogin(bean));
    }

    @PostMapping("/logout")
    @ApiOperation(value = "退出", notes = "退出")
	@SysLog(sysModule = SysModuleEnum.USER_LOGOUT)
    public Response logout(@RequestHeader("Authorization") String accessToken) {
        if (StringUtils.isEmpty(accessToken)) {
            return Response.createError("请在header中传递token");
        }
        accessToken = accessToken.replaceAll("bearer ", "");
        LogoutDTO bean = new LogoutDTO();
        bean.setApplicationId(UserSubjectUtil.getAppId());
        bean.setAccessToken(accessToken);
        return logoutClient.logout(bean);
    }
}
