package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.ImportSimDTO;
import com.cusc.nirvana.user.rnr.enterprise.excel.CarInfoRow;
import com.cusc.nirvana.user.rnr.enterprise.excel.utils.EasyExcelUtil;
import com.cusc.nirvana.user.rnr.enterprise.service.ICarInfoUploadService;
import com.cusc.nirvana.user.rnr.enterprise.service.impl.CarInfoImportServiceImpl;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FileRecordDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpCarInfoDTO;
import com.cusc.nirvana.user.rnr.fp.dto.ImageBase64DTO;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

/**
 * @className: CarInfoController
 * @description: 车辆信息
 * @author: jk
 * @date: 2022/6/28 16:05
 * @version: 1.0
 **/
@RestController
@RequestMapping("/car/info")
public class CarInfoController {

    @Resource
    private CarInfoImportServiceImpl carInfoImportService;

    @Resource
    private ICarInfoUploadService iCarInfoUploadService;

    @Resource
    private FileSystemClient fileSystemClient;

    @Value("${template.carInfo}")
    private String carInfoTemplate;

    @PostMapping("import")
    @ApiOperation(value = "导入车辆信息")
//    @SysLog(sysModule = SysModuleEnum.CAR_INFO_IMPORT)
    public Response<?> importSim(@RequestBody ImportSimDTO importSimDTO) {
        return carInfoImportService.importSim(importSimDTO);
    }

    @PostMapping("queryCarInfo")
    @ApiOperation(value = "车辆信息查询")
    public Response<PageResult<FpCarInfoDTO>> queryCarInfo(@RequestBody FpCarInfoDTO fpCarInfoDTO){

        return Response.createSuccess(iCarInfoUploadService.queryCarInfo(fpCarInfoDTO));
    }


    @GetMapping("/download/template")
    @ApiOperation(value = "车卡关系模板下载")
    @ApiImplicitParam(name = "fileType", value = "文件类型,3:车卡关系导入模板", required = true)
    public void downloadTemplate(@NotNull(message = "模板类型不能为空") Integer fileType, HttpServletResponse response) throws IOException {
        String url ="";
        if(null ==fileType){
            throw new CuscUserException("","文件类型不可为空");
        }

        FileDownloadDTO FileRecordDTO = new FileDownloadDTO();
        FileRecordDTO.setUuid(carInfoTemplate);

        Response<ImageBase64DTO> fileRecordDTOResponse = fileSystemClient.getBase64(FileRecordDTO);

        if(!fileRecordDTOResponse.isSuccess()){
            throw new CuscUserException("","没有查询到车辆信息模板!");
        }
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        String fileName = "车辆信息模板.xlsx";
        response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        byte[] bytes = Base64.getDecoder().decode(fileRecordDTOResponse.getData().getBase64());
        InputStream in = new ByteArrayInputStream(bytes);
        OutputStream outputStream = new BufferedOutputStream(response.getOutputStream());

        byte[] buff =new byte[1024];
        //所读取的内容使用n来接收
        int n;
        //当没有读取完时,继续读取,循环
        while((n=in.read(buff))!=-1){
            //将字节数组的数据全部写入到输出流中
            outputStream.write(buff,0,n);
        }
        //强制将缓存区的数据进行输出
        outputStream.flush();
        //关流
        outputStream.close();
        in.close();
    }
}
