package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.SecondHandUnbindRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.validator.groups.VehicleCardRnrGroup;
import com.cusc.nirvana.user.rnr.enterprise.service.ICardUnBindService;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * Description: 卡解绑控制层
 * <br />
 * CreateDate 2022-04-16 15:16:55
 *
 * @author yuyi
 **/
@RestController
@RequestMapping("/card/unbind")
public class CardUnBindController {

    @Autowired
    ICardUnBindService cardUnBindService;

    /**
     * Description:
     * <br />
     * CreateDate 2022-04-20 16:43:36
     *
     * @author yuyi
     **/
    @ApiOperation(value = "原车主解绑", notes = "原车主解绑")
    @PostMapping("/originalOwner")
	@SysLog(sysModule = SysModuleEnum.ORIGINALOWNER_UNBIND)
    public Response originalOwner(@Validated(VehicleCardRnrGroup.Submit.class) @RequestBody VehicleCardRnrDTO bean) {
        return cardUnBindService.originalOwner(bean);
    }



    @ApiOperation(value = "发送短信", notes = "发送短信")
    @PostMapping("/sendMessage")
    public Response sendMessage(@RequestBody VehicleCardRnrDTO bean) {
        return cardUnBindService.sendMessage(bean);
    }



    @ApiOperation(value = "轮询查询工单状态", notes = "轮询查询工单状态")
    @PostMapping("/checkOrderStatus")
    public Response checkOrderStatus(@RequestBody RnrOrderDTO bean) {
        return cardUnBindService.checkOrderStatus(bean);
    }

    @ApiOperation(value = "根据vin获取iccid", notes = "根据vin获取iccid")
    @PostMapping("/getIccidsByVin")
    public Response getIccidsByVin(@RequestBody VehicleCardRnrDTO bean) {
        return cardUnBindService.getIccidsByVin(bean);
    }



    /**
     * Description: 二手车现车主解绑-自然人
     * <br />
     * CreateDate 2022-04-20 16:43:36
     *
     * @author yuyi
     **/
    @ApiOperation(value = "二手车现车主解绑-自然人", notes = "二手车现车主解绑-自然人")
    @PostMapping("/secondHandPersonal")
	@SysLog(sysModule = SysModuleEnum.SECONDOWNER_UNBIND)
    public Response secondHandPersonal(@Valid @RequestBody SecondHandUnbindRequestDTO bean) {
        return cardUnBindService.secondHandPersonalUnbind(bean);
    }
}
