package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.ChangeTboxDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.validator.groups.VehicleCardRnrGroup;
import com.cusc.nirvana.user.rnr.enterprise.service.ICardUnBindService;
import com.cusc.nirvana.user.rnr.enterprise.service.IChangeTboxService;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Description: 更换车机控制层
 * <br />
 * CreateDate 2022-04-16 15:16:55
 *
 * @author yuyi
 **/
@RestController
@RequestMapping("/change/tbox")
public class ChangeTboxController {
    @Autowired
    private IChangeTboxService changeTboxService;

    @Autowired
    private ICardUnBindService cardUnBindService;

    /**
     * Description: 提交车机更换
     * <br />
     * CreateDate 2022-04-20 16:43:36
     *
     * @author yuyi
     **/
    @ApiOperation(value = "提交车机更换", notes = "提交车机更换")
    @PostMapping("/submit")
	@SysLog(sysModule = SysModuleEnum.CHANGE_BOX)
    public Response submit(@RequestBody ChangeTboxDTO bean) {
        return changeTboxService.submit(bean);
    }



    /**
     * 轮询查询工单状态
     * @param bean
     * @return
     */
    @ApiOperation(value = "轮询查询工单状态", notes = "轮询查询工单状态")
    @PostMapping("/checkOrderStatus")
    public Response checkOrderStatus(@RequestBody RnrOrderDTO bean) {
        return cardUnBindService.checkOrderStatus(bean);
    }

    /**
     * Description: 查询绑定结果
     * <br />
     * CreateDate 2022-04-20 16:43:36
     *
     * @author yuyi
     **/
    @ApiOperation(value = "查询绑定结果", notes = "查询绑定结果")
    @PostMapping("/queryResult")
    public Response queryResult(@Validated(VehicleCardRnrGroup.Query.class) @RequestBody ChangeTboxDTO bean) {
        return changeTboxService.queryResult(bean);
    }

}
