package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FileUploadDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 自然人实名图片上传与下载
 * 图片处理
 * 图片处理放到user-rnr-fp中
 * @author stayand
 */
@RestController
@RequestMapping("/cuscImage")
@Slf4j
public class CuscImageController {

    @Autowired
    private FileSystemClient fileSystemClient;

    //图片文件后缀常量
    private static final List<String> IMG_FILE_SUFFIX_LIST = Arrays.asList("jpg", "jpeg", "png", "gif", "svg", "bmp", "tif");

    @PostMapping("/upload")
    public Response upload(FileUploadDTO fileUploadDTO) throws Exception {
        if(fileUploadDTO.getFile() == null){
            return Response.createError("文件不能为空");
        }

		//文件大小必须大于0
		if (fileUploadDTO.getFile().getSize() < 1) {
			return Response.createError("文件内容不能为空");
		}

        String originalFilename = fileUploadDTO.getFile().getOriginalFilename();
        //检查文件后缀
        Response fileSuffixResp = chechImgSuffix(originalFilename);
        if(!fileSuffixResp.isSuccess()){
            return fileSuffixResp;
        }
		return fileSystemClient.uploadImage(fileUploadDTO);
    }


    /**
     * 文件下载
     *
     * @param fileDownloadDTO
     * @return
     */
    @PostMapping("/viewBase64")
    public Response viewBase64(HttpServletResponse response, @RequestBody FileDownloadDTO fileDownloadDTO) {
        if (fileDownloadDTO == null || StringUtils.isBlank(fileDownloadDTO.getUuid())) {
            Response.createError("参数uuid为空");
        }
        return fileSystemClient.getBase64(fileDownloadDTO);
    }

    //------------私有方法区域-------------
    private Response chechImgSuffix(String fileName) {
        if(CuscStringUtils.isEmpty(fileName)){
            return Response.createError("图片名称不能为空");
        }
        int startIndex = fileName.lastIndexOf(".");
        if (startIndex <= 0) {
            return Response.createError("请上传规定的图片格式");
        }
        String suffixName = fileName.substring(startIndex+1).toLowerCase();
        if (IMG_FILE_SUFFIX_LIST.contains(suffixName)) {
            return Response.createSuccess();
        }
        return Response.createError("请上传规定的图片格式");
    }
}
