package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.dto.*;
import com.cusc.nirvana.user.rnr.enterprise.service.IDistributorService;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;


/**
 * @author stayAnd
 * @date 2022/4/14
 *
 */
@RestController
@RequestMapping("/userEnterprise/distributor")
@ApiModel("经销商")
public class DistributorController {

	@Resource
	private IDistributorService distributorService;

	@PostMapping("/add")
	@ApiOperation("新增经销商")
	public Response add(@Validated @RequestBody DistributorAddDTO dto){
		distributorService.add(dto);
		return Response.createSuccess();
	}

	@PostMapping("/page")
	@ApiOperation("分页查询经销商")
	public Response<PageResult<DistributorDTO>> page(@RequestBody DistributorPageQueryDTO dto){
//		return Response.createSuccess(distributorService.page(dto));
		Map<String,Object> selectMap = distributorService.pageToMap(dto);
		Integer errorCode = (Integer) selectMap.get("errorCode");
		String errorMsg = (String) selectMap.get("errorMsg");
		if (errorCode != 10000) {
			return Response.createError(errorMsg,errorCode);
		} else {
			PageResult<DistributorDTO> list = (PageResult<DistributorDTO>) selectMap.get("returnList");
			return Response.createSuccess(list);
		}
	}

	@PostMapping("/update")
	@ApiOperation("更新经销商")
	public Response update(@RequestBody DistributorDTO distributorDTO){
		distributorService.update(distributorDTO);
		return Response.createSuccess();
	}

	@PostMapping("/changeAdmin")
	@ApiOperation("切换经销商管理员")
	public Response changeAdmin(@RequestBody DistributorChangeAdminDTO dto){
		distributorService.changeAdmin(dto);
		return Response.createSuccess();
	}

}
