package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseCorporationChangeDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseRnrPersonDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.RnrResponseListDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.validator.groups.EnterpriseCorporationChangeGroup;
import com.cusc.nirvana.user.rnr.enterprise.service.IEnterpriseRnrService;
import com.cusc.nirvana.user.rnr.fp.common.RnrResultEnum;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 企业责任人变更
 *
 * @author huchenhui
 * @date 2022-04-14
 */
@RestController
@RequestMapping("/enterprise/corporation/change")
public class EnterpriseCorporationChangeController {

	@Resource
	private IEnterpriseRnrService enterpriseRnrService;

    /**
     * Description: 查询企业信息
     * <br />
     * CreateDate 2022-04-20 16:43:36
     *
     * @author yuyi
     **/
    @ApiOperation(value = "查询企业信息", notes = "查询企业信息")
    @PostMapping("/queryEnterprise")
    public Response<RnrResponseListDTO> queryEnterprise(@Validated(EnterpriseCorporationChangeGroup.Query.class)  @RequestBody EnterpriseCorporationChangeDTO dto) {
        RnrResponseListDTO<EnterpriseRnrPersonDTO> ret = new RnrResponseListDTO<>();
//		List<EnterpriseRnrPersonDTO> list = enterpriseRnrService.queryEnterpriseRnrPerson(dto);
//		ret.setList(list);
        Map<String, Object> selectMap = enterpriseRnrService.queryEnterpriseRnrPerson(dto);
        Integer errorCode = (Integer) selectMap.get("errorCode");
        String errorMsg = (String) selectMap.get("errorMsg");
        if (errorCode != 10000) {
            return Response.createError(errorMsg,errorCode);
        } else {
            List<EnterpriseRnrPersonDTO> list = (List<EnterpriseRnrPersonDTO>) selectMap.get("returnList");
            ret.setList(list);
            return Response.createSuccess(ret);
        }
    }

    /**
     * Description: 提交企业责任人变更
     * <br />
     * CreateDate 2022-04-20 16:43:36
     *
     * @author yuyi
     **/
    @ApiOperation(value = "提交企业责任人变更", notes = "提交企业责任人变更")
    @PostMapping("/submit")
	@SysLog(sysModule = SysModuleEnum.ENTERPRISE_CHANGE_RNR)
    public Response submit(@Validated(EnterpriseCorporationChangeGroup.Submit.class)  @RequestBody EnterpriseCorporationChangeDTO bean) {
    	enterpriseRnrService.submitChange(bean);
        return Response.createSuccess(RnrResultEnum.REVIEW.getCode());
    }
}
