package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.CompanyRnrResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EMVinCardResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseRnrCertificationInfoDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseRnrVinCardInfoDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IEnterpriseRnrService;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 企业实名认证控制层
 *
 * @author huchenhui
 * @date 2022-04-14
 */
@RestController
@RequestMapping("/enterpriseRnr")
public class EnterpriseRnrController {

    @Resource
    private IEnterpriseRnrService enterpriseRnrService;

    /**
     * 企业实名认证上传Excel文件，需要传入文件系统id，从文件系统下载后解析，成功则继续，失败则删除原文件，新文件添加失败原因后重新上传至文件系统。
     *
     * @param dto dto bean
     * @return 单条数据
     */
    @ApiOperation(value = "校验车卡信息", notes = "企业实名认证校验车卡信息")
    @PostMapping("/verifyVinCards")
    public Response<EMVinCardResponseDTO> verifyVinCards(@RequestBody EnterpriseRnrVinCardInfoDTO dto) {
        return enterpriseRnrService.verifyVinCards(dto);
    }

    /**
     * 提交企业实名认证
     *
     * @param dto dto bean
     * @return 单条数据
     */
    @ApiOperation(value = "提交企业实名认证", notes = "提交企业实名认证")
    @PostMapping("/submitRnr")
	@SysLog(sysModule = SysModuleEnum.ENTERPRISE_WEB_RNR)
    public Response<CompanyRnrResponseDTO> submitRnr(@RequestBody EnterpriseRnrCertificationInfoDTO dto) {
        return enterpriseRnrService.submitRnr(dto);
    }
}
