package com.cusc.nirvana.user.rnr.enterprise.controller;


import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileUploadDTO;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.dto.FileRecordDTO;
import com.cusc.nirvana.user.rnr.fp.dto.ImageBase64DTO;
import com.cusc.nirvana.web.exception.AppGlobalException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.List;

/**
 * @author stayAnd
 * @date 2022/4/12
 */
@RestController
@RequestMapping("/file")
@Slf4j
public class FileController {

    @Resource
    private FileSystemClient fileSystemClient;

    //文件后缀常量
    private static final List<String> FILE_SUFFIX_LIST =
            Arrays.asList("jpg","jpeg", "png", "gif", "bmp", "pdf", "mp4", "avi", "wav",
                    "rm","xls","xlsx","doc","docx");

    @PostMapping("/getBase64ByIds")
    public Response<List<ImageBase64DTO>> getBase64ByIds(@RequestBody FileDownloadDTO fileDownload) throws Exception {
        if (CollectionUtils.isEmpty(fileDownload.getFileSystemIds())) {
            Response.createError("参数文件ids为空");
        }
        List<ImageBase64DTO> base64DTOList = new ArrayList<>(fileDownload.getFileSystemIds().size());
        //复用接口，循环获取
        fileDownload.getFileSystemIds().forEach(fileSystemId -> {
            com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO fileDownloadDTO =
                    new com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO();
            fileDownloadDTO.setUuid(fileSystemId);
            Response<ImageBase64DTO> base64DTOResponse = fileSystemClient.getBase64(fileDownloadDTO);
            if (base64DTOResponse.isSuccess() && base64DTOResponse.getData() != null) {
                base64DTOList.add(base64DTOResponse.getData());
            }
        });
        return Response.createSuccess(base64DTOList);
    }

    @PostMapping("/upload")
    public Response<FileRecordDTO> upload(FileUploadDTO fileUploadDTO) throws Exception {
		//文件大小必须大于0
		if (fileUploadDTO.getFile().getSize() < 1) {
			return Response.createError("文件内容不能为空");
		}
        String originalFilename = fileUploadDTO.getFile().getOriginalFilename();
        //检查文件后缀
        Response fileSuffixResp =  chechFileSuffix(originalFilename);
        if(!fileSuffixResp.isSuccess()){
            return fileSuffixResp;
        }
		com.cusc.nirvana.user.rnr.fp.dto.FileUploadDTO dto = new com.cusc.nirvana.user.rnr.fp.dto.FileUploadDTO();
		dto.setFile(fileUploadDTO.getFile());
		return fileSystemClient.uploadFile(dto);
    }

    @PostMapping("/getAccessUrl")
    public String getAccessUrl(@RequestBody FileUploadDTO fileUploadDTO) throws Exception {
        //return MinIOUtils.getPresignedObjectUrl(minIOConfig.getBucketName(), fileUploadDTO.getUuid());
        return null;
    }

    /**
     * 文件下载
     *
     * @param fileDownloadDTO
     * @return
     */
    @PostMapping("/download")
    public Response download(HttpServletResponse response, @RequestBody FileDownloadDTO fileDownloadDTO) {
        try {

            response.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode(
                            StringUtils.isNotEmpty(fileDownloadDTO.getFileName()) ? fileDownloadDTO.getFileName() :
                                    "文件.xlsx", "utf-8"));
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");

            byte[] result = null;
            try {
                log.info("下载文件-开始, uuid : {}", fileDownloadDTO.getUuid());

				com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO dto = new com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO();
				dto.setUuid(fileDownloadDTO.getUuid());
				Response<ImageBase64DTO> base64Response = fileSystemClient.getBase64(dto);

				String base64 = base64Response.getData().getBase64();

				result = Base64.getDecoder().decode(base64);

				log.info("下载文件-结束, uuid : {}", fileDownloadDTO.getUuid());
            } catch (Exception e) {
                log.error("getImageBase64 error: {}", e.getMessage(), e);
                return Response.createError(e.getMessage());
            }
            //InputStream fileStream = MinIOUtils.getObject(minIOConfig.getBucketName(), fileDownloadDTO.getUuid());
            response.getOutputStream().write(result);
        } catch (Exception e) {
            throw new AppGlobalException("下载失败", e);
        } finally {
            try {
                if (response.getOutputStream() != null) {
                    response.getOutputStream().close();
                }
            } catch (IOException e) {
                log.error("---response.getOutputStream().close()--exception--", e);
            }
        }

        return Response.createSuccess();
    }

    /**
     * Description: 文件后缀检查
     * <br />
     * CreateDate 2022-07-09 23:56:03
     *
     * @author yuyi
     **/
    private Response chechFileSuffix(String fileName) {
        int startIndex = fileName.lastIndexOf(".");
        if (startIndex <= 0) {
            return Response.createError("请上传规定的文件格式");
        }
        String suffixName = fileName.substring(startIndex+1).toLowerCase();
        if (FILE_SUFFIX_LIST.contains(suffixName)) {
            return Response.createSuccess();
        }
        return Response.createError("请上传规定的文件格式");
    }
}
