package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrClient;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrOrderClient;
import com.cusc.nirvana.user.rnr.fp.dto.RnrOrderListQueryDTO;
import com.cusc.nirvana.user.rnr.mg.client.LocalVerifyClient;
import com.cusc.nirvana.user.rnr.mg.dto.*;
import com.cusc.nirvana.user.util.DateUtils;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController
@RequestMapping("/localverify")
@Slf4j
public class LocalVerifyController {

    @Autowired
    private LocalVerifyClient localVerifyClient;
    @Autowired
    private FpRnrClient fpRnrClient;

    @Resource
	private FpRnrOrderClient fpRnrOrderClient;

    /**
     * 本地人工审核列表
     */
    @ApiOperation(value = "本地人工审核列表", notes = "本地人工审核列表")
    @PostMapping("/list")
    public Response list(@RequestBody LocalVerifyListRqDTO rq) {
		RnrOrderListQueryDTO query = new RnrOrderListQueryDTO();
		query.setVin(rq.getVin());
		query.setIccid(rq.getIccid());
		query.setCreateTimeQueryStart(DateUtils.parseDate(rq.getRnrStartDate(),"yyyy-MM-dd HH:mm:ss"));
		query.setCreateTimeQueryEnd(DateUtils.parseDate(rq.getRnrEndDate(),"yyyy-MM-dd HH:mm:ss"));
		query.setFullName(rq.getVinOwnerName());
		query.setPhone(rq.getVinOwnerPhone());
		if (StringUtils.isNotBlank(rq.getOrderType())) {
			query.setOrderType(Integer.parseInt(rq.getOrderType()));
		}
		query.setCompanyName(rq.getVinCompanyName());
		query.setOrganizationName(rq.getOrgName());
		query.setCurrPage(rq.getCurrPage());
		query.setPageSize(rq.getPageSize());
		return fpRnrOrderClient.localverifyList(query);
    }

    /**
     * 查询工单企业详情
     *
     * @return 单条数据
     */
    @ApiOperation(value = "查询工单企业详情", notes = "查询工单企业详情")
    @PostMapping("/getEnterpriseDetail")
    public Response<EnterpriseVerifyDetailDTO> getEnterpriseDetail(@RequestBody RnrOrderDTO rnrOrderDTO) {
        rnrOrderDTO.setUserId(UserSubjectUtil.getUserId());
        rnrOrderDTO.setApplicationId(UserSubjectUtil.getAppId());
        rnrOrderDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        return Response.createSuccess(localVerifyClient.getEnterpriseDetail(rnrOrderDTO));
    }


    /**
     * 企业负责人变更审核详情
     * @return 单条数据
     */
    @ApiOperation(value = "企业负责人变更审核详情", notes = "企业负责人变更审核详情")
    @PostMapping("/enterpriseRnrPersonDetail")
    public Response<CompanyCorporationChangeRespDTO> enterpriseRnrPersonDetail(@RequestParam RnrOrderDTO dto) {
        return Response.createSuccess(localVerifyClient.enterpriseRnrPersonDetail(dto.getUuid()));
    }



    /**
     * 自然人审核详情
     */
    @ApiOperation(value = "自然人审核详情", notes = "自然人审核详情")
    @PostMapping("/personDetail")
    public Response personDetail(@RequestBody LocalVerifyListRqDTO rq) {
        rq.setUserId(UserSubjectUtil.getUserId());
        rq.setApplicationId(UserSubjectUtil.getAppId());
        rq.setTenantNo(UserSubjectUtil.getTenantNo());
        return localVerifyClient.personDetail(rq);
    }

    /**
     * 提交
     */
    @ApiOperation(value = "提交审核", notes = "提交审核")
    @PostMapping("/submit")
    public Response submit(@RequestBody LocalVerifySubmitRq rq) {
        rq.setUserId(UserSubjectUtil.getUserId());
        return localVerifyClient.submit(rq);
    }

    /**
     * 重试按钮
     */
    @ApiOperation(value = "重试按钮", notes = "重试")
    @PostMapping("/retry")
    public Response retry(@RequestBody LocalVerifySubmitRq rq) {
        Response<RnrRelationDTO> response =localVerifyClient.retry(rq);
        RnrRelationDTO bean = response.getData();
        log.info("mg retry return:"+ JSONObject.toJSONString(bean));
        return fpRnrClient.personalInfoOnlyRnr(bean);
    }


}
