package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.auth.client.RandomIdClient;
import com.cusc.nirvana.user.auth.identification.dto.RandomIdReq;
import com.cusc.nirvana.user.eiam.client.UserClient;
import com.cusc.nirvana.user.eiam.constants.CommonStatusEnum;
import com.cusc.nirvana.user.eiam.dto.UserDTO;
import com.cusc.nirvana.user.eiam.dto.UserPasswordDTO;
import com.cusc.nirvana.user.rnr.enterprise.config.EnterpriseConfig;
import com.cusc.nirvana.user.rnr.enterprise.constants.EnterpriseLoginSourceEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.enterprise.convert.UserPasswordConvert;
import com.cusc.nirvana.user.rnr.enterprise.dto.CaptchaResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginPasswordReqDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.UserPasswReqDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import com.cusc.nirvana.user.util.crypt.Sm4Util;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * Description: 登录密码控制层
 * <br />
 * CreateDate 2022-06-11 21:39:39
 *
 * @author yuyi
 **/
@RestController
@RequestMapping("/password")
@ApiModel("登录密码控制层")
public class LoginPasswordController {

    @Resource
    private UserClient userClient;

    @Resource
    private RandomIdClient randomIdClient;

    @Resource
    private EnterpriseConfig enterpriseConfig;

    @PostMapping("/forget")
    @ApiOperation("忘记密码")
    public Response<Boolean> forget(@RequestBody UserPasswReqDTO bean) {
        bean.setTenantNo(enterpriseConfig.getTenantNo());
        if (CuscStringUtils.isEmpty(bean.getSmsCode()) || CuscStringUtils.isEmpty(bean.getPhone()) || CuscStringUtils.isEmpty(bean.getPlatformSource())) {
            return Response.createError(ResponseCode.PARAMETER_NULL.getMsg(), ResponseCode.PARAMETER_NULL.getCode());
        }

        //指定忘记密码的短信模板
        //SmsSendConfig smsConfig = new SmsSendConfig();
        //smsConfig.setStrategyCode("");
        //smsConfig.setSmsTemplateCode("");

        UserPasswordDTO pwd = UserPasswordConvert.INSTANCE.userPasswordReqToEiamDTO(bean);
        pwd.setApplicationId(EnterpriseLoginSourceEnum.getByPlatform(bean.getPlatformSource()).getAppId());
        //pwd.setSmsSendConfig(smsConfig);

        //密码不为空，则获取密码
        if(CuscStringUtils.isNotEmpty(bean.getPassword())){
            RandomIdReq randomIdReq = new RandomIdReq();
            randomIdReq.setRequestId(bean.getRequestId());
            randomIdReq.setApplicationId(EnterpriseLoginSourceEnum.getByPlatform(bean.getPlatformSource()).getAppId());
            Response<String> keyResp =  randomIdClient.getContentByRequestId(randomIdReq);
            if(!keyResp.isSuccess() || CuscStringUtils.isEmpty(keyResp.getData())){
                return Response.createError(ResponseCode.USER_PASSWORD_KEY_NOT_FOUND.getMsg(), ResponseCode.USER_PASSWORD_KEY_NOT_FOUND.getCode());
            }
            String secretKey = keyResp.getData();
            randomIdClient.delByRequestId(randomIdReq);
            pwd.setPassword(Sm4Util.decryptEcbPadding(secretKey, bean.getPassword()));
        }

        //根据手机号查询用户id
        UserDTO userDTO = new UserDTO();
        userDTO.setPhone(bean.getPhone());
        userDTO.setTenantNo(enterpriseConfig.getTenantNo());
        userDTO.setApplicationId(EnterpriseLoginSourceEnum.getByPlatform(bean.getPlatformSource()).getAppId());
        userDTO.setStatus(CommonStatusEnum.ENABLE.getCode());
        Response<UserDTO> userResp = userClient.getByPhone(userDTO);
        if(!userResp.isSuccess() || userResp.getData() == null){
            return Response.createError(ResponseCode.USER_NAME_PASSWORD_ERROR.getMsg(), ResponseCode.USER_NAME_PASSWORD_ERROR.getCode());
        }
        pwd.setUuid(userResp.getData().getUuid());
        return userClient.resetPassword(pwd);
    }

    @PostMapping("/change")
    @ApiOperation("修改密码")
    public Response<CaptchaResponseDTO> change(@RequestBody LoginPasswordReqDTO bean) {
        bean.setTenantNo(enterpriseConfig.getTenantNo());
        if (CuscStringUtils.isEmpty(bean.getPassword()) || CuscStringUtils.isEmpty(bean.getNewPassword()) || CuscStringUtils.isEmpty(bean.getRequestId())) {
            return Response.createError(ResponseCode.PARAMETER_NULL.getMsg(), ResponseCode.PARAMETER_NULL.getCode());
        }

        UserDTO userDTO = new UserDTO();
        userDTO.setUuid(UserSubjectUtil.getUserId());
        userDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        userDTO.setApplicationId(UserSubjectUtil.getAppId());

        RandomIdReq randomIdReq = new RandomIdReq();
        randomIdReq.setRequestId(bean.getRequestId());
        randomIdReq.setApplicationId(UserSubjectUtil.getAppId());
        Response<String> keyResp =  randomIdClient.getContentByRequestId(randomIdReq);
        if(!keyResp.isSuccess() || CuscStringUtils.isEmpty(keyResp.getData())){
            return Response.createError(ResponseCode.USER_PASSWORD_KEY_NOT_FOUND.getMsg(), ResponseCode.USER_PASSWORD_KEY_NOT_FOUND.getCode());
        }
        String secretKey = keyResp.getData();
        randomIdClient.delByRequestId(randomIdReq);
        userDTO.setPassword(Sm4Util.decryptEcbPadding(secretKey, bean.getPassword()));
        userDTO.setNewPassword(Sm4Util.decryptEcbPadding(secretKey, bean.getNewPassword()));
        return Response.createSuccess(userClient.changePassword(userDTO));
    }
}
