package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.enterprise.service.impl.SmsServiceImpl;
import com.cusc.nirvana.user.rnr.fp.client.SmsClient;
import com.cusc.nirvana.user.rnr.fp.constants.BizTypeEnum;
import com.cusc.nirvana.user.rnr.fp.dto.SmsRequestDTO;
import com.cusc.nirvana.user.rnr.fp.dto.UnbindReceiceSMSDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author yubo
 * @since 2022-05-06 16:34
 */
@RestController
@RequestMapping("/message")
@Slf4j
public class MessageController {

    @Autowired
    private SmsClient smsClient;

    @Autowired
    SmsServiceImpl smsService;

    @ApiOperation(value = "短信回调", notes = "短信回调")
    @PostMapping("/callBack")
    public Response callBack(@RequestBody UnbindReceiceSMSDTO bean) {
        log.info("message callback rq : {}", JSON.toJSONString(bean));
        return smsClient.callBack(bean);
    }

    @ApiOperation(value = "短信验证码核验", notes = "短信验证码核验")
    @PostMapping("/checkSmsCaptcha")
    public Response checkSmsCaptcha(@RequestBody SmsRequestDTO bean) {
        if(CuscStringUtils.isEmpty(bean.getPhone()) || CuscStringUtils.isEmpty(bean.getCaptcha())){
            return Response.createError(ResponseCode.PARAMETER_NULL.getMsg(), ResponseCode.PARAMETER_NULL.getCode());
        }
        bean.setTenantNo(UserSubjectUtil.getTenantNo());
        if (StringUtils.isBlank(bean.getBizType())) {
			bean.setBizType(BizTypeEnum.RNR.getCode());
		}

        Response smsResponse = smsService.checkSmsCaptcha(bean);
        if (!smsResponse.isSuccess()) {
            return Response.createError(smsResponse.getMsg(), smsResponse.getCode());
        }
        return smsResponse;
    }
}
