package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.dto.UserDTO;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.constants.OrganBizzTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.dto.*;
import com.cusc.nirvana.user.rnr.enterprise.service.IOrganService;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.weaver.Lint;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.w3c.dom.stylesheets.LinkStyle;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * Description: B端组织管理控制层
 * <br />
 * CreateDate 2022-05-19 16:14:06
 *
 * @author yuyi
 **/
@RestController
@RequestMapping("/organ")
@ApiModel("B端组织管理控制层")
@Slf4j
public class OrganController {

	@Resource
	private IOrganService organService;

	@ApiOperation("创建车企组织")
	@PostMapping("/create")
	@SysLog(sysModule = SysModuleEnum.ADD_ORGAN)
	public Response createCarOrgan(@Validated @RequestBody OrganDTO bean){
		log.warn("创建车企组织入参{}", JSONObject.toJSONString(bean));
		organService.add(bean);
		return Response.createSuccess();
	}

	@ApiOperation("查询车企组织")
	@PostMapping("/pageList")
	public Response<PageResult<OrganDTO>> queryPage(@RequestBody OrganPageDTO bean){
		log.warn("查询车企组织入参{}", JSONObject.toJSONString(bean));
		return Response.createSuccess(organService.page(bean));
	}

	@ApiOperation("更新车企组织")
	@PostMapping("/update")
	@SysLog(sysModule = SysModuleEnum.UPDATE_ORGAN)
	public Response update(@Validated @RequestBody UpdateOrganDTO bean){
		log.warn("更新车企组织入参{}", JSONObject.toJSONString(bean));
		organService.update(bean);
		return Response.createSuccess();
	}

	@ApiOperation("变更管理员")
	@PostMapping("/changeAdmin")
	public Response changeAdmin(@Validated @RequestBody OrganChangeAdminDto dto){
		log.warn("变更管理员入参{}", JSONObject.toJSONString(dto));
		organService.changeAdmin(dto);
		return Response.createSuccess();
	}

	@ApiOperation("用户账号列表")
	@PostMapping("/queryUserAccount")
	public Response<List<UserAccountDTO>> queryUserAccount(@Validated @RequestBody OrganUserAccountDTO dto){
		log.warn("用户账号列表入参{}", JSONObject.toJSONString(dto));
		return Response.createSuccess(organService.queryUserAccount(dto));
	}

	@ApiOperation("组织类型")
	@PostMapping("/queryOrganType")
	public Response queryOrganType(){
		OrganBizzTypeEnum[] organBizzTypeEnums = OrganBizzTypeEnum.getEnumByCodeMap();
		List<OrganBizTypeDTO> organBizTypeDTOS= new ArrayList<>();
		for(OrganBizzTypeEnum  organBizzTypeEnum : organBizzTypeEnums){
			OrganBizTypeDTO organBizTypeDTO= new OrganBizTypeDTO();
			organBizTypeDTO.setCode(organBizzTypeEnum.getCode());
			organBizTypeDTO.setName(organBizzTypeEnum.getName());
			organBizTypeDTOS.add(organBizTypeDTO);
		}
		return Response.createSuccess(organBizTypeDTOS);
	}


	@GetMapping("/queryCurrentSubOrgList")
	public Response<List<OrganDTO>> queryCurrentSubOrgList(){
		return Response.createSuccess(organService.queryCurrentSubOrgList());
	}
}
