package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VinDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IPersonalRnrService;
import com.cusc.nirvana.user.rnr.fp.dto.IccidListDTO;
import com.cusc.nirvana.user.rnr.fp.dto.LivenessCodeResponseDTO;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardInfoDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * 自然人实名认证控制层
 *
 * @author huchenhui
 * @date 2022-04-14
 */
@RestController
@RequestMapping("/personalRnr")
@Api(value = "自然人实名认证", tags = "自然人实名认证")
public class PersonalRnrController {

    @Resource
    private IPersonalRnrService personalRnrService;

    /**
     * 获取活体认证code
     */
    @ApiOperation(value = "获取活体认证code", notes = "获取活体认证code")
    @GetMapping("/getLivenessCode")
    public Response<LivenessCodeResponseDTO> getLivenessCode() {
        return personalRnrService.getLivenessCode();
    }

    /**
     * 通过VIN查询ICCID集合
     */
    @ApiOperation(value = "通过VIN查询ICCID集合", notes = "通过VIN查询ICCID集合")
    @PostMapping("/getIccidListByVin")
    public Response<IccidListDTO> getIccidListByVin(@RequestBody VinDTO vinDTO) {
        return personalRnrService.getIccidListByVin(vinDTO);
    }

    /**
     * 验证车卡信息
     */
    @ApiOperation(value = "验证车卡信息", notes = "验证车卡信息")
    @PostMapping("/verifyVinCards")
    public Response<PersonalRnrResponseDTO> verifyVinCards(@Valid @RequestBody VinCardInfoDTO requestDTO) {
        return personalRnrService.verifyVinCards(requestDTO);
    }

    /**
     * 提交自然人实名认证
     */
    @ApiOperation(value = "提交自然人实名认证", notes = "提交自然人实名认证")
    @PostMapping("/submitRnr")
	@SysLog(sysModule = SysModuleEnum.PERSON_WEB_RNR)
    public Response<PersonalRnrDTO> submitRnr(@Valid @RequestBody PersonalRnrRequestDTO requestDTO) {
        return personalRnrService.submitRnr(requestDTO);
    }
}
