package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.*;
import com.cusc.nirvana.user.rnr.enterprise.service.IPersonalRnrService;
import com.cusc.nirvana.user.rnr.fp.dto.H5ValidationOCRReqDTO;
import com.cusc.nirvana.user.rnr.fp.dto.RnrResponseDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * H5自然人实名认证
 *
 * @author huchenhui
 * @date 2022-04-14
 */
@RestController
@RequestMapping("/personalRnr/h5")
@Api(value = "H5自然人实名认证", tags = "H5自然人实名认证")
public class PersonalRnrH5Controller {

    @Resource
    private IPersonalRnrService personalRnrService;

    /**
     * h5自然人实名认证
     */
    @ApiOperation(value = "提交自然人实名认证", notes = "提交自然人实名认证")
    @PostMapping("/submitRnr")
	@SysLog(sysModule = SysModuleEnum.PERSON_H5_RNR)
    public Response<PersonalRnrH5RespDTO> submitRnr(@Valid @RequestBody PersonalRnrH5ReqDTO bean) {
        //本接口只入库不做实名，入库时  RnrOrderStatusEnum.COMMIT  ，工单类型为：RnrOrderType.NEW_VEHICLE，RnrOrderSourceEnum.H5
        return Response.createSuccess(personalRnrService.submitRnrH5(bean));
    }


    /**
     * h5验证身份证信息
     * @param bean
     * @return
     */
    @PostMapping("/h5ValidationOCR")
    public Response<RnrResponseDTO> h5ValidationOCR(@Valid @RequestBody H5ValidationOCRReqDTO bean) {
        return personalRnrService.h5ValidationOCR(bean);
    }


    /**
     * 活体检测结果回调
     */
    @ApiOperation(value = "活体检测结果回调", notes = "活体检测结果回调")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "操作成功返回数据类型", response = Response.class)})
    @RequestMapping(value = "/livenessCallback", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Response<PersonalRnrH5CallBackRespDTO> afreshLivenessUrl(@Valid @RequestBody LivenessCallbackReqDTO bean) {
        return Response.createSuccess(personalRnrService.afreshLivenessUrl(bean));
    }
}
