package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.constants.OrganBizzTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileMouldDto;
import com.cusc.nirvana.user.rnr.enterprise.dto.ProtocolManageDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.ProtocolManageUpdateDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IProtocolManageService;
import com.cusc.nirvana.user.rnr.enterprise.service.ISimVehicleService;
import com.cusc.nirvana.user.rnr.fp.constants.BizTypeEnum;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @className: FpRnrProtocolManageController
 * @description: 协议管理
 * @author: jk
 * @date: 2022/6/1 13:43
 * @version: 1.0
 **/
@RestController
@RequestMapping("/protocolManage")
@Slf4j
public class ProtocolManageController {

    @Autowired
    private IProtocolManageService protocolManageService;

    @Autowired
    private ISimVehicleService simVehicleService;

    @ApiOperation("创建协议")
    @PostMapping("/create")
	@SysLog(sysModule = SysModuleEnum.ADD_PROTOCOL)
    public Response createProtocolManage(@Validated @RequestBody ProtocolManageDTO dto){
        log.warn("协议管理创建协议入参{}", JSONObject.toJSONString(dto));
        protocolManageService.add(dto);
        return Response.createSuccess();
    }

    @ApiOperation("更新协议")
    @PostMapping("/update")
	@SysLog(sysModule = SysModuleEnum.UPDATE_PROTOCOL)
    public Response update(@Validated @RequestBody ProtocolManageUpdateDTO dto){
        log.warn("协议管理更新协议入参{}", JSONObject.toJSONString(dto));
        protocolManageService.update(dto);
        return Response.createSuccess();
    }

    @ApiOperation("查询协议")
    @PostMapping("/query")
    public Response<List<ProtocolManageDTO>> query(@Validated @RequestBody ProtocolManageUpdateDTO dto){
        log.warn("协议管理查询协议入参{}", JSONObject.toJSONString(dto));
        return protocolManageService.query(dto);
    }

    @GetMapping("queryOrgByList")
    @ApiOperation(value = "查询组织信息")
    public Response<List<OrganizationDTO>> queryOrgByList(OrganizationDTO dto) {
        OrganizationDTO organizationDTO = new OrganizationDTO();
        organizationDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        //只查询车企和车企子组织
        List<Integer> list = new ArrayList<>();
        list.add(OrganBizzTypeEnum.CAR_FACTORY.getCode());
        list.add(OrganBizzTypeEnum.SON_ORGAN.getCode());
        organizationDTO.setBizTypeList(list);
        return simVehicleService.queryOrgByList(organizationDTO);
    }

    @PostMapping("queryFileMould")
    @ApiOperation(value = "查询文件模板")
    public Response<Map> queryFileMould() {
        return protocolManageService.queryFileMould();
    }


    @ApiOperation("查询logo")
    @PostMapping("/noLogin")
    public Response<List<ProtocolManageDTO>> noLogin(){
        return protocolManageService.noLogin();
    }
}
