package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.common.FileTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.BindSimVehicleRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.DeleteByUuidDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.ImportSimDTO;
import com.cusc.nirvana.user.rnr.enterprise.excel.VehicleSimRelRow;
import com.cusc.nirvana.user.rnr.enterprise.excel.utils.EasyExcelUtil;
import com.cusc.nirvana.user.rnr.enterprise.service.ISimVehicleService;
import com.cusc.nirvana.user.rnr.enterprise.util.IccidUtil;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.client.SimVehicleRelClient;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.client.OrgSimRelClient;
import com.cusc.nirvana.user.rnr.mg.dto.OrgBusinessTagDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgBusinessTagQueryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelQueryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryQueryDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * @author xxx
 */
@RestController
@RequestMapping("/simvehicle/")
@ApiModel("账号管理")
@Validated
@Slf4j
public class SimVehicleRelController {

    @Autowired
    private ISimVehicleService simVehicleService;

    @Autowired
    private SimVehicleRelClient simVehicleRelClient;

    @Resource
	private FileSystemClient fileSystemClient;


    @Autowired
    private OrgSimRelClient orgSimRelClient;

    @Value("${template.carCard}")
    private String carCardTemplate;

    @Value("${template.carEnableOrDisableTemplate}")
    private String carEnableOrDisableTemplate;

    /**
     * 查询车厂组织列表
     * @return
     */
    @GetMapping("queryOrgByList")
    @ApiOperation(value = "查询是车厂的组织信息")
    public Response<List<OrganizationDTO>> queryOrgByList() {
        OrganizationDTO organizationDTO = new OrganizationDTO();
        organizationDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        organizationDTO.setBizType(1);
        return simVehicleService.queryOrgByList(organizationDTO);
    }

    /**
     * 导入SIM卡或车卡关系
     * @param importSimDTO
     * @return
     */
    @PostMapping("import")
    @ApiOperation(value = "导入SIM卡或车卡关系")
	@SysLog(sysModule = SysModuleEnum.SIM_VEHICLE)
    public Response<?> importSim(@RequestBody @Validated ImportSimDTO importSimDTO) {
        return simVehicleService.importSim(importSimDTO);
    }

    @GetMapping("/download/errorfile")
    @ApiOperation(value = "下载错误文件")
    @ApiImplicitParam(name = "uuid", value = "记录uuid", required = true)
    public void downloadErrorFile(@NotBlank(message = "记录uuid不能为空") String uuid,
                                  HttpServletResponse servletResponse) throws Exception {
        downloadExcelFile(uuid, true, servletResponse);
    }

    @GetMapping("/download/file")
    @ApiOperation(value = "下载上传文件")
    @ApiImplicitParam(name = "uuid", value = "记录uuid", required = true)
    public void downloadFile(@NotBlank(message = "记录uuid不能为空") String uuid,
                             HttpServletResponse servletResponse)  throws Exception {
        downloadExcelFile(uuid, false, servletResponse);
    }

    private void downloadExcelFile(String uuid, boolean isErrorFile, HttpServletResponse servletResponse) throws Exception {
        Response<SimFileHistoryDTO> response = simVehicleService.getImportHistory(uuid);
        if (response.isSuccess() && response.getData() != null) {
            FileDownloadDTO fileDownloadDTO = new FileDownloadDTO();

            String fileName = response.getData().getFileName();
            if (isErrorFile) {
                fileDownloadDTO.setUuid(response.getData().getErrorFileAddr());
                fileName = fileName.substring(0, fileName.lastIndexOf(".")) + "-错误信息" + fileName.substring(fileName.lastIndexOf("."));
            } else {
                fileDownloadDTO.setUuid(response.getData().getFileAddr());
            }
            fileDownloadDTO.setFileName(fileName);

            OutputStream outputStream = null;
            InputStream inputStream = null;
            // 以文件形式下载
            servletResponse.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode(StringUtils.isNotEmpty(fileDownloadDTO.getFileName()) ? fileDownloadDTO.getFileName() : "文件.xlsx", "utf-8"));
            servletResponse.setHeader("Access-Control-Expose-Headers","Content-Disposition");
            try {

				com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO dto = new com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO();
				dto.setUuid(fileDownloadDTO.getUuid());
				String base64 = fileSystemClient.getBase64(dto).getData().getBase64();

				byte[] bytes = Base64.getDecoder().decode(base64);

				inputStream = new ByteArrayInputStream(bytes);

                BufferedInputStream in = new BufferedInputStream(inputStream);
                outputStream = new BufferedOutputStream(servletResponse.getOutputStream());
                byte[] car = new byte[1024];
                int len;
                while ((len = in.read(car)) != -1) {
                    outputStream.write(car, 0, len);
                }
                outputStream.flush();
            }catch (Exception e) {
                log.error("下载文件失败", e);
            } finally {
                IOUtils.closeQuietly(inputStream);
                IOUtils.closeQuietly(outputStream);
            }
        } else {
            throw new Exception("没有找到文件");
        }
    }

    /**
     * SIM卡或者车卡关系模板下载
     * @param fileType
     * @param response
     */
    @GetMapping("/download/template")
    @ApiOperation(value = "车卡关系模板下载")
    @ApiImplicitParam(name = "fileType", value = "文件类型,1:车卡关系导入模板", required = true)
    public void downloadTemplate(@NotNull(message = "模板类型不能为空") Integer fileType, HttpServletResponse response) throws IOException {
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");

        String fileName = "";

        if(fileType == FileTypeEnum.SIMCARD.getCode()){
            fileName = "SIM卡导入模板.xlsx";
        }else if(fileType == FileTypeEnum.CARSTARTUP.getCode()){
            fileName = "卡启用模板.xlsx";
        }else if(fileType == FileTypeEnum.CARCLOSE.getCode()){
            fileName = "卡禁用模板.xlsx";
        }else {
            fileName = "车卡关系导入模板.xlsx";
        }

        com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO FileRecordDTO = new com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO();

        FileRecordDTO.setUuid(fileType == FileTypeEnum.CARSTARTUP.getCode()||fileType == FileTypeEnum.CARCLOSE.getCode()?carEnableOrDisableTemplate:carCardTemplate);

        Response<ImageBase64DTO> fileRecordDTOResponse = fileSystemClient.getBase64(FileRecordDTO);

        if(!fileRecordDTOResponse.isSuccess()){
            throw new CuscUserException("", fileType == 0 ? "没有查询到SIM卡导入模板.xlsx" : "没有查询到车卡关系导入模板.xlsx");
        }
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        byte[] bytes = Base64.getDecoder().decode(fileRecordDTOResponse.getData().getBase64());
        InputStream in = new ByteArrayInputStream(bytes);
        OutputStream outputStream = new BufferedOutputStream(response.getOutputStream());

        byte[] buff =new byte[1024];
        //所读取的内容使用n来接收
        int n;
        //当没有读取完时,继续读取,循环
        while((n=in.read(buff))!=-1){
            //将字节数组的数据全部写入到输出流中
            outputStream.write(buff,0,n);
        }
        //强制将缓存区的数据进行输出
        outputStream.flush();
        //关流
        outputStream.close();
        in.close();

//        String sheetName = "Sheet1";
//        EasyExcelUtil excelUtil = new EasyExcelUtil();
//        if (fileType == 1) {
//            excelUtil.init(response.getOutputStream(), sheetName, VehicleSimRelRow.class);
//            List<VehicleSimRelRow> list = new ArrayList<>();
//            VehicleSimRelRow row = new VehicleSimRelRow();
//            list.add(row);
//            excelUtil.addExcel(list);
//        }
//        excelUtil.finish();
//        response.getOutputStream().close();
    }

    @PostMapping("unbind")
    @ApiOperation(value = "车卡关系解绑")
    public Response<SimVehicleDTO> unbind(@RequestBody @Validated BindRequestDTO requestDTO) {
        return simVehicleService.unbind(requestDTO);
    }

    @PostMapping("bind")
    @ApiOperation(value = "车卡关系绑定")
    public Response<SimVehicleBindingResult> bind(@RequestBody @Validated BindSimVehicleRequestDTO requestDTO) {

        Response<OrgSimRelDTO> simrelResponse = orgSimRelClient.getByIccid(requestDTO.getIccid());
        SimVehicleBindingDTO simVehicleBindingDTO = new SimVehicleBindingDTO();
        SimVehicleDTO simVehicleDTO = new SimVehicleDTO();
        simVehicleDTO.setIccid(requestDTO.getIccid());
        simVehicleDTO.setVin(requestDTO.getVin());

        if (simrelResponse.isSuccess() && simrelResponse.getData() != null) {
            simVehicleDTO.setRealBySelf(simrelResponse.getData().getRealBySelf());
            simVehicleBindingDTO.setTagUuid(simrelResponse.getData().getTagUuid());
        }
        List<SimVehicleDTO> list = new ArrayList<>();
        list.add(simVehicleDTO);
        simVehicleBindingDTO.setBindingList(list);
        simVehicleBindingDTO.setOrganizationId(requestDTO.getOrganizationId());
        return simVehicleRelClient.importSimVehicle(simVehicleBindingDTO);
    }

    /**
     * 根据业务编码获取指定Sim卡信息
     * @param uuid
     * @return
     */
    @GetMapping("/getSimByUuid")
    @ApiOperation("根据业务编码获取指定Sim卡信息")
    public Response<IccidDetailDTO> getSimByUuid(@NotBlank(message = "业务编码不能为空") String uuid) {
        return simVehicleService.getSimByUuid(uuid);
    }

    /**
     * 分析查询SIM卡信息列表
     * @param queryDTO
     * @return
     */
    @PostMapping("/querySimByPage")
    @ApiOperation("分析查询SIM卡信息列表")
    public Response<PageResult<IccidDetailDTO>> querySimByPage(@RequestBody OrgSimRelQueryDTO queryDTO) {
        return simVehicleService.querySimByPage(queryDTO);
    }

    /**
     * 根据VIN码查询车卡绑定信息
     * @param orgUuid
     * @param vin
     * @return
     */
    @GetMapping("/querySimVehicleByVin")
    @ApiOperation("根据VIN码查询车卡绑定信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgUuid", value = "组织编码", required = true),
            @ApiImplicitParam(name = "vin", value = "VIN码", required = true)
    })
    public Response<List<VinIccidDTO>> querySimVehicleByVin(
            @NotBlank(message = "组织编码不能为空") String orgUuid,
            @NotBlank(message = "VIN不能为空") String vin) {

        return simVehicleService.querySimVehicleByVin(orgUuid, vin);
    }

    /**
     * 查询车卡绑定列表信息
     * @param queryDTO
     * @return
     */
    @PostMapping("/querySimVehicleByPage")
    @ApiOperation("查询车卡绑定列表信息")
    public Response<PageResult<SimVehicleResultDTO>> querySimVehiclePage(@RequestBody @Validated SimVehicleQueryDTO queryDTO) {
        log.warn("查询车卡绑定列表信息入参{}",JSONObject.toJSONString(queryDTO));
        OrganizationDTO organizationDTO = new OrganizationDTO();
        organizationDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        organizationDTO.setBizType(1);
        log.warn("queryOrgByList{}", JSONObject.toJSONString(organizationDTO));
        Response<List<OrganizationDTO>> organizations = simVehicleService.queryOrgByList(organizationDTO);
        log.warn("organizations{}", JSONObject.toJSONString(organizations));

        Map<String, OrganizationDTO> organzationMap = new HashMap<>();
        organizations.getData().forEach(organ -> {
            organzationMap.put(organ.getOrganCode(), organ);
        });
        log.warn("organzationMap{}", JSONObject.toJSONString(organzationMap));

        Response<PageResult<SimVehicleResultDTO>> response = simVehicleService.querySimVehiclePage(queryDTO);
        log.warn("response{}", JSONObject.toJSONString(response));

        if (!response.isSuccess()) {
            return response;
        }
        response.getData().getList().forEach(result -> {
            result.setOrganizationName(organzationMap.get(result.getOrgUuid()).getOrganName());
            result.setTelecomSupplier(IccidUtil.getTelcom(result.getIccid()));
        });
        return response;
    }

    /**
     * 删除历史导入信息
     * @param uuidDTO
     * @return
     */
    @PostMapping("/deleteImportHisotry")
    @ApiOperation("删除历史导入信息")
    public Response deleteImportHisotry(@RequestBody DeleteByUuidDTO uuidDTO) {
        return simVehicleService.deleteImportHisotry(uuidDTO);
    }

    /**
     * 查询指定导入历史信息
     * @param uuid
     * @return
     */
    @GetMapping("/getImportHistory")
    @ApiOperation("查询指定导入历史信息")
    @ApiImplicitParam(name = "uuid", value = "业务编码")
    public Response<SimFileHistoryDTO> getImportHistory(@NotBlank(message = "业务编码不能为空") String uuid) {
        return simVehicleService.getImportHistory(uuid);
    }

    /**
     * 分页查询导入历史信息
     * @param queryDTO
     * @return
     */
    @PostMapping("/queryImportHistory")
    @ApiOperation("分页查询导入历史信息")
    public Response<PageResult<SimFileHistoryDTO>> queryImportHistory(@RequestBody SimFileHistoryQueryDTO queryDTO) {
        return simVehicleService.queryImportHistory(queryDTO);
    }

    /**
     * 查询指定业务Tag数据
     * @param uuid
     * @return
     */
    @GetMapping("/getTagByUuid")
    @ApiOperation("查询指定业务Tag数据")
    @ApiImplicitParam(name = "uuid", value = "业务编码", required = true)
    public Response<OrgBusinessTagDTO> getTagByUuid(@NotBlank(message = "业务编码不能为空") String uuid) {
        return simVehicleService.getTagByUuid(uuid);
    }

    /**
     * 查询标签信息列表
     * @param queryDto
     * @return
     */
    @PostMapping("/queryTags")
    @ApiOperation("查询标签信息列表")
    public Response<List<OrgBusinessTagDTO>> queryTags(@RequestBody @Validated OrgBusinessTagQueryDTO queryDto) {
        queryDto.setTenantNo(CuscStringUtils.isEmpty(queryDto.getTenantNo()) ? UserSubjectUtil.getTenantNo() : queryDto.getTenantNo());
        return simVehicleService.queryTags(queryDto);
    }
}
