package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.dto.T1CommonResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.T1CompletionReqDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.T1InstructionService;
import com.cusc.nirvana.user.rnr.enterprise.util.IpAddrUtil;
import com.cusc.nirvana.user.rnr.enterprise.util.SpringValidationUtil;
import com.cusc.nirvana.user.rnr.fp.dto.InstructionDTO;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * @className: T1InstructionController
 * @description: T1
 * @author: jk
 * @date: 2022/7/26 16:55
 * @version: 1.0
 **/
@RestController
@RequestMapping("/t1/ct")
@Slf4j
public class T1InstructionCtccController {

    @Autowired
    private T1InstructionService t1InstructionService;

    @Value("${auth.allowVisitCt:}")
    private String allowVisitCt;

    @PostMapping("/instructions")
    @ApiOperation(value = "指令下发", notes = "指令下发")
    public Response<InstructionDTO> instruction(@RequestBody InstructionDTO instructionDTO) {
        ServletRequestAttributes sra = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = sra.getRequest();
        String ip= IpAddrUtil.getIpAddr(request);
        if(allowVisitCt.indexOf(ip)==-1){
            Response.createError("不允许访问",instructionDTO.getRequestID());
        }
        return t1InstructionService.instructionsIssue(instructionDTO);
    }

    @PostMapping("/callbackResult")
    public T1CommonResponseDTO callbackResult(@RequestBody @Validated T1CompletionReqDTO t1CompletionReqDTO) {
        ServletRequestAttributes sra = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = sra.getRequest();
        String ip= IpAddrUtil.getIpAddr(request);
        if(allowVisitCt.indexOf(ip)==-1){
            T1CommonResponseDTO.createError("不允许访问",t1CompletionReqDTO.getRequestID());
        }
        return t1InstructionService.callbackResult(t1CompletionReqDTO);
    }
}
