package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.dto.UnBindIccidDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VinDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.validator.groups.VehicleCardVerify;
import com.cusc.nirvana.user.rnr.enterprise.service.IVehicleCardService;
import com.cusc.nirvana.user.rnr.fp.dto.IccidListDTO;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardDTO;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardInfoDTO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * Description: 车卡关系控制层
 * <br />
 * CreateDate 2022-04-16 15:16:55
 *
 * @author yuyi
 **/
@RestController
@RequestMapping("/vehicleCard")
public class VehicleCardController {

    @Autowired
    private IVehicleCardService vehicleCardService;

    /**
     *  新车实名查询iccid，如果已经有实名信息，提醒走一车多卡
     * 通过VIN查询ICCID集合
     */
    @ApiOperation(value = "新车实名查询iccid", notes = "新车实名查询iccid")
    @PostMapping("/getNeCarUnBindIccidList")
    public Response<IccidListDTO> getNeCarUnBindIccidList(@RequestBody VinDTO vinDTO) {
        return vehicleCardService.getNewCarUnBindIccidList(vinDTO.getVin());
    }

    /**
     * 通过VIN查询ICCID集合
     *
     */
    @ApiOperation(value = "通过VIN查询ICCID集合", notes = "通过VIN查询ICCID集合")
    @PostMapping("/getIccidListByVin")
    public Response<IccidListDTO> getIccidListByVin(@RequestBody VinDTO vinDTO) {
        return vehicleCardService.getIccidListByVin(vinDTO.getVin());
    }

    /**
     * 验证车卡信息
     */
    @ApiOperation(value = "验证车卡信息", notes = "验证车卡信息")
    @PostMapping("/verifyVinCard")
    public Response verifyVinCard(@Validated({VehicleCardVerify.class}) @RequestBody VinCardInfoDTO vinCardListDTO) {
        return vehicleCardService.verifyVinCard(vinCardListDTO);
    }

    @ApiOperation(value = "批量验证车卡信息", notes = "批量验证车卡信息")
    @PostMapping("/verifyVinCardBatch")
    public Response verifyVinCardBatch(@Validated({VehicleCardVerify.class}) @RequestBody List<VinCardDTO> verifyDTOs) {
        return vehicleCardService.verifyVinCardBatch(verifyDTOs);
    }

    /**
     * Description: 通过VIN查询已实名信息
     * <br />
     * CreateDate 2022-04-20 16:43:36
     *
     * @author yuyi
     **/
    @ApiOperation(value = "通过VIN查询已实名信息", notes = "通过VIN查询已实名信息")
    @PostMapping("/getRnrInfoByVin")
    public Response<VehicleCardRnrDTO> getRnrInfoByVin(@Valid @RequestBody VinDTO vinDTO) {
        return vehicleCardService.getBindIccidList(vinDTO);
    }

    /**
     * 通过VIN查询未绑定的ICCID集合
     */
    @ApiOperation(value = "通过VIN查询未绑定的ICCID集合并返回实名信息", notes = "通过VIN查询未绑定的ICCID集合并返回实名信息")
    @PostMapping("/getUnBindIccidList")
    public Response<UnBindIccidDTO> getUnBindIccidList(@Valid @RequestBody VinDTO vinDTO) {
        return vehicleCardService.getUnBindIccidList(vinDTO);
    }
}
