package com.cusc.nirvana.user.rnr.enterprise.dto;

import com.cusc.nirvana.user.rnr.enterprise.common.DesensitizationFieId;
import com.cusc.nirvana.user.rnr.enterprise.dto.validator.groups.EnterpriseCorporationChangeGroup;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import java.io.*;
import java.util.List;

/**
 * Description: 企业责任人变更DTO
 * <br />
 * CreateDate 2022-04-20 16:41:45
 *
 * @author yuyi
 **/
@Data
public class EnterpriseCorporationChangeDTO implements Serializable {
    private static final long serialVersionUID = -5712879585098743410L;

	@ApiModelProperty(value = "活体唇语的requestId")
	@NotBlank(message = "活体唇语的requestId不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private String requestId;

    @ApiModelProperty(value = "企业名称")
    @NotBlank(message = "企业名称不能为空",groups = {EnterpriseCorporationChangeGroup.Query.class,
            EnterpriseCorporationChangeGroup.Submit.class})
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_NONE,sort = 0)
    private String companyName;


	@ApiModelProperty(value = "原责任人姓名")
	@NotBlank(message = "原责任人姓名不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_NAME,sort = 1)
    private String origCorporationName;

    @ApiModelProperty(value = "责任人姓名")
    @NotBlank(message = "责任人姓名不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_NAME,sort = 2)
    private String corporationName;

    @ApiModelProperty(value = "性别")
    @NotBlank(message = "性别不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private String corporationGender;

    @ApiModelProperty(value = "责任人证件类型")
    @NotBlank(message = "责任人证件类型不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private String corporationCertType;

    @ApiModelProperty(value = "责任人证件号")
    @NotBlank(message = "责任人证件号不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private String corporationCertNumber;

    @ApiModelProperty(value = "责任人证件地址")
    @NotBlank(message = "责任人证件地址不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private String corporationCertAddress;

    @ApiModelProperty(value = "责任人证件图片集合，身份证第一张必须是人像面")
    @NotEmpty(message = "责任人证件图片不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private List<String> corporationCertPic;

	@ApiModelProperty(value = "责任人证件有效期开始时间")
	@NotBlank(message = "责任人证件有效期开始时间不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
	private String corporationCertEffectiveDate;

    @ApiModelProperty(value = "责任人证件有效期")
    @NotBlank(message = "责任人证件有效期不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private String corporationCertExpirationDate;

    @ApiModelProperty(value = "责任人通讯地址")
    private String corporationContactAddress;

    @ApiModelProperty(value = "责任人手机号码")
    @NotBlank(message = "责任人手机号码不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_PHONE,sort = 3)
    private String corporationPhone;

    @ApiModelProperty(value = "短信验证码")
    @NotBlank(message = "短信验证码不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private String captcha;

    @ApiModelProperty(value = "企业实名认证授权书")
    @NotEmpty(message = "企业实名认证授权书不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private List<String> authorizationLetterPic;

    @ApiModelProperty(value = "责任人活体视频")
    @NotBlank(message = "责任人活体视频不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
    private String corporationVideo;
}
