package com.cusc.nirvana.user.rnr.enterprise.dto;

import com.cusc.nirvana.user.rnr.enterprise.common.DesensitizationFieId;
import com.cusc.nirvana.user.rnr.enterprise.dto.validator.groups.EnterpriseCorporationChangeGroup;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.*;
import java.util.List;

@Data
public class EnterpriseRnrCertificationInfoDTO implements Serializable {
    private static final long serialVersionUID = 8325801116331488860L;
    @ApiModelProperty("请求ID")
    private String requestId;
    @ApiModelProperty("文件系统ID")
    @NotEmpty(
            message = "文件系统ID不能为空"
    )
    private String fileId;
    @ApiModelProperty("企业名称")
    @NotEmpty(
            message = "企业名称不能为空"
    )
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_NONE,sort = 0)
    private String companyName;
    @ApiModelProperty("企业性质")
    private String companyType;
    @ApiModelProperty("行业类型")
    private String industryType;
    @ApiModelProperty("企业证件类型")
	@NotBlank(message = "企业证件类型不能为空")
    private String companyCertType;
    @ApiModelProperty("企业证件号码")
    @NotEmpty(
            message = "企业证件号码不能为空"
    )
    private String companyCertNumber;
    @ApiModelProperty("企业证件照片")
    @NotNull(
            message = "企业证件照片不能为空"
    )
    private List<String> licenseImages;
    @ApiModelProperty("企业证件地址")
    @NotEmpty(
            message = "企业证件地址不能为空"
    )
    private String companyCertAddress;
    @ApiModelProperty("企业通讯地址")
    @NotEmpty(
            message = "企业通讯地址不能为空"
    )
    private String companyContactAddress;
    @ApiModelProperty("责任人姓名")
    @NotEmpty(
            message = "责任人姓名不能为空"
    )
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_NAME,sort = 1)
    private String corporationName;
    @ApiModelProperty("性别")
    @NotEmpty(
            message = "性别不能为空"
    )
    private String corporationGender;
    @ApiModelProperty("责任人证件类型")
    private String corporationCertType;
    @ApiModelProperty("责任人证件号")
    @NotEmpty(
            message = "责任人证件号不能为空"
    )
    private String corporationCertNumber;
    @ApiModelProperty("责任人证件地址")
    @NotEmpty(
            message = "责任人证件地址不能为空"
    )
    private String corporationCertAddress;


	@ApiModelProperty(value = "责任人证件有效期开始时间")
	@NotBlank(message = "责任人证件有效期开始时间不能为空",groups = {EnterpriseCorporationChangeGroup.Submit.class})
	private String corporationCertEffectiveDate;

    @ApiModelProperty("责任人证件有效期")
    @NotEmpty(
            message = "责任人证件有效期不能为空"
    )
    private String corporationCertExpirationDate;


    @ApiModelProperty("责任人证件照片")
    @NotNull(
            message = "责任人证件照片不能为空"
    )
    private List<String> corporationPhotoPic;
    @ApiModelProperty("责任人通讯地址")
    private String corporationContactAddress;
    @ApiModelProperty("责任人手机号码")
    @NotEmpty(
            message = "责任人手机号码不能为空"
    )
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_PHONE,sort = 2)
    private String corporationPhone;
    @ApiModelProperty("企业实名认证授权书")
    @NotNull(
            message = "企业实名认证授权书不能为空"
    )
    private List<String> authorizationLetterPic;
    @ApiModelProperty("活体视频")
    @NotEmpty(
            message = "活体视频不能为空"
    )
    private String liveVerificationVideo;

    @ApiModelProperty("二手车标志")
    private Integer customerType;

    @ApiModelProperty(value = "短信验证码")
    private String verificationCode;

    @ApiModelProperty("车企信息id")
    private String vehicleEnterpriseId;

    @ApiModelProperty("是否是车企 0 否，默认值  1 是")
    private Integer isVehicleCompany;

	@ApiModelProperty(value = "责任告知书")
	private List<String> dutyPic;

	@ApiModelProperty(value = "入网合同")
    //按于总要求先去掉校验
	//@NotEmpty(message = "入网合同不能为空")
	private List<String> contractPic;
}
