package com.cusc.nirvana.user.rnr.enterprise.dto;

import com.cusc.nirvana.user.eiam.constants.OrganTypeEnum;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.eiam.dto.UserDTO;
import com.cusc.nirvana.user.rnr.enterprise.common.DesensitizationFieId;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * Description: B端组织管理
 * <br />
 * CreateDate 2022-05-19 16:24:17
 *
 * @author yuyi
 **/
@ApiModel(value = "B端组织管理", description = "B端组织管理")
@Data
public class OrganDTO {

	@ApiModelProperty(value = "组织uuid")
	private String uuid;

	@ApiModelProperty(value = "客户租户号")
	private String tenantNo;

	@ApiModelProperty(value = "组织编码")
	private String organCode;

	@ApiModelProperty(value = "组织名称")
	@NotNull(message = "组织名称不可为空")
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_NONE,sort = 0)
	private String organName;

	@ApiModelProperty(value = "业务类型 1 车企 2 经销商 3、车企子组织")
	@NotNull(message = "组织类型不可为空")
	private Integer bizType;

	@ApiModelProperty(value = "组织描述")
	private String comment;

	@ApiModelProperty(value = "上级组织")
	private String parentId;
	@ApiModelProperty(value = "上级组织名称")
	private String parentOrganName;

	@ApiModelProperty(value = "状态： 0 禁用 1启用")
	private Integer status;

	@ApiModelProperty("管理员Uuid")
	private String adminUserId;

	@ApiModelProperty("管理员姓名")
	@NotNull(message = "管理员姓名不可为空")
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_NAME,sort = 1)
	private String organAdminName;

	@ApiModelProperty("管理员账号")
	@NotNull(message = "管理员账号不可为空")
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_NONE,sort = 2)
	private String organAdminAccount;

	@ApiModelProperty("手机号")
	@NotNull(message = "手机号不可为空")
	@DesensitizationFieId(desensitizationRule = DesensitizationUtil.DESENSITIZATION_PHONE,sort = 3)
	private String organAdminPhone;

	@ApiModelProperty(value = "管理员密码")
	@NotNull(message = "管理员密码不能为空")
	private String password;

	@ApiModelProperty(value = "用户id")
	private String userId;

	@ApiModelProperty("车企组织管理员角色")
	private String roleCode;

	@ApiModelProperty("创建时间")
	private Date createTime;

	public UserDTO toUserDTO() {
		UserDTO userDto = new UserDTO();
		userDto.setUserName(this.getOrganAdminAccount());
		userDto.setFullName(this.getOrganAdminName());
		userDto.setPassword(this.getPassword());
		userDto.setPhone(this.getOrganAdminPhone());
		return userDto;
	}


	public OrganizationDTO toOrganDTO() {
		OrganizationDTO organizationDTO = new OrganizationDTO();
		organizationDTO.setParentId(this.getParentId());
		organizationDTO.setOrganName(this.getOrganName());
		organizationDTO.setOrganType(OrganTypeEnum.COMPANY.getCode());
		organizationDTO.setComment(this.getComment());
		organizationDTO.setTenantNo(this.getTenantNo());
		organizationDTO.setBizType(this.getBizType());
		return organizationDTO;
	}
}
