package com.cusc.nirvana.user.rnr.enterprise.excel;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.exception.ExcelAnalysisStopException;
import com.cusc.nirvana.user.rnr.enterprise.constants.FuelTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.PlaceOfOriginOfVehicleEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.VehicleChannelTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.VehicleTypeEnum;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import org.apache.commons.lang3.StringUtils;

import java.text.SimpleDateFormat;
import java.util.Map;

public class CarInfoRowCountListener extends AnalysisEventListener<CarInfoRow> {

    private SimFileHistoryDTO simFileHistoryDTO;

    private int rowIndex = 2;

    public CarInfoRowCountListener(SimFileHistoryDTO simFileHistoryDTO) {
        this.simFileHistoryDTO = simFileHistoryDTO;
    }

    @Override
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
        super.invokeHeadMap(headMap, context);
        if (headMap.size() < 20) {
            simFileHistoryDTO.setErrorInfo("文件标题不符合模板要求，文件标题：[" + headMap.values() + "]");
            throw new ExcelAnalysisStopException("文件不符合模板要求。文件标题：[" + headMap.values() + "]");
        }
    }

    @Override
    public void invoke(CarInfoRow data, AnalysisContext context) {
//        String failReason = validata(data);
        simFileHistoryDTO.setTotalCount(simFileHistoryDTO.getTotalCount() + 1);
//        if (CuscStringUtils.isNotEmpty(failReason)) {
//            throw new ExcelAnalysisStopException("导入文件在["+rowIndex+"]行校验失败，错误原因["+failReason+"]，请核实!");
//        }
//        rowIndex++;
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {

    }

    /**
     * 验证iccid和vin是否正确
     * @param row
     * @return
     */
    private String validata(CarInfoRow row) {
        String failReason = "";

        if (StringUtils.isBlank(row.getNameVehicleEnterprise())) {
            failReason = "道路机动车辆生产企业名称为空";
        }
        if (StringUtils.isBlank(row.getVin())) {
            failReason = "VIN为空";
        }
        if (StringUtils.isBlank(row.getVehicleType())) {
            failReason = "车辆类型为空";
        }else {
            if(null == VehicleTypeEnum.getEnumByCode(row.getVehicleType())){
                failReason = "车辆类型与法规不一致";
            }
        }

        if (StringUtils.isBlank(row.getPlaceOfOriginOfVehicle())) {
            failReason = "车辆产地为空";
        }else {
            if(null == PlaceOfOriginOfVehicleEnum.getEnumByCode(row.getPlaceOfOriginOfVehicle())){
                failReason = "车辆产地与法规不一致";
            }
        }

        if (StringUtils.isBlank(row.getVehicleDepartment())) {
            failReason = "车辆品牌为空";
        }

        if (StringUtils.isBlank(row.getVehicleName())) {
            failReason = "车辆名称为空";
        }

        if (StringUtils.isBlank(row.getVehicleNum())) {
            failReason = "车辆型号为空";
        }

        if (StringUtils.isBlank(row.getVehicleModel())) {
            failReason = "车型通用名称为空";
        }

        if (StringUtils.isBlank(row.getBodyColor())) {
            failReason = "车身颜色为空";
        }

        if (StringUtils.isBlank(row.getFuelType())) {
            failReason = "燃料种类为空";
        }else {
            if(null == FuelTypeEnum.getEnumByCode(row.getFuelType())){
                failReason = "燃料种类与法规不一致";
            }
        }

//        if (StringUtils.isBlank(row.getEngineNum())) {
//            failReason = "发动机号码为空";
//        }
//
//        if (StringUtils.isBlank(row.getMotorNum())) {
//            failReason = "电动机序列号为空";
//        }

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String vehicleDepartureTime = simpleDateFormat.format(row.getVehicleDepartureTime());
        if (StringUtils.isBlank(vehicleDepartureTime)) {
            failReason = "车辆出厂时间为空";
        }
        String vehicleSalesTime=simpleDateFormat.format(row.getVehicleSalesTime());;
        if (StringUtils.isBlank(vehicleSalesTime)) {
            failReason = "车辆销售时间为空";
        }

//        if (StringUtils.isBlank(row.getVehicleSalesUpdateTime())) {
//            failReason = "车辆销售信息变更时间为空";
//        }
//
//        if (StringUtils.isBlank(row.getLicensePlateNumber())) {
//            failReason = "车牌号码为空";
//        }

        if (StringUtils.isBlank(row.getVehicleChannelName())) {
            failReason = "车辆销售渠道名称为空";
        }

        if (StringUtils.isBlank(row.getVehicleChannelType())) {
            failReason = "车辆销售渠道类型为空";
        }else {
            if(null == VehicleChannelTypeEnum.getEnumByCode(row.getVehicleChannelType())){
                failReason = "车辆销售渠道类型不符合法规要求";
            }
        }

        if (StringUtils.isBlank(row.getVehicleStaffName())) {
            failReason = "车辆销售渠道员工姓名为空";
        }

        if (StringUtils.isBlank(row.getVehicleLogginAddress())) {
            failReason = "车辆销售渠道地址为空";
        }
        return failReason;
    }
}
