package com.cusc.nirvana.user.rnr.enterprise.excel;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.exception.ExcelAnalysisStopException;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.excel.utils.EasyExcelUtil;
import com.cusc.nirvana.user.rnr.enterprise.util.IccidUtil;
import com.cusc.nirvana.user.rnr.fp.client.SimVehicleRelClient;
import com.cusc.nirvana.user.rnr.fp.dto.SimVehicleBindingDTO;
import com.cusc.nirvana.user.rnr.fp.dto.SimVehicleBindingResult;
import com.cusc.nirvana.user.rnr.fp.dto.SimVehicleDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 车卡关系导入excel处理监听器
 */
@Slf4j
public class VehicleSimListener extends AnalysisEventListener<VehicleSimRelRow> {

    private final static Logger LOGGER = LoggerFactory.getLogger(VehicleSimListener.class);

    /**每隔500条提交数据处理*/
    private static final int BATCH_COUNT = 500;

    private SimVehicleRelClient simVehicleClient;

    private SimFileHistoryDTO simFileHistoryDTO;

    private String errorExcelFile;

    private EasyExcelUtil easyExcelUtil;

    private String tagUuid;

    /**存放批数据*/
    private List<VehicleSimRelInfoRow> successlist = new ArrayList<>();


    private List<VehicleSimRelInfoRow> errorList = new ArrayList<>();

    public VehicleSimListener(SimVehicleRelClient simVehicleClient,
                              SimFileHistoryDTO simFileHistoryDTO, String errorExcelFile, String tagUuid) {
        this.simVehicleClient = simVehicleClient;
        this.simFileHistoryDTO = simFileHistoryDTO;
        this.errorExcelFile = errorExcelFile;
        this.tagUuid = tagUuid;
    }

    @Override
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
        super.invokeHeadMap(headMap, context);
        if (headMap.size() < 12) {
            simFileHistoryDTO.setErrorInfo("文件标题不符合模板要求，文件标题：[" + headMap.values() + "]");
            throw new ExcelAnalysisStopException("文件不符合模板要求。文件标题：[" + headMap.values() + "]");
        }
    }

    @Override
    public void invoke(VehicleSimRelRow vehicleSimRelRow, AnalysisContext analysisContext) {
        String failReason = validata(vehicleSimRelRow);
        VehicleSimRelInfoRow infoRow = rowToInfoRow(vehicleSimRelRow);
        if (CuscStringUtils.isNotEmpty(failReason)) {
            infoRow.setErrorInfo(failReason);
			errorList.add(infoRow);
        } else {
        	successlist.add(infoRow);
		}

    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
    	if (!CollectionUtils.isEmpty(successlist)) {
			processData();
		}

        if (!CollectionUtils.isEmpty(errorList)) {
			simFileHistoryDTO.setErrorCount(errorList.size());
			simFileHistoryDTO.setErrorInfo("导入车卡关系有问题，请下载文件查看详情");
        	processErrorData();
		}
        //excelFinish();
    }

	private void processErrorData() {
		EasyExcel.write(errorExcelFile, VehicleSimRelInfoRow.class)
				.sheet("sheet0")
				.doWrite(errorList);
	}

	@Override
    public void onException(Exception exception, AnalysisContext context) throws Exception {
        log.error("e : {}",exception);
        simFileHistoryDTO.setErrorInfo("导入车卡关系有问题，请下载文件查看详情");
        excelFinish();
        throw new ExcelAnalysisStopException("导入车卡关系有问题，请下载文件查看详情");
    }

    private void processData() {
        SimVehicleBindingDTO bindingDTO = new SimVehicleBindingDTO();
        bindingDTO.setOrganizationId(simFileHistoryDTO.getOrgUuid());
        bindingDTO.setTagUuid(tagUuid);
        bindingDTO.setCreator(simFileHistoryDTO.getCreator());
        List<SimVehicleDTO> simVehicleDTOS = new ArrayList<>();
        successlist.forEach(infoRow -> {
            SimVehicleDTO dto = infoRowToDto(infoRow);
            simVehicleDTOS.add(dto);

        });

        bindingDTO.setBindingList(simVehicleDTOS);
        Response<SimVehicleBindingResult> response;
        try {
            response = simVehicleClient.importSimVehicle(bindingDTO);
        }catch (Exception e) {
            LOGGER.error("调用FP导入车卡关系失败，原因:", e);
            response = Response.createError();
        }
        if (!response.isSuccess()) {
            simVehicleDTOS.forEach(simVehicleDTO -> {
                VehicleSimRelInfoRow infoRow = dtoToInfoRow(simVehicleDTO);
                infoRow.setErrorInfo("导入失败");
                //writeErrorExcel(infoRow);
				errorList.add(infoRow);
            });
            return;
        }
        SimVehicleBindingResult result = response.getData();
        if (result != null && result.getFailCount() != null) {
            result.getFailData().forEach(failSim -> {
                VehicleSimRelInfoRow infoRow = dtoToInfoRow(failSim);
                infoRow.setErrorInfo(CuscStringUtils.isEmpty(infoRow.getErrorInfo()) ? "导入失败" :infoRow.getErrorInfo());
                //writeErrorExcel(infoRow);
				errorList.add(infoRow);
            });
        }


    }

    private VehicleSimRelInfoRow rowToInfoRow(VehicleSimRelRow row) {
        VehicleSimRelInfoRow vehicleSimRelInfoRow = new VehicleSimRelInfoRow();
        BeanUtils.copyProperties(row, vehicleSimRelInfoRow);
        return vehicleSimRelInfoRow;
    }

    private SimVehicleDTO infoRowToDto(VehicleSimRelInfoRow infoRow) {
        SimVehicleDTO simVehicleDTO = new SimVehicleDTO();
        BeanUtils.copyProperties(infoRow, simVehicleDTO);
        simVehicleDTO.setRealBySelf((infoRow.getRealBySelf() != null && infoRow.getRealBySelf() == 1) ? true : false);
        return simVehicleDTO;
    }

    private VehicleSimRelInfoRow dtoToInfoRow(SimVehicleDTO dto) {
        VehicleSimRelInfoRow infoRow = new VehicleSimRelInfoRow();
        BeanUtils.copyProperties(dto, infoRow);
        infoRow.setRealBySelf((dto.getRealBySelf() != null && dto.getRealBySelf()) ? 1 : 0);
        infoRow.setErrorInfo(dto.getFailReason());
        return infoRow;
    }

    /**
     * 生成错误文件信息
     * @param infoRow
     */
    private void writeErrorExcel(VehicleSimRelInfoRow infoRow) {
        simFileHistoryDTO.setErrorCount(simFileHistoryDTO.getErrorCount() + 1);
        if (easyExcelUtil == null) {
            easyExcelUtil = new EasyExcelUtil();
            easyExcelUtil.init(errorExcelFile, "Sheet1", VehicleSimRelInfoRow.class);
        }
        List<VehicleSimRelInfoRow> infoRows = new ArrayList<>();
        infoRows.add(infoRow);
        easyExcelUtil.addExcel(infoRows);
    }

    /**
     * 验证iccid和vin是否正确
     * @param row
     * @return
     */
    private String validata(VehicleSimRelRow row) {
        if (StringUtils.isBlank(row.getIccid()) || !IccidUtil.check(row.getIccid())) {
            return "ICCID为空或格式不正确";
        }
        return "";
    }

    private void excelFinish() {
        if (easyExcelUtil != null) {
            easyExcelUtil.finish();
        }
    }
}
