package com.cusc.nirvana.user.rnr.enterprise.excel;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.exception.ExcelAnalysisStopException;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.common.FileTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.excel.utils.EasyExcelUtil;
import com.cusc.nirvana.user.rnr.enterprise.util.IccidUtil;
import com.cusc.nirvana.user.rnr.fp.client.SimVehicleRelClient;
import com.cusc.nirvana.user.rnr.fp.dto.SimVehicleBindingResult;
import com.cusc.nirvana.user.rnr.fp.dto.SimVehicleDTO;
import com.cusc.nirvana.user.rnr.mg.client.OrgSimRelClient;
import com.cusc.nirvana.user.rnr.mg.dto.IccIdListRequestDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 车卡关系导入excel处理监听器
 */
@Slf4j
public class VehicleStatusListener extends AnalysisEventListener<VehicleStatusRelRow> {

    private final static Logger LOGGER = LoggerFactory.getLogger(VehicleStatusListener.class);

    /**每隔500条提交数据处理*/
    private static final int BATCH_COUNT = 500;

    private SimVehicleRelClient simVehicleClient;

    private SimFileHistoryDTO simFileHistoryDTO;

    private String errorExcelFile;

    private EasyExcelUtil easyExcelUtil;

    private String tagUuid;

    private OrgSimRelClient orgSimRelClient;

    /**存放批数据*/
    private List<VehicleStatusRelInfoRow> successlist = new ArrayList<>();


    private List<VehicleStatusRelInfoRow> errorList = new ArrayList<>();

    public VehicleStatusListener(SimVehicleRelClient simVehicleClient,
                                 SimFileHistoryDTO simFileHistoryDTO, String errorExcelFile, String tagUuid, OrgSimRelClient orgSimRelClient) {
        this.simVehicleClient = simVehicleClient;
        this.simFileHistoryDTO = simFileHistoryDTO;
        this.errorExcelFile = errorExcelFile;
        this.tagUuid = tagUuid;
        this.orgSimRelClient = orgSimRelClient;
    }

    @Override
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
        super.invokeHeadMap(headMap, context);
        if (headMap.size() > 1) {
            simFileHistoryDTO.setErrorInfo("文件标题不符合模板要求，文件标题：[" + headMap.values() + "]");
            throw new ExcelAnalysisStopException("文件不符合模板要求。文件标题：[" + headMap.values() + "]");
        }
    }

    @Override
    public void invoke(VehicleStatusRelRow vehicleSimRelRow, AnalysisContext analysisContext) {
        String failReason = validata(vehicleSimRelRow);
        VehicleStatusRelInfoRow infoRow = rowToInfoRow(vehicleSimRelRow);
        if (CuscStringUtils.isNotEmpty(failReason)) {
            infoRow.setErrorInfo(failReason);
			errorList.add(infoRow);
        } else {
        	successlist.add(infoRow);
		}

        if (successlist.size() >= BATCH_COUNT) {
            processData();
            successlist.clear();
        }

    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
    	if (!CollectionUtils.isEmpty(successlist)) {
			processData();
		}
        if (!CollectionUtils.isEmpty(errorList)) {
			simFileHistoryDTO.setErrorCount(errorList.size());
			simFileHistoryDTO.setErrorInfo("导入车卡关系有问题，请下载文件查看详情");
        	processErrorData();
		}
        //excelFinish();
    }

	private void processErrorData() {
		EasyExcel.write(errorExcelFile, VehicleStatusRelInfoRow.class)
				.sheet("sheet0")
				.doWrite(errorList);
	}

	@Override
    public void onException(Exception exception, AnalysisContext context) throws Exception {
        log.error("e : {}",exception);
        simFileHistoryDTO.setErrorInfo("导入车卡关系有问题，请下载文件查看详情");
        excelFinish();
        throw new ExcelAnalysisStopException("导入车卡关系有问题，请下载文件查看详情");
    }

    private void processData() {

        //启动
        IccIdListRequestDTO iccIdListStartOrDisable = new IccIdListRequestDTO();



        List<String> iccidList = new ArrayList<>();

        successlist.forEach(infoRow -> {
            iccidList.add(infoRow.getIccid());
        });

        iccIdListStartOrDisable.setIccids(iccidList);
        iccIdListStartOrDisable.setRnrStatus(simFileHistoryDTO.getFileType()== FileTypeEnum.CARCLOSE.getCode()?0:1);

        //查询是否包含实名数据
        Response<List<OrgSimRelDTO>> listResponse = orgSimRelClient.queryInfoByIccidAndStatus(iccIdListStartOrDisable);

        if(!listResponse.isSuccess()||CollectionUtils.isEmpty(listResponse.getData())){
            iccIdListStartOrDisable.getIccids().forEach(iccid -> {
                VehicleStatusRelInfoRow infoRow = dtoToInfoRowExistIccid(iccid);
                infoRow.setErrorInfo("iccid不正确，或没有实名数据");
                errorList.add(infoRow);
            });
            return;
        }
        //查出存在的iccid
        List<String> existenceList = listResponse.getData().stream().map(a->a.getIccid()).collect(Collectors.toList());

        //过滤出iccid
        List<String> nothingness = iccidList.stream().filter(a->existenceList.indexOf(a)==-1).collect(Collectors.toList());

        if(!CollectionUtils.isEmpty(nothingness)){
            nothingness.forEach(iccid -> {
                VehicleStatusRelInfoRow infoRow = dtoToInfoRowExistIccid(iccid);
                infoRow.setErrorInfo("iccid不正确，或没有实名数据");
                errorList.add(infoRow);
            });
        }

        iccIdListStartOrDisable.setIccids(existenceList);

        //如果是解绑的，需要查询卡是否有存在实名的
        if(simFileHistoryDTO.getFileType()== FileTypeEnum.CARCLOSE.getCode()) {
            //解绑状态为1
            iccIdListStartOrDisable.setRnrStatus(1);
        }else {
            iccIdListStartOrDisable.setRnrStatus(0);
        }

        Response<SimVehicleBindingResult> response = orgSimRelClient.updateStatusByIccid(iccIdListStartOrDisable);

        if (!response.isSuccess()) {
            existenceList.forEach(iccid -> {
                VehicleStatusRelInfoRow infoRow = dtoToInfoRowExistIccid(iccid);
                infoRow.setErrorInfo("更新失败");
				errorList.add(infoRow);
            });
            return;
        }

    }

    private VehicleStatusRelInfoRow rowToInfoRow(VehicleStatusRelRow row) {
        VehicleStatusRelInfoRow vehicleSimRelInfoRow = new VehicleStatusRelInfoRow();
        BeanUtils.copyProperties(row, vehicleSimRelInfoRow);
        return vehicleSimRelInfoRow;
    }

    private VehicleStatusRelInfoRow dtoToInfoRowExistIccid(String iccid) {
        VehicleStatusRelInfoRow infoRow = new VehicleStatusRelInfoRow();
        infoRow.setIccid(iccid);
        return infoRow;
    }

    private VehicleStatusRelInfoRow dtoToInfoRow(VehicleStatusRelInfoRow message) {
        VehicleStatusRelInfoRow infoRow = new VehicleStatusRelInfoRow();
        infoRow.setIccid(message.getIccid());
        return infoRow;
    }


    /**
     * 验证iccid和vin是否正确
     * @param row
     * @return
     */
    private String validata(VehicleStatusRelRow row) {
        if (StringUtils.isBlank(row.getIccid()) || !IccidUtil.check(row.getIccid())) {
            return "ICCID为空或格式不正确";
        }
        return "";
    }

    private void excelFinish() {
        if (easyExcelUtil != null) {
            easyExcelUtil.finish();
        }
    }
}
