package com.cusc.nirvana.user.rnr.enterprise.excel.utils;

import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * EasyExcel操作工具类(写，追加)
 */
public class EasyExcelUtil {

    private final static Logger LOGGER = LoggerFactory.getLogger(EasyExcelUtil.class);

    private ExcelWriter excelWriter = null;

    private WriteSheet writeSheet = null;

    /**
     * EasyExcel工具类，初始化
     * @param absFilePath   绝对路径
     * @param sheetName     标签名字
     * @param clazz     标题头（第一行）
     */
    public void init(String absFilePath, String sheetName, Class clazz) {
        if (excelWriter == null && writeSheet == null) {

            File file = new File(absFilePath);
            if (!file.exists()) {
                file.getParentFile().mkdir();
            }

            // 指定写哪个标题头
            excelWriter = EasyExcelFactory.write(absFilePath).head(clazz).build();
            // 吐过同一个sheet只要创建一次
            writeSheet = EasyExcelFactory.writerSheet(sheetName).build();
        }
    }

    /**
     * EasyExcel工具类，初始化
     * @param outputStream   输出流
     * @param sheetName     标签名字
     * @param clazz     标题头（第一行）
     */
    public void init(OutputStream outputStream, String sheetName, Class clazz) {
        if (excelWriter == null && writeSheet == null) {
            // 指定写哪个标题头
            excelWriter = EasyExcelFactory.write(outputStream).head(clazz).build();
            // 吐过同一个sheet只要创建一次
            writeSheet = EasyExcelFactory.writerSheet(sheetName).build();
        }
    }

    /**
     * 写入Excel文件内容
     * @param dataList
     */
    public void addExcel(List<?> dataList) {
        try {
            excelWriter.write(dataList, writeSheet);
        } catch (Exception e) {
            LOGGER.error("追加Excel文件失败，原因：", e);
        }
    }

    /**
     * 最后调用，关闭流
     */
    public void finish() {
        if (excelWriter != null) {
            excelWriter.finish();
        }
    }

}
