package com.cusc.nirvana.user.rnr.enterprise.service;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.dto.*;
import com.cusc.nirvana.user.rnr.fp.dto.H5ValidationOCRReqDTO;
import com.cusc.nirvana.user.rnr.fp.dto.LivenessCodeResponseDTO;
import com.cusc.nirvana.user.rnr.fp.dto.RnrResponseDTO;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;

/**
 * 个人实名认证服务接口
 *
 * @author huchenhui
 * @date 2022-04-14
 */
public interface IPersonalRnrService {

    /**
     * 验证车辆VIN号和ICCID，包括车卡关系、卡状态等
     *
     * @param vinCardListDTO dto bean
     * @return 实例对象
     */
    Response<PersonalRnrResponseDTO> verifyVinCards(VinCardInfoDTO vinCardListDTO);

    /**
     * 提交个人实名认证
     *
     * @param requestDTO dto bean
     * @return 实例对象
     */
    Response<PersonalRnrDTO> submitRnr(PersonalRnrRequestDTO requestDTO);

    /**
     * 根据VIN查询ICCID集合
     *
     * @param vinDTO VinDTO
     * @return 实例对象
     */
    Response getIccidListByVin(VinDTO vinDTO);

    /**
     * 检查参数并转换
     *
     * @param requestDTO
     * @return
     */
    Response<RnrRelationDTO> checkAndConvertToRelation(PersonalRnrRequestDTO requestDTO, boolean unbind);

    /**
     * 获取活体认证code
     */
    Response<LivenessCodeResponseDTO> getLivenessCode();

    /**
     * 个人实名H5提交
     *
     * @param bean
     * @return
     */
    PersonalRnrH5RespDTO submitRnrH5(PersonalRnrH5ReqDTO bean);


	/**
	 * 腾讯活体的回调处理
	 * @param bean
	 * @return
	 */
	PersonalRnrH5CallBackRespDTO afreshLivenessUrl(LivenessCallbackReqDTO bean);

    Response<RnrResponseDTO> h5ValidationOCR(H5ValidationOCRReqDTO bean);
}
