package com.cusc.nirvana.user.rnr.enterprise.service;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.DeleteByUuidDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.ImportSimDTO;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.dto.OrgBusinessTagDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgBusinessTagQueryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelQueryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryQueryDTO;

import javax.validation.constraints.NotBlank;
import java.util.List;

public interface ISimVehicleService {

    /**
     * 查询组织信息列表
     * @param organizationDTO
     * @return
     */
    Response<List<OrganizationDTO>> queryOrgByList(OrganizationDTO organizationDTO);

    /**
     * 导入卡信息
     * @param importSimDTO
     * @return
     */
    Response<?> importSim(ImportSimDTO importSimDTO);

    /**
     * 插卡解绑
     * @param requestDTO
     * @return
     */
    Response<SimVehicleDTO> unbind(BindRequestDTO requestDTO);

    /**
     * 根据业务编码获取指定Sim卡信息
     * @param uuid
     * @return
     */
    Response<IccidDetailDTO> getSimByUuid(String uuid);

    /**
     * 分析查询SIM卡信息列表
     * @param queryDTO
     * @return
     */
    Response<PageResult<IccidDetailDTO>> querySimByPage(OrgSimRelQueryDTO queryDTO);

    /**
     * 根据VIN码查询车卡绑定信息
     * @param orgUuid
     * @param vin
     * @return
     */
    Response<List<VinIccidDTO>> querySimVehicleByVin(String orgUuid, String vin);

    /**
     * 询车卡绑定列表信息
     * @param queryDTO
     * @return
     */
    Response<PageResult<SimVehicleResultDTO>> querySimVehiclePage(SimVehicleQueryDTO queryDTO);

    /**
     * 删除历史导入信息
     * @param uuidDTO
     * @return
     */
    Response deleteImportHisotry(DeleteByUuidDTO uuidDTO);

    /**
     * 查询指定导入历史信息
     * @param uuid
     * @return
     */
    Response<SimFileHistoryDTO> getImportHistory(String uuid);

    /**
     * 分页查询导入历史信息
     * @param queryDTO
     * @return
     */
    Response<PageResult<SimFileHistoryDTO>> queryImportHistory(SimFileHistoryQueryDTO queryDTO);

    /**
     * 查询指定业务Tag数据
     * @param uuid
     * @return
     */
    Response<OrgBusinessTagDTO> getTagByUuid(String uuid);

    /**
     * 查询标签信息列表
     * @param queryDto
     * @return
     */
    Response<List<OrgBusinessTagDTO>> queryTags(OrgBusinessTagQueryDTO queryDto);
}
