package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.client.EiamUrlClient;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.client.RoleClient;
import com.cusc.nirvana.user.eiam.client.UserClient;
import com.cusc.nirvana.user.eiam.dto.EiamUrlDTO;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.eiam.dto.RoleDTO;
import com.cusc.nirvana.user.eiam.dto.UserDTO;
import com.cusc.nirvana.user.eiam.dto.UserRoleDTO;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.dto.UserAccountDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.UserAccountPageQueryDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.UserAccountUpdateDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.UserAddRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IAccountService;
import com.cusc.nirvana.user.rnr.enterprise.service.IUserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author stayAnd
 * @date 2022/4/15
 */
@Service
public class AccountServiceImpl implements IAccountService {

	@Resource
	private IUserService userService;

	@Resource
	private UserClient userClient;

	@Value("${user.eiam.applicationId:4}")
	private String applicationId;

	@Resource
	private OrganizationClient organizationClient;

	@Resource
	private RoleClient roleClient;

	@Resource
	private EiamUrlClient eiamUrlClient;

	@Override
	public void addAccount(UserAddRequestDTO dto) {

		UserDTO queryAccount = userService.queryUserInInfoByPhone(dto.getPhone());
		if (null != queryAccount) {
			throw new CuscUserException("","用户手机号码已存在");
		}

		String organId;
		if (StringUtils.isBlank(dto.getOrganId())) {
			String userId = UserSubjectUtil.getUserId();
			organId = userService.getOrganId(userId,UserSubjectUtil.getTenantNo());
		} else {
			organId = dto.getOrganId();
		}

		UserDTO userDTO = dto.toUserDto();
		userDTO.setTenantNo(UserSubjectUtil.getTenantNo());
		userDTO.setApplicationId(applicationId);
		UserDTO addUserDto = userService.addUser(userDTO);
		userService.addUserRelationRole(addUserDto.getUuid(),organId,dto.getRoleCode());

		//刷新用户权限
		EiamUrlDTO eiamUrlDTO = new EiamUrlDTO();
		eiamUrlDTO.setTenantNo(UserSubjectUtil.getTenantNo());
		eiamUrlDTO.setApplicationId(applicationId);
		eiamUrlDTO.setUserId(addUserDto.getUuid());
		eiamUrlClient.userRoleResUrlToRedis(eiamUrlDTO);
	}

	@Override
	public PageResult<UserAccountDTO> pageList(UserAccountPageQueryDTO dto) {
		String organId;
		if (StringUtils.isBlank(dto.getOrganId())) {
			String userId = UserSubjectUtil.getUserId();
			organId = userService.getOrganId(userId,UserSubjectUtil.getTenantNo());
		} else {
			organId = dto.getOrganId();
		}

		UserDTO pageQuery = dto.toQueryDto();
		pageQuery.setApplicationId(applicationId);
		pageQuery.setTenantNo(UserSubjectUtil.getTenantNo());
		pageQuery.setQueryOrganId(organId);
		Response<PageResult<UserDTO>> response = userClient.page(pageQuery);
		PageResult<UserDTO> page = response.getData();

		List<UserDTO> userDTOList = page.getList();

		OrganizationDTO organDto = new OrganizationDTO();
		organDto.setUuid(organId);
		organDto.setTenantNo(UserSubjectUtil.getTenantNo());
		OrganizationDTO organizationDTO = organizationClient.getByUuid(organDto).getData();

		List<UserAccountDTO> userAccountDTOList = userDTOList.stream().map(userDTO -> {
			UserAccountDTO userAccountDTO = new UserAccountDTO();
			userAccountDTO.setId(userDTO.getUuid());
			userAccountDTO.setUserName(userDTO.getFullName());
			userAccountDTO.setNickName(userDTO.getNickName());
			userAccountDTO.setPhone(userDTO.getPhone());
			userAccountDTO.setAccount(userDTO.getUserName());
			userAccountDTO.setStatus(userDTO.getStatus());
			userAccountDTO.setOrganName(organizationDTO.getOrganName());
			userAccountDTO.setCreateTime(userDTO.getCreateTime());
			UserRoleDTO roleDto = new UserRoleDTO();
			roleDto.setTenantNo(UserSubjectUtil.getTenantNo());
			roleDto.setApplicationId(applicationId);
			roleDto.setUserId(userDTO.getUuid());
			Response<List<RoleDTO>> roleResponse = roleClient.queryRoleByUserId(roleDto);
			if (null != roleResponse && roleResponse.isSuccess() && null !=roleResponse.getData()){
				List<RoleDTO> roleDTOList = roleResponse.getData();
				roleDTOList.stream().findFirst().ifPresent(roleDTO -> userAccountDTO.setRoleName(roleDTO.getRoleName()));
			}

			return userAccountDTO;
		}).collect(Collectors.toList());



		return new PageResult<>(userAccountDTOList,page.getTotalCount(),page.getPageSize(),page.getCurrPage());
	}

	@Override
	public void update(UserAccountUpdateDTO dto) {
		UserDTO updateDto = new UserDTO();
		updateDto.setApplicationId(applicationId);
		updateDto.setTenantNo(UserSubjectUtil.getTenantNo());
		updateDto.setUuid(dto.getId());
		updateDto.setFullName(dto.getUserName());
		updateDto.setUserName(dto.getAccount());
		updateDto.setPhone(dto.getPhone());
		updateDto.setNickName(dto.getNickName());
		userClient.update(updateDto);

		//更改角色
		if (StringUtils.isNotBlank(dto.getRoleCode())) {
			userService.updateUserRole(dto.getRoleCode(),dto.getId());
		}

		//刷新用户权限
		EiamUrlDTO eiamUrlDTO = new EiamUrlDTO();
		eiamUrlDTO.setTenantNo(UserSubjectUtil.getTenantNo());
		eiamUrlDTO.setApplicationId(applicationId);
		eiamUrlDTO.setUserId(updateDto.getUuid());
		eiamUrlClient.userRoleResUrlToRedis(eiamUrlDTO);
	}


}
