package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.client.UserClient;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.service.IAdminRnrOrderService;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrOrderClient;
import com.cusc.nirvana.user.rnr.fp.dto.RnrOrderDetailDTO;
import com.cusc.nirvana.user.rnr.fp.dto.RnrOrderListQueryDTO;
import com.cusc.nirvana.user.rnr.fp.dto.RnrOrderListResponseDTO;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author stayAnd
 * @date 2022/8/3
 */
@Service
public class AdminRnrOrderServiceImpl implements IAdminRnrOrderService {

	@Resource
	private FpRnrOrderClient fpRnrOrderClient;



	@Resource
	private UserClient userClient;


	@Override
	public PageResult<RnrOrderListResponseDTO> pageListQuery(RnrOrderListQueryDTO dto) {
		PageResult<RnrOrderListResponseDTO> result = fpRnrOrderClient.pageListQuery(dto).getData();
		List<String> userIdList = result.getList().stream().map(RnrOrderListResponseDTO::getOperator).distinct().collect(Collectors.toList());

		//List<String> tenantNoList = result.getList().stream().map(RnrOrderListResponseDTO::getTenantNo).distinct().collect(Collectors.toList());
		//Map<String, TenantDTO> tenantMap = getTenantMap(tenantNoList);

		for (RnrOrderListResponseDTO responseDTO : result.getList()) {


			/*Optional.ofNullable(tenantMap.get(responseDTO.getTenantNo()))
					.ifPresent(tenantDTO -> responseDTO.setTenantName(tenantDTO.getTenantName()));*/

			//Vin脱敏
			responseDTO.setVin(DesensitizationUtil.desensitizeVin(responseDTO.getVin()));

			//iccid脱敏
			responseDTO.setIccid(DesensitizationUtil.desensitizeIccid(responseDTO.getIccid()));

			//车主姓名脱敏
			responseDTO.setFullName(DesensitizationUtil.desensitizeName(responseDTO.getFullName()));

			//手机号码脱敏
			responseDTO.setPhone(DesensitizationUtil.desensitizePhone(responseDTO.getPhone()));

			//证件号码脱敏
			responseDTO.setCertNumber(DesensitizationUtil.desensitizeCertNumber(responseDTO.getCertNumber()));
		}

		return result;
	}



	@Override
	public PageResult<VinCardDTO> queryCardPageByOrderId(MgRnrCardInfoDTO dto) {
		PageResult<VinCardDTO> data = fpRnrOrderClient.queryCardPageByOrderId(dto).getData();
		data.getList().forEach(vinCardDTO -> {
			vinCardDTO.setVin(DesensitizationUtil.desensitizeVin(vinCardDTO.getVin()));
			vinCardDTO.setIccid(DesensitizationUtil.desensitizeIccid(vinCardDTO.getIccid()));
		});
		return data;
	}

	@Override
	public RnrOrderDetailDTO queryOrderDetail(String orderId) {
		Response<RnrOrderDetailDTO> response = fpRnrOrderClient.queryOrderDetail(orderId);
		if (!response.isSuccess()) {
			throw new CuscUserException(response.getCode(),response.getMsg());
		}

		String tenantNo = response.getData().getBizDetail().getTenantNo();
		//Map<String, TenantDTO> tenantMap = getTenantMap(Collections.singletonList(tenantNo));
		//Optional.ofNullable(tenantMap.get(tenantNo)).ifPresent(tenantDTO -> response.getData().getBizDetail().setTenantName(tenantDTO.getTenantName()));


		RnrOrderDetailDTO.PersonDetailDTO rnrDetail = response.getData().getRnrDetail();
		//新车主/责任人姓名脱敏
		rnrDetail.setFullName(DesensitizationUtil.desensitizeName(rnrDetail.getFullName()));
		//新车主/责任人证件号码脱敏
		rnrDetail.setCertNumber(DesensitizationUtil.desensitizeCertNumber(rnrDetail.getCertNumber()));
		//新车主/责任人手机号码脱敏
		rnrDetail.setPhone(DesensitizationUtil.desensitizePhone(rnrDetail.getPhone()));

		RnrOrderDetailDTO.PersonDetailDTO liaisonDetail = response.getData().getLiaisonDetail();
		if (null != liaisonDetail) {
			//代办人姓名脱敏
			liaisonDetail.setFullName(DesensitizationUtil.desensitizeName(liaisonDetail.getFullName()));
			//代办人证件号码脱敏
			liaisonDetail.setCertNumber(DesensitizationUtil.desensitizeCertNumber(liaisonDetail.getCertNumber()));
			//代办人手机号码脱敏
			liaisonDetail.setPhone(DesensitizationUtil.desensitizePhone(liaisonDetail.getPhone()));
		}

		RnrOrderDetailDTO.CompanyDetailDTO companyDetail = response.getData().getCompanyDetail();
		if (null != companyDetail) {
			companyDetail.setCompanyCertNumber(DesensitizationUtil.desensitizeCertNumber(companyDetail.getCompanyCertNumber()));
		}

		return response.getData();
	}

	/*private Map<String, TenantDTO> getTenantMap(List<String> tenantNoList) {
		Map<String, TenantDTO> tenantMap = new ConcurrentHashMap<>(tenantNoList.size());
		tenantNoList.parallelStream().forEach(tenantNo->{
			TenantDTO tenantQuery = new TenantDTO();
			tenantQuery.setTenantNo(tenantNo);
			TenantDTO tenantDTO = tenantClient.getByTenantNo(tenantQuery).getData();
			tenantMap.put(tenantNo,tenantDTO);
		});
		return tenantMap;
	}*/


}

