package com.cusc.nirvana.user.rnr.enterprise.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.constants.AuthConstant;
import com.cusc.nirvana.user.auth.authentication.plug.util.ThreadLocalUtil;
import com.cusc.nirvana.user.auth.client.CaptchaClient;
import com.cusc.nirvana.user.auth.client.LoginClient;
import com.cusc.nirvana.user.auth.client.RandomIdClient;
import com.cusc.nirvana.user.auth.common.constants.CaptchaTypeEnum;
import com.cusc.nirvana.user.auth.identification.dto.CaptchaCreateReq;
import com.cusc.nirvana.user.auth.identification.dto.CaptchaCreateResp;
import com.cusc.nirvana.user.auth.identification.dto.CaptchaVerificationReq;
import com.cusc.nirvana.user.auth.identification.dto.MobileLoginReq;
import com.cusc.nirvana.user.auth.identification.dto.Oauth2Token;
import com.cusc.nirvana.user.auth.identification.dto.RandomIdReq;
import com.cusc.nirvana.user.auth.identification.dto.RandomIdResp;
import com.cusc.nirvana.user.auth.identification.dto.SmsSendConfig;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.config.EnterpriseConfig;
import com.cusc.nirvana.user.rnr.enterprise.constants.EnterpriseLoginSourceEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.enterprise.dto.CaptchaResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginSmsRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LoginUserNameReqDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IAuthService;
import com.cusc.nirvana.user.rnr.enterprise.service.IUserService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * @author stayAnd
 * @date 2022/4/12
 */
@Service
@Slf4j
public class AuthServiceImpl implements IAuthService {


    @Resource
    private CaptchaClient captchaClient;

    @Resource
    private LoginClient loginClient;

    @Resource
    private IUserService userService;

    @Resource
    private RandomIdClient randomIdClient;

    @Resource
    private EnterpriseConfig enterpriseConfig;

    @Override
    public CaptchaResponseDTO getGraphic(String platformSource) {
        CaptchaCreateReq req = new CaptchaCreateReq();
        req.setApplicationId(EnterpriseLoginSourceEnum.WEB.getAppId());
        req.setCaptchaHeight(40);
        req.setCaptchaWidth(90);
        req.setCaptchaType(CaptchaTypeEnum.TYPE_DEFAULT);
        req.setCaptchaLength(4);
        Response<CaptchaCreateResp> response = captchaClient.simpleGraphic(req);
        if (null == response || !response.isSuccess() || null == response.getData()) {
            throw new CuscUserException(response.getCode(), response.getMsg());
        }
        CaptchaCreateResp data = response.getData();
        return new CaptchaResponseDTO().setCaptchaImg(data.getCaptchaImg()).setRequestId(data.getRequestId());
    }

    @Override
    public void getSms(LoginSmsRequestDTO dto) {
		dto.setTenantNo(enterpriseConfig.getTenantNo());
        MobileLoginReq mobileLoginReq = new MobileLoginReq();
        BeanUtils.copyProperties(dto, mobileLoginReq);
        mobileLoginReq.setApplicationId(EnterpriseLoginSourceEnum.WEB.getAppId());
        mobileLoginReq.setCaptchaExpire(1800);
        mobileLoginReq.setCheckCaptchaImg(true);
        SmsSendConfig config = new SmsSendConfig();
		config.setStrategyCode("verification_code_sync_strategy");
        config.setSmsTemplateCode("verification_code_template_code");
        mobileLoginReq.setSmsSendConfig(config);
        log.info("getSms request = {}", JSONObject.toJSONString(mobileLoginReq));
        Response response = loginClient.sendSmsCaptcha(mobileLoginReq);
        if (!response.isSuccess()) {
            throw new CuscUserException(response.getCode(), response.getMsg());
        }
    }

    @Override
    public LoginResponseDTO login(LoginRequestDTO loginRequestDTO) {
		loginRequestDTO.setTenantNo(enterpriseConfig.getTenantNo());
        MobileLoginReq req = new MobileLoginReq();
		req.setApplicationId(EnterpriseLoginSourceEnum.WEB.getAppId());
        req.setTenantNo(enterpriseConfig.getTenantNo());
        req.setPhone(loginRequestDTO.getPhone());
        req.setCaptcha(loginRequestDTO.getSms());
        log.info("login  req = {}", JSONObject.toJSONString(req));
        Response<Oauth2Token> response = loginClient.mobileLogin(req);
        if (null == response) {
            throw new CuscUserException("", "登录系统异常");
        }
        if(!response.isSuccess()){
            throw new CuscUserException(response.getCode(), response.getMsg());
        }
        if(null == response.getData()){
            throw new CuscUserException("", "登录失败");
        }
        Oauth2Token data = response.getData();
		String organId = userService.getOrganId(data.getInfo().getUserId(), enterpriseConfig.getTenantNo());

		ThreadLocalUtil.set(AuthConstant.USER_ID_NAME, data.getInfo().getUserId());
		ThreadLocalUtil.set(AuthConstant.TENANT_NO_NAME, loginRequestDTO.getTenantNo());
		ThreadLocalUtil.set(AuthConstant.ORGAN_ID_NAME,organId);
        return new LoginResponseDTO().setAccessToken(data.getAccess_token())
                .setRefreshToken(data.getRefresh_token())
                .setExpiresIn(data.getExpires_in())
                .setLoginName(data.getInfo().getLoginName())
                .setNickName(data.getInfo().getNickName())
                .setUserId(data.getInfo().getUserId())
                .setOrganId(organId)
                .setOrganName(userService.getOrganName(organId, enterpriseConfig.getTenantNo()));

    }

    /**
     * 获取随机请求id-登录密码加密
     *
     * @return
     */
    @Override
    public Response<RandomIdResp> getLoginSecretKey(String platformSource) {
        RandomIdReq randomIdReq = new RandomIdReq();
        randomIdReq.setApplicationId(EnterpriseLoginSourceEnum.WEB.getAppId());
        return randomIdClient.getRequestIdByLogin(randomIdReq);
    }

    /**
     * 登录-用户名+密码登录
     *
     * @param bean
     * @return
     */
    @Override
    public LoginResponseDTO userNameLogin(LoginUserNameReqDTO bean) {
        bean.setTenantNo(enterpriseConfig.getTenantNo());
        if(CuscStringUtils.isNotEmpty(bean.getCaptchaCode()) && CuscStringUtils.isNotEmpty(bean.getCaptchaId())){
            //验证图形验证码
            CaptchaVerificationReq verificationReq = new CaptchaVerificationReq();
            verificationReq.setTenantNo(enterpriseConfig.getTenantNo());
            verificationReq.setApplicationId(EnterpriseLoginSourceEnum.WEB.getAppId());
            verificationReq.setCaptchaValue(bean.getCaptchaCode());
            verificationReq.setRequestId(bean.getCaptchaId());
            Response<Boolean> captchaResp = captchaClient.verificationCaptcha(verificationReq);
            if(!captchaResp.isSuccess() || !captchaResp.getData()){
                throw new CuscUserException(ResponseCode.CAPTCHA_IMAGE_CODE_ERROR.getCode(),
                        ResponseCode.CAPTCHA_IMAGE_CODE_ERROR.getMsg());
            }
        }

        bean.setApplicationId(EnterpriseLoginSourceEnum.WEB.getAppId());
        log.info("login userName  req = {}", JSONObject.toJSONString(bean));
        Response<Oauth2Token> response = loginClient.userNameLogin(bean);
        if (null == response ) {
            throw new CuscUserException(ResponseCode.USER_NAME_PASSWORD_ERROR.getCode(),
                    ResponseCode.USER_NAME_PASSWORD_ERROR.getMsg());
        }
        if (!response.isSuccess()) {
            throw new CuscUserException(response.getCode(), response.getMsg());
        }
        Oauth2Token data = response.getData();

		String organId = userService.getOrganId(data.getInfo().getUserId(), enterpriseConfig.getTenantNo());

		ThreadLocalUtil.set(AuthConstant.USER_ID_NAME, data.getInfo().getUserId());
		ThreadLocalUtil.set(AuthConstant.TENANT_NO_NAME,bean.getTenantNo());
		ThreadLocalUtil.set(AuthConstant.ORGAN_ID_NAME,organId);
        return new LoginResponseDTO().setAccessToken(data.getAccess_token())
                .setRefreshToken(data.getRefresh_token())
                .setExpiresIn(data.getExpires_in())
                .setLoginName(data.getInfo().getLoginName())
                .setNickName(data.getInfo().getNickName())
                .setUserId(data.getInfo().getUserId())
                .setOrganId(organId)
                .setOrganName(userService.getOrganName(organId, enterpriseConfig.getTenantNo()));
    }


}
