package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.constants.CustomerTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.convert.PersonalRnrRequestConvert;
import com.cusc.nirvana.user.rnr.enterprise.dto.*;
import com.cusc.nirvana.user.rnr.enterprise.service.ICardUnBindService;
import com.cusc.nirvana.user.rnr.enterprise.service.IPersonalRnrService;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrUnbindClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.constants.BizTypeEnum;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 解绑实现类
 *
 * @author yubo
 * @since 2022-04-25 09:32
 */
@Service
@Slf4j
public class CardUnBindServiceImpl implements ICardUnBindService {

    @Autowired
    IPersonalRnrService personalRnrService;

    @Autowired
    FpRnrUnbindClient rnrUnbindClient;

    @Resource
    private CardUnBindClient cardUnBindClient;

    @Resource
	private MgRnrCardInfoClient mgRnrCardInfoClient;

    /**
     * 二手车现车主解绑-自然人
     */
    @Override
    public Response secondHandPersonalUnbind(SecondHandUnbindRequestDTO bean) {
        //调用新车主实名认证接口
        PersonalRnrRequestDTO personalRnrRequestDTO = PersonalRnrRequestConvert.INSTANCE.secondHandUnbindDTOToPersonalRnrDTO(bean);
        personalRnrRequestDTO.setCustomerType(CustomerTypeEnum.USED_CAR_OWNER.getCode());
        personalRnrRequestDTO.setRnrBizzTypeEnum(RnrBizzTypeEnum.Unbound.getCode());
        Response<RnrRelationDTO> rnrRelationDTOResponse = personalRnrService.checkAndConvertToRelation(personalRnrRequestDTO, true);
        if (!rnrRelationDTOResponse.isSuccess()) {
            return Response.createError(rnrRelationDTOResponse.getMsg(), rnrRelationDTOResponse.getCode());
        }
        //调用fp接口
        RnrRelationDTO rnrRelationDTO = rnrRelationDTOResponse.getData();
        //修改order信息
        changeOrder(rnrRelationDTO.getOrder());
        Response rnrResponse = rnrUnbindClient.secondHandUnbind(rnrRelationDTO.getOrder().getSerialNumber(), bean.getRequestId(), rnrRelationDTO);
        if (rnrResponse.isSuccess()) {
            return Response.createSuccess(new PersonalRnrDTO((Integer) rnrResponse.getData()));
        }
        return rnrResponse;
    }



    /**
     * 校验工单状态
     * @param bean
     * @return
     */
    @Override
    public Response checkOrderStatus(RnrOrderDTO bean){
        //短信校验
        Response response = cardUnBindClient.checkMessageOutTime(bean);
        if(!response.isSuccess()){
            return response;
        }
        Response<RnrOrderDTO> orderStatus = cardUnBindClient.getOrderStatus(bean);
        if(!orderStatus.isSuccess()){
            return Response.createError("获取工单信息失败",orderStatus.getData());
        }
        return orderStatus;
    }

    /**
     * 车主 解绑
     *
     * @param bean
     * @return
     */
    @Override
    public Response originalOwner(VehicleCardRnrDTO bean) {
        return this.sendMessage(bean);
    }



    private RnrOrderDTO createOrder(MgRnrInfoDTO rnrInfo) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderType(RnrOrderType.UNBIND.getCode());
        rnrOrderDTO.setRnrId(rnrInfo.getUuid());
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setIsBatchOrder(1);
        rnrOrderDTO.setSerialNumber(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.PORTAL.getCode());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setTenantNo(rnrInfo.getTenantNo());
        rnrOrderDTO.setOrgId(rnrInfo.getOrgId());
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.TO_EXAMINE.getCode());
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        return rnrOrderDTO;
    }


    private MgRnrInfoDTO getRnrInfo(VehicleCardRnrDTO bean) {
        log.info("获取实名人员信息入参: {}", JSON.toJSONString(bean));
        //获取人员信息
        FpVehicleCardRnrDTO fpVehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        fpVehicleCardRnrDTO.setRnrId(bean.getRnrId());
        log.info("获取实名人员信息转换参数: {}", JSON.toJSONString(fpVehicleCardRnrDTO));
        Response<MgRnrInfoDTO> rnrInfo = cardUnBindClient.getRnrInfo(fpVehicleCardRnrDTO);
        if (!rnrInfo.isSuccess()) {
            return null;
        }
        return rnrInfo.getData();
    }

    /**
     * 发送短信
     *
     * @param bean
     * @return
     */
    @Override
    public Response sendMessage(VehicleCardRnrDTO bean) {

		//校验
		FpVehicleCardRnrDTO fpVehicleCardRnrDTO = new FpVehicleCardRnrDTO();
		fpVehicleCardRnrDTO.setVin(bean.getVin());
		Response<List<MgRnrCardInfoDTO>> cardList = cardUnBindClient.getCardList(fpVehicleCardRnrDTO);
		if (!cardList.isSuccess() || cardList.getData() == null) {
			//卡未实名或没有绑定在车上
			return Response.createError(cardList.getMsg(), cardList.getCode());
		}
		List<MgRnrCardInfoDTO> rnrCardList = cardList.getData();
		List<String> iccidList = bean.getIccidList();
		//需要解绑的卡
		List<MgRnrCardInfoDTO> toUnbindCardList = rnrCardList.stream().filter(t -> iccidList.contains(t.getIccid())).collect(Collectors.toList());

		if (toUnbindCardList.size() != iccidList.size()){
			throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"输入的卡存在未实名的卡信息");
		}

		MgRnrInfoDTO rnrInfo = getRnrInfo(bean);
		if (rnrInfo == null) {
			return Response.createError("获取实名人员信息失败");
		}

        log.info("自然人解绑短信发送入参: {}", JSON.toJSONString(bean));
        MgRnrInfoDTO rnrInfoData = getRnrInfo(bean);
        if (rnrInfoData == null) {
            return Response.createError("实名信息为空");
        }
        RnrOrderDTO order = this.createOrder(rnrInfoData);
        //插入工单信息
        Response<RnrOrderDTO> orderResponse = cardUnBindClient.insertOrder(order);
        if(!orderResponse.isSuccess() || orderResponse.getData() == null){
            return Response.createError(orderResponse.getMsg(),orderResponse.getData());
        }

        //插入卡信息
		List<MgRnrCardInfoDTO> cardInfoDTOList = createCardInfoDTOS(toUnbindCardList,rnrInfo,bean.getVin(),order.getUuid());

		Response<Boolean> cardResponse = mgRnrCardInfoClient.insertBatch(cardInfoDTOList);
		if(!cardResponse.isSuccess() || cardResponse.getData() == null){
			return Response.createError(orderResponse.getMsg(),orderResponse.getData());
		}

		//配置短信参数
        List<String> paramsList = new ArrayList<>();
        paramsList.add(rnrInfoData.getFullName() + "女士/先生");
        paramsList.add(bean.getVin());

        //拼接iccid参数列表
        String iccIdList = "";
        for (String iccid : bean.getIccidList()) {
            iccIdList += iccid + ",";
        }
        String subIccIdList = iccIdList.substring(0, iccIdList.length() - 1);

        paramsList.add(subIccIdList);
        SmsRequestDTO smsRequestDTO = new SmsRequestDTO();
        smsRequestDTO.setBizType(BizTypeEnum.UNBOUND.getCode());
        smsRequestDTO.setParams(paramsList);
        smsRequestDTO.setPhone(rnrInfoData.getPhone());

        FpSendMessageDTO fpSendMessageDTO = new FpSendMessageDTO();
        fpSendMessageDTO.setRnrid(bean.getRnrId());
        fpSendMessageDTO.setUser(rnrInfoData.getCreator());
        fpSendMessageDTO.setIccidLists(subIccIdList);
        fpSendMessageDTO.setSmsRequestDTO(smsRequestDTO);
        fpSendMessageDTO.setOrderId(order.getUuid());

        //发送短信
        Response<FpRnrSmsInfoDTO> smsResponse = cardUnBindClient.sendSms(fpSendMessageDTO);
        if(!smsResponse.isSuccess() && smsResponse.getData() == null){
            return Response.createError("短信发送失败");
        }else {
            return Response.createSuccess("短信发送成功",orderResponse.getData());
        }
    }

    /**
     * 根据vin 获取以绑定的iccids集合，以及车主信息
     */
    @Override
    public Response getIccidsByVin(VehicleCardRnrDTO bean){
        UserInfoAndCardListDTO userInfoAndCardListDTO = new UserInfoAndCardListDTO();
        FpVehicleCardRnrDTO fpVehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        fpVehicleCardRnrDTO.setVin(bean.getVin());

        Response<List<MgRnrCardInfoDTO>> cardList = cardUnBindClient.getCardList(fpVehicleCardRnrDTO);
        if(!cardList.isSuccess() || cardList.getData() == null || cardList.getData().size()<1){
            return Response.createError("卡未绑定");
        }
        //List<MgRnrCardInfoDTO> collect = cardList.getData().stream().filter(t -> t.getRnrBizzType()==RnrBizzTypeEnum.Bind.getCode()).collect(Collectors.toList());
		List<MgRnrCardInfoDTO> collect = cardList.getData();
        List<String> iccids = new ArrayList<>();
        for(MgRnrCardInfoDTO cardInfoDTO: collect){
            iccids.add(cardInfoDTO.getIccid());
        }
        VehicleCardRnrDTO cardRnrDTO = new VehicleCardRnrDTO();
        cardRnrDTO.setVin(bean.getVin());
        cardRnrDTO.setRnrId(collect.get(0).getRnrId());
        cardRnrDTO.setIccidList(iccids);
        userInfoAndCardListDTO.setVehicleCardRnrDTO(cardRnrDTO);

        bean.setRnrId(collect.get(0).getRnrId());
        MgRnrInfoDTO rnrInfo = getRnrInfo(bean);
        if(rnrInfo == null){
            return Response.createError("获取车主信息失败");
        }
        String phone = DesensitizationUtil.desensitizePhone(rnrInfo.getPhone());
        rnrInfo.setPhone(phone);
        userInfoAndCardListDTO.setMgRnrInfoDTO(rnrInfo);


        return Response.createSuccess("获取信息成功",userInfoAndCardListDTO);
    }

    //-----------------------私有方法区
    //修改order信息
    private void changeOrder(RnrOrderDTO order) {
        order.setAuditType(RnrOrderAuditTypeEnum.MANUAL.getCode());
        order.setAutoRnr(false);
        order.setOrderType(RnrOrderType.SEC_UNBIND.getCode());
        order.setSendWorkOrder(true);
    }

    private List<MgRnrCardInfoDTO> createCardInfoDTOS(List<MgRnrCardInfoDTO> toUnBindCardList, MgRnrInfoDTO rnrInfoDTO, String vin, String orderId) {
        List<MgRnrCardInfoDTO> cardInfoDTOList = new ArrayList<>();
        for (MgRnrCardInfoDTO unBindCard : toUnBindCardList) {
            MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
            mgRnrCardInfoDTO.setUuid(CuscStringUtils.generateUuid());
            mgRnrCardInfoDTO.setIccid(unBindCard.getIccid());
            mgRnrCardInfoDTO.setOldCardId(unBindCard.getUuid());
            mgRnrCardInfoDTO.setTenantNo(rnrInfoDTO.getTenantNo());
            mgRnrCardInfoDTO.setIotId(vin);
            mgRnrCardInfoDTO.setRnrId(rnrInfoDTO.getUuid());
            mgRnrCardInfoDTO.setOrderId(orderId);
			mgRnrCardInfoDTO.setRnrStatus(RnrStatus.INIT.getCode());
            mgRnrCardInfoDTO.setCreator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setOperator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
			cardInfoDTOList.add(mgRnrCardInfoDTO);
        }
        return cardInfoDTOList;
    }
}
