package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.rnr.enterprise.dto.ChangeTboxDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.IccidChangeDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IChangeTboxService;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.ChangeTboxClient;
import com.cusc.nirvana.user.rnr.fp.client.VinCardClient;
import com.cusc.nirvana.user.rnr.fp.constants.BizTypeEnum;
import com.cusc.nirvana.user.rnr.fp.dto.FpRnrSmsInfoDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpSendMessageDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpVehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.fp.dto.SmsRequestDTO;
import com.cusc.nirvana.user.rnr.fp.dto.UnbindReceiceSMSDTO;
import com.cusc.nirvana.user.rnr.fp.dto.VerifyVinCardResponseDTO;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardDTO;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * @author changh
 * @date 2022/4/24
 */
@Service
@Slf4j
public class ChangeTboxServiceImpl implements IChangeTboxService {

    @Resource
    private CardUnBindClient cardUnBindClient;

    @Resource
    private ChangeTboxClient changeTboxClient;

    @Resource
    private VinCardClient vinCardClient;

    @Resource
	private MgRnrCardInfoClient mgRnrCardInfoClient;

    @Override
    public Response submit(ChangeTboxDTO bean) {
        log.info("换卡入参：{}", JSON.toJSONString(bean));
        List<String > oldIccids = new ArrayList<>();
        List<String> newIccids = new ArrayList<>();
        for(IccidChangeDTO changeDTO: bean.getIccidList()){
            oldIccids.add(changeDTO.getOldIccid());
            newIccids.add(changeDTO.getNewIccid());
        }
        //校验旧的iccid和vin
        /*FpVehicleCardRnrDTO fpVehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        fpVehicleCardRnrDTO.setVin(bean.getVin());
        Response<List<MgRnrCardInfoDTO>> cardList = cardUnBindClient.getCardList(fpVehicleCardRnrDTO);
        if(!cardList.isSuccess() || cardList.getData() == null){
            return Response.createError("vin无法获取iccid列表");
        }
        List<MgRnrCardInfoDTO> cardDatas = cardList.getData();
        List<MgRnrCardInfoDTO> rnrCardInfoDTOS = cardDatas.stream().filter(t -> oldIccids.contains(t.getIccid())).collect(Collectors.toList());
        if(rnrCardInfoDTOS.size() != oldIccids.size()){
            return Response.createError("iccids与vin不匹配:{}",JSON.toJSONString(rnrCardInfoDTOS));
        }*/
        List<VinCardDTO> vinCardDTOList = new ArrayList<>();
        for (int i=0;i<bean.getIccidList().size();i++){
            VinCardDTO vinCardDTO = new VinCardDTO();
            vinCardDTO.setVin(bean.getVin());
            vinCardDTO.setIccid(bean.getIccidList().get(i).getOldIccid());
            vinCardDTOList.add(vinCardDTO);
        }
        log.info("校验车卡关系入参：{}", JSON.toJSONString(vinCardDTOList));
        Response<VerifyVinCardResponseDTO> verifyVinCard = vinCardClient.verifyVinCard(vinCardDTOList);
        log.info("校验车卡关系：{}", JSON.toJSONString(verifyVinCard));
        if(!verifyVinCard.isSuccess()){
            return verifyVinCard;
        }
        //校验新的iccid
        Response<Boolean> checkStatus = changeTboxClient.checkStatus(newIccids);
        log.info("校验新车卡出参：{}", JSON.toJSONString(checkStatus));
        if(!checkStatus.isSuccess() || !checkStatus.getData()){
            return Response.createError("换卡iccid状态为已实名");
        }
        //发送短信
        return this.sendSMS(bean);
        /*RnrRelationDTO rnrRelationDTO = turnParams(oldIccids,newIccids,bean.getVin(),cardDatas.get(0).getRnrId(),cardDatas.get(0).getTenantNo());
        log.info("换卡转换参数：{}", JSON.toJSONString(rnrRelationDTO));
        return  changeTboxClient.changeTbox(rnrRelationDTO);*/
    }


    private MgRnrInfoDTO getRnrInfo(ChangeTboxDTO bean){
        //获取人员信息
        FpVehicleCardRnrDTO fpVehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        fpVehicleCardRnrDTO.setVin(bean.getVin());
        Response<List<MgRnrCardInfoDTO>> cardList = cardUnBindClient.getCardList(fpVehicleCardRnrDTO);
        if(!cardList.isSuccess()){
            return null;
        }

        FpVehicleCardRnrDTO vehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        vehicleCardRnrDTO.setRnrId(cardList.getData().get(0).getRnrId());
        Response<MgRnrInfoDTO> rnrInfo = cardUnBindClient.getRnrInfo(vehicleCardRnrDTO);
        if(!rnrInfo.isSuccess()){
            return null;
        }
        return rnrInfo.getData();
    }

    /*private  RnrRelationDTO turnParams(List<String> oldIccids, List<String> newIccids, String vin,String rnrid, String tenNo){
        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();
        RnrOrderDTO order = createOrder(rnrid, tenNo);
        List<MgRnrCardInfoDTO> cardInfoDTOS = new ArrayList<>();
        List<MgRnrCardInfoDTO> oldCardInfoDTOS = createCardInfoDTOS(oldIccids, tenNo, vin, rnrid,order.getUuid());
        List<MgRnrCardInfoDTO> newCardInfoDTOS = createCardInfoDTOS(newIccids, tenNo, vin, rnrid,order.getUuid());
        cardInfoDTOS.addAll(oldCardInfoDTOS);
        cardInfoDTOS.addAll(newCardInfoDTOS);
        rnrRelationDTO.setOrder(order);
        rnrRelationDTO.setCardList(cardInfoDTOS);
        return rnrRelationDTO;
    }*/

    private RnrOrderDTO createOrder(MgRnrInfoDTO rnrInfo){
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.ChangeBinding.getCode());
        rnrOrderDTO.setOrderType(RnrOrderType.TBOX_CHANGE.getCode());
        rnrOrderDTO.setRnrId(rnrInfo.getUuid());
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setIsBatchOrder(1);
        rnrOrderDTO.setSerialNumber(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.PORTAL.getCode());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setTenantNo(rnrInfo.getTenantNo());
        rnrOrderDTO.setOrgId(rnrInfo.getOrgId());
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.TO_EXAMINE.getCode());
        return rnrOrderDTO;
    }
    private List<MgRnrCardInfoDTO> createCardInfoDTOS(ChangeTboxDTO changeTboxDTO,String tenNo,String rnrid,String orderId,Long routingKey){

    	//旧卡
		List<String> oldIccidList = changeTboxDTO.getIccidList().stream().map(IccidChangeDTO::getOldIccid).collect(Collectors.toList());

		//查询旧卡已经实名的业务
		MgRnrCardInfoDTO query = new MgRnrCardInfoDTO();
		query.setIccidList(oldIccidList);
		query.setRnrId(rnrid);
		query.setRnrStatus(RnrStatus.RNR.getCode());
		Map<String, String> oldIccidMap = mgRnrCardInfoClient.queryByList(query).getData().stream().collect(Collectors.toMap(MgRnrCardInfoDTO::getIccid, MgRnrCardInfoDTO::getUuid, (o1, o2) -> o1));

		List<MgRnrCardInfoDTO> cardInfoDTOList = new ArrayList<>();
        for(IccidChangeDTO dto : changeTboxDTO.getIccidList()){
            MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
            mgRnrCardInfoDTO.setUuid(CuscStringUtils.generateUuid());
            mgRnrCardInfoDTO.setIccid(dto.getNewIccid());
            mgRnrCardInfoDTO.setOldCardId(oldIccidMap.getOrDefault(dto.getOldIccid(),""));
            mgRnrCardInfoDTO.setTenantNo(tenNo);
            mgRnrCardInfoDTO.setIotId(changeTboxDTO.getVin());
            mgRnrCardInfoDTO.setRnrId(rnrid);
            mgRnrCardInfoDTO.setOrderId(orderId);
            mgRnrCardInfoDTO.setCreator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setOperator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setRnrStatus(RnrStatus.INIT.getCode());
            mgRnrCardInfoDTO.setNoticeStatus(NoticeStatusEnum.NONEED.getCode());
            mgRnrCardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.ChangeBinding.getCode());
            mgRnrCardInfoDTO.setRoutingKey(routingKey);
			cardInfoDTOList.add(mgRnrCardInfoDTO);
        }
        return cardInfoDTOList;
    }


    @Override
    public Response queryResult(ChangeTboxDTO bean) {
        //1.通过submit方法响应的新的rnrId查询业务结果
        return Response.createSuccess(true);
    }
	/**
	 * 发送短信
	 */
    @Override
    public Response sendSMS(ChangeTboxDTO bean) {
        log.info("换卡发送短信入参：{}", JSON.toJSONString(bean));

        MgRnrInfoDTO rnrInfoData = getRnrInfo(bean);
        if (rnrInfoData == null){
            return Response.createError("无法获取到手机号信息");
        }
        List<String> paramsList = getMessageContext(bean);

        SmsRequestDTO smsRequestDTO = new SmsRequestDTO();
        smsRequestDTO.setBizType(BizTypeEnum.CHANGETBOX.getCode());
        smsRequestDTO.setParams(paramsList);
        smsRequestDTO.setPhone(rnrInfoData.getPhone());

        FpSendMessageDTO fpSendMessageDTO = new FpSendMessageDTO();
        fpSendMessageDTO.setRnrid(rnrInfoData.getUuid());
        fpSendMessageDTO.setUser(rnrInfoData.getCreator());
        fpSendMessageDTO.setIccidLists(paramsList.get(2));
        fpSendMessageDTO.setSmsRequestDTO(smsRequestDTO);

        RnrOrderDTO order = createOrder(rnrInfoData);
        order.setRoutingKey(rnrInfoData.getRoutingKey());
        fpSendMessageDTO.setOrderId(order.getUuid());

        //构建卡信息
		List<MgRnrCardInfoDTO> cardInfoDTOList = createCardInfoDTOS(bean, rnrInfoData.getTenantNo(), rnrInfoData.getUuid(), order.getUuid(), rnrInfoData.getRoutingKey());


        //发送短信
        Response<FpRnrSmsInfoDTO> smsResponse = cardUnBindClient.sendSms(fpSendMessageDTO);
        if(!smsResponse.isSuccess()){
            return Response.createError("短信发送失败");
        }else {
            Response<RnrOrderDTO> orderResponse = cardUnBindClient.insertOrder(order);
			Response<Boolean> cardResponse = mgRnrCardInfoClient.insertBatch(cardInfoDTOList);
			if(orderResponse.isSuccess() && cardResponse.isSuccess()){
                return Response.createSuccess("短信发送成功",order);
            }else {
                return Response.createError("创建工单/卡信息失败");
            }
        }
    }

    private List<String> getMessageContext(ChangeTboxDTO bean){
        MgRnrInfoDTO rnrInfoData = getRnrInfo(bean);
        //配置短信参数
        List<String> paramsList = new ArrayList<>();
        /*if (rnrInfoData.getGender() == 0) {
            paramsList.add(rnrInfoData.getFullName() + "女士");
        } else {
            paramsList.add(rnrInfoData.getFullName() + "先生");
        }*/
        paramsList.add(rnrInfoData.getFullName() + "女士/先生");
        paramsList.add(bean.getVin());
        //paramsList.add(bean.getVin());

        String iccIdList = "";
        for (IccidChangeDTO changeDTO: bean.getIccidList()){
            iccIdList += changeDTO.getOldIccid() + "换为"+changeDTO.getNewIccid() + ",";
        }
        String substring = iccIdList.substring(0,iccIdList.length()-1);
        paramsList.add(substring);
        return paramsList;
    }

    //短信回调接口
    @Override
    public Response receiveMessage(UnbindReceiceSMSDTO bean) {
        log.info("换卡短信回调入参：{}", JSON.toJSONString(bean));
        return cardUnBindClient.receiveMessage(bean);
    }

    @Override
    public Response checkSMSStatus(FpRnrSmsInfoDTO bean) {
        log.info("换卡校验短信状态入参：{}", JSON.toJSONString(bean));
        Response<FpRnrSmsInfoDTO> messageStatus = cardUnBindClient.getMessageStatus(bean);
        if(!messageStatus.isSuccess() || messageStatus.getData() == null){
            return Response.createError("获取短信失败");
        }
        long currentTimeMillis = System.currentTimeMillis();
        //校验时间2分钟内
        Date createTime = messageStatus.getData().getCreateTime();
        long startTime = createTime.getTime();
        if(currentTimeMillis-120*1000 > startTime){
            return Response.createError("短信获取超时");
        }
        //校验内容
        if(!messageStatus.getData().getReceiveContent().toLowerCase().equals("y")){
            return Response.createError("短信回复为N");
        }
        return Response.createSuccess("短信校验通过");
    }
}
