package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.dto.*;
import com.cusc.nirvana.user.rnr.enterprise.service.IEnterpriseRnrService;
import com.cusc.nirvana.user.rnr.enterprise.service.IPersonalRnrService;
import com.cusc.nirvana.user.rnr.enterprise.service.IUserService;
import com.cusc.nirvana.user.rnr.enterprise.service.IVehicleCardService;
import com.cusc.nirvana.user.rnr.enterprise.util.FileUtil;
import com.cusc.nirvana.user.rnr.fp.client.EnterpriseRnrClient;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrClient;
import com.cusc.nirvana.user.rnr.fp.client.NewVinCardClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.constants.BizTypeEnum;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.constants.CertTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrFileType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderAuditTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderSourceEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCompanyInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrFileDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrLiaisonInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrTagDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import com.cusc.nirvana.user.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class EnterpriseRnrServiceImpl implements IEnterpriseRnrService {


    @Autowired
    IVehicleCardService vehicleCardService;

    @Autowired
    SmsServiceImpl smsService;

    @Autowired
    IPersonalRnrService personalRnrService;

    @Resource
    private EnterpriseRnrClient enterpriseClient;

   @Resource
   private OrganizationClient organizationClient;

   @Resource
   private IUserService userService;

   @Resource
   private NewVinCardClient checkClient;

    @Autowired
    FpRnrClient fpRnrClient;

   private static final String SUCCESS_CODE = "0";

   @Value("${rnr.frontEnterpriseH5CallBackUrl}")
   private String frontEnterpriseH5CallBackUrl;


	@Value("${rnr.frontEnterprisePadCallBackUrl}")
	private String frontEnterprisePadCallBackUrl;


    /**
     * 校验车卡关系
     *
     * @param dto dto bean
     * @return
     */
    @Override
    public Response<EMVinCardResponseDTO> verifyVinCards(EnterpriseRnrVinCardInfoDTO dto) {
        if (StringUtils.isBlank(dto.getFileId())) {
            return Response.createError("文件ID不能为空", ResponseCode.INVALID_DATA.getCode());
        }
        Response<EMVinCardResponseDTO> emVinCardResponseDTOResponse = vehicleCardService.verifyAndUploadVinCard(dto.getFileId());
        if (emVinCardResponseDTOResponse.isSuccess() && emVinCardResponseDTOResponse.getData().getTotalCount() > 5000) {
            return Response.createError("文件条数应小于5000条");
        }
        return emVinCardResponseDTOResponse;
    }

    @Override
    public Response<CompanyRnrResponseDTO> submitRnr(EnterpriseRnrCertificationInfoDTO dto) {
        log.info("入参信息:{}", JSON.toJSONString(dto));

        //验证车卡关系
        Response<List<VinCardDTO>> listResponse = vehicleCardService.checkIccidList(dto.getFileId());
        log.info("验证车卡关系", dto);
        if (!listResponse.isSuccess()) {
            return Response.createSuccess(false);
        }
        //验证责任人证件有效期
        String currentDate = new SimpleDateFormat("yyyy-MM-dd").format(new Date(System.currentTimeMillis()));
        String corporationCertExpirationDate = dto.getCorporationCertExpirationDate();
        if (currentDate.compareTo(corporationCertExpirationDate) > 0) {
            return Response.createError("证件已过期");
        }

        //验证短信
        /*SmsRequestDTO smsRequestDTO = new SmsRequestDTO();
        smsRequestDTO.setPhone(dto.getCorporationPhone());
        smsRequestDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        smsRequestDTO.setBizType(BizTypeEnum.RNR.getCode());
        smsRequestDTO.setCaptcha(dto.getVerificationCode());
        Response smsResponse = smsService.checkSmsCaptcha(smsRequestDTO);
        if (!smsResponse.isSuccess()) {
            return Response.createError(smsResponse.getMsg(), smsResponse.getCode());
        }*/
        //调用fp接口 提交企业实名
        RnrRelationDTO rnrRelationDTO = turnParams(dto, listResponse.getData());

        if (CollectionUtils.isEmpty(rnrRelationDTO.getCardList())) {
            return Response.createError("ICCID不能为空");
        }

        log.info("dto111111111111:{}", JSON.toJSONString(dto));
        log.info("RnrRelationDTO11:{}", JSON.toJSONString(rnrRelationDTO));
        rnrRelationDTO.setRequestId(dto.getRequestId());
        rnrRelationDTO.setSerialNumber(rnrRelationDTO.getOrder().getSerialNumber());
        Response<Boolean> enterpriseResponse = enterpriseClient.enterpriseRnr(rnrRelationDTO, dto.getRequestId());
        log.info("提交企业实名:{}", JSON.toJSONString(enterpriseResponse));
        if (!enterpriseResponse.isSuccess()) {
            return Response.createError(enterpriseResponse.getMsg(), enterpriseResponse.getCode());
        } else {
            Response submitResponse = Response.createSuccess();
            CompanyRnrResponseDTO companyRnrResponseDTO = new CompanyRnrResponseDTO();
            companyRnrResponseDTO.setVerifyStatus(1);
            submitResponse.setData(companyRnrResponseDTO);
            return submitResponse;
        }
    }

    @Override
    public Map<String, Object> queryEnterpriseRnrPerson(EnterpriseCorporationChangeDTO dto) {
        Map<String, Object> returnMap = new HashMap<>();
        EnterpriseRnrPersonNameReqDTO query = new EnterpriseRnrPersonNameReqDTO();
        query.setCompanyName(dto.getCompanyName());
        query.setTenantNo(UserSubjectUtil.getTenantNo());

        String organId = userService.getOrganId(UserSubjectUtil.getUserId(), UserSubjectUtil.getTenantNo());
        if (StringUtils.isEmpty(organId)) {
            returnMap.put("errorMsg", "该用户没有组织id");
            returnMap.put("errorCode", 10001);
            return returnMap;
        }
        OrganizationDTO orgQuery = new OrganizationDTO();
        orgQuery.setUuid(organId);
        orgQuery.setTenantNo(UserSubjectUtil.getTenantNo());
        OrganizationDTO data = organizationClient.getByUuid(orgQuery).getData();

        //查询的人的组织 是否是车企
        if (null == data.getBizType()) {
            returnMap.put("errorMsg", "该用户不是车企");
            returnMap.put("errorCode", 10001);
            return returnMap;
        }
        query.setVehicleCompany(data.getBizType() == 1);

        Response<List<EnterpriseRnrPersonNameResponseDTO>> listResponse = enterpriseClient.queryEnterpriseRnrPersonName(query);
        if (!listResponse.getSuccess()) {
            throw new CuscUserException(listResponse.getCode(), listResponse.getMsg());
        }
//		return listResponse.getData().stream().map(responseDTO->{
//			EnterpriseRnrPersonDTO rnrPersonDTO = new EnterpriseRnrPersonDTO();
//			rnrPersonDTO.setCompanyName(responseDTO.getCompanyName());
//			rnrPersonDTO.setCorporationName(responseDTO.getFullName());
//			return rnrPersonDTO;
//		}).collect(Collectors.toList());
        List<EnterpriseRnrPersonDTO> returnList = listResponse.getData().stream().map(responseDTO -> {
            EnterpriseRnrPersonDTO rnrPersonDTO = new EnterpriseRnrPersonDTO();
            rnrPersonDTO.setCompanyName(responseDTO.getCompanyName());
            rnrPersonDTO.setCorporationName(responseDTO.getFullName());
            return rnrPersonDTO;
        }).collect(Collectors.toList());
        if (returnList.size() <= 0) {
            returnMap.put("errorMsg", "该用户未查询相关公司");
            returnMap.put("errorCode", 10001);
            return returnMap;
        } else {
            returnMap.put("errorCode", 10000);
            returnMap.put("returnList", returnList);
            return returnMap;
        }
    }

    @Override
    public void submitChange(EnterpriseCorporationChangeDTO bean) {
        EnterpriseChangeRnrPersonDTO dto = new EnterpriseChangeRnrPersonDTO();
        BeanUtils.copyProperties(bean, dto);
        dto.setTenantNo(UserSubjectUtil.getTenantNo());
        dto.setCreator(UserSubjectUtil.getUserId());
        Response response = enterpriseClient.changeEnterpriseRnrPerson(CuscStringUtils.generateUuid(), bean.getRequestId(), dto);
        if (!response.getSuccess()) {
            throw new CuscUserException(response.getCode(), response.getMsg());
        }
    }

    //转换成RelationDto
    private RnrRelationDTO turnParams(EnterpriseRnrCertificationInfoDTO dto, List<VinCardDTO> vinCardDTOList) {
        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();
        String rnrID = CuscStringUtils.generateUuid();

        String tenantNo = UserSubjectUtil.getTenantNo();

        //查询当前用户的组织id
        String orgId = userService.getOrganId(UserSubjectUtil.getUserId(), tenantNo);

        //租户
        rnrRelationDTO.setTenantNo(tenantNo);
        rnrRelationDTO.setIsTrust(0);
//        rnrRelationDTO.setIsSecondHandCar(dto.getCustomerType() == 1 ? 1 : 0);
        //公司信息
        MgRnrCompanyInfoDTO mgRnrCompanyInfoDTO = createCompanyInfoDTO(dto, rnrID, tenantNo);
        //人的信息
        MgRnrInfoDTO mgRnrInfoDTO = createRnrInfo(dto, rnrID, tenantNo, mgRnrCompanyInfoDTO.getUuid());
        mgRnrInfoDTO.setOrgId(orgId);
		mgRnrInfoDTO.setIsTrust(0);
        int orderType = 0;
        if (mgRnrCompanyInfoDTO.getIsVehicleCompany() == 0) {
            orderType = RnrOrderType.COMPANY_NEW_VEHICLE.getCode();
        } else {
            orderType = RnrOrderType.CARMAKER_NEW_VEHICLE.getCode();
        }
        //订单信息
        RnrOrderDTO rnrOrderDTO = createOrder(dto, mgRnrInfoDTO, mgRnrInfoDTO.getUuid(), tenantNo, orderType);
        rnrOrderDTO.setOrgId(orgId);
        //卡信息
        List<MgRnrCardInfoDTO> cardInfoDTOS = createCardInfoDTOs(dto, rnrRelationDTO, tenantNo, rnrID, rnrOrderDTO.getUuid(), vinCardDTOList);
        //文件列表
        List<MgRnrFileDTO> fileDTOS = getMgRnrFileDTOs(dto, new RnrRelationDTO(), mgRnrInfoDTO.getUuid(), tenantNo, mgRnrCompanyInfoDTO.getUuid());
        //taglist
        List<MgRnrTagDTO> tagDTOS = new ArrayList<>();
        //实名联系人信息
        List<MgRnrLiaisonInfoDTO> liaisonInfoDTOS = new ArrayList<>();

        rnrRelationDTO.setRnrLiaisonList(liaisonInfoDTOS);
        rnrRelationDTO.setCompanyInfo(mgRnrCompanyInfoDTO);
        rnrRelationDTO.setInfo(mgRnrInfoDTO);
        rnrRelationDTO.setCardList(cardInfoDTOS);
        rnrRelationDTO.setOrder(rnrOrderDTO);
        rnrRelationDTO.setRnrFileList(fileDTOS);
        rnrRelationDTO.setRnrTagList(tagDTOS);
        return rnrRelationDTO;
    }

    //创建工单
    private RnrOrderDTO createOrder(EnterpriseRnrCertificationInfoDTO dto, MgRnrInfoDTO rnrInfoDTO, String uuid, String tenNo, int orderType) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderType(orderType);
        rnrOrderDTO.setRnrId(uuid);
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.MANUAL.getCode());
        rnrOrderDTO.setAutoRnr(false);
        rnrOrderDTO.setIsBatchOrder(1);
        rnrOrderDTO.setSerialNumber(rnrInfoDTO.getSerial_number());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.PORTAL.getCode());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setTenantNo(tenNo);
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.TO_EXAMINE.getCode());
        rnrOrderDTO.setSendWorkOrder(true);
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
        return rnrOrderDTO;
    }

    //卡信息
    private List<MgRnrCardInfoDTO> createCardInfoDTOs(EnterpriseRnrCertificationInfoDTO dto, RnrRelationDTO relationDTO, String tenantNo, String rnrId, String uuid, List<VinCardDTO> vinCardDTOS) {
        List<MgRnrCardInfoDTO> cardInfoDTOS = new ArrayList<>();
        for (VinCardDTO cardDTO : vinCardDTOS) {
            MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
            mgRnrCardInfoDTO.setUuid(CuscStringUtils.generateUuid());
            mgRnrCardInfoDTO.setIccid(cardDTO.getIccid());
            mgRnrCardInfoDTO.setTenantNo(tenantNo);
            mgRnrCardInfoDTO.setIotId(cardDTO.getVin());
            mgRnrCardInfoDTO.setRnrId(rnrId);
            mgRnrCardInfoDTO.setOrderId(uuid);
            mgRnrCardInfoDTO.setCreator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setOperator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
            cardInfoDTOS.add(mgRnrCardInfoDTO);
        }
        return cardInfoDTOS;
    }

    //公司信息
    private MgRnrCompanyInfoDTO createCompanyInfoDTO(EnterpriseRnrCertificationInfoDTO dto, String rnrId, String tenNo) {
        MgRnrCompanyInfoDTO mgRnrCompanyInfoDTO = new MgRnrCompanyInfoDTO();
        mgRnrCompanyInfoDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrCompanyInfoDTO.setRnrId(rnrId);
        mgRnrCompanyInfoDTO.setCompanyCertAddress(dto.getCompanyCertAddress());
        mgRnrCompanyInfoDTO.setCompanyCertNumber(dto.getCompanyCertNumber());
        mgRnrCompanyInfoDTO.setCompanyCertType(dto.getCompanyCertType());
        mgRnrCompanyInfoDTO.setCompanyContactAddress(dto.getCompanyContactAddress());
        mgRnrCompanyInfoDTO.setCompanyName(dto.getCompanyName());
        mgRnrCompanyInfoDTO.setCompanyType(dto.getCompanyType());
        mgRnrCompanyInfoDTO.setIndustryType(dto.getIndustryType());
        mgRnrCompanyInfoDTO.setTenantNo(tenNo);
        mgRnrCompanyInfoDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrCompanyInfoDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrCompanyInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
        //是否是车企，数据库默认值是0
        if (dto.getIsVehicleCompany() == null) {
            dto.setIsVehicleCompany(0);
        }
        mgRnrCompanyInfoDTO.setIsVehicleCompany(dto.getIsVehicleCompany());
        return mgRnrCompanyInfoDTO;
    }

    //人信息
    private MgRnrInfoDTO createRnrInfo(EnterpriseRnrCertificationInfoDTO dto, String rnrId, String tenantNo, String uuid) {
        MgRnrInfoDTO mgRnrInfoDTO = new MgRnrInfoDTO();
        mgRnrInfoDTO.setUuid(rnrId);
        mgRnrInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        mgRnrInfoDTO.setSerial_number(CuscStringUtils.generateUuid());
        mgRnrInfoDTO.setIsCompany(1);
        mgRnrInfoDTO.setCertAddress(dto.getCorporationCertAddress());
        mgRnrInfoDTO.setCertType(dto.getCorporationCertType());
        mgRnrInfoDTO.setCertNumber(dto.getCorporationCertNumber());
        mgRnrInfoDTO.setContactAddress(dto.getCorporationContactAddress());
        mgRnrInfoDTO.setFullName(dto.getCorporationName());
        mgRnrInfoDTO.setGender(Integer.valueOf(dto.getCorporationGender()));
        mgRnrInfoDTO.setPhone(dto.getCorporationPhone());
        mgRnrInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
		mgRnrInfoDTO.setEffectiveDate(DateUtils.parseDate(dto.getCorporationCertEffectiveDate(),"yyyy-MM-dd"));
        mgRnrInfoDTO.setExpiredDate(dto.getCorporationCertExpirationDate());
        //mgRnrInfoDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrInfoDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setIsCompany(1);
        mgRnrInfoDTO.setRnrCompanyId(uuid);
        mgRnrInfoDTO.setRnrStatus(0);
        mgRnrInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
        return mgRnrInfoDTO;
    }


    private List<MgRnrFileDTO> getMgRnrFileDTOs(EnterpriseRnrCertificationInfoDTO requestDTO, RnrRelationDTO rnrRelationDTO, String uuid, String tenNo, String companyId) {
        List<MgRnrFileDTO> mgRnrFileDTOS = new ArrayList<>();
        //企业实名认证授权书
        List<String> authorizationLetterPic = requestDTO.getAuthorizationLetterPic();
        for (int i = 0; i < authorizationLetterPic.size(); i++) {
            mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO, authorizationLetterPic.get(i), RnrFileType.ENTERPRISE_AUTH_FILE.getCode(), "0", uuid, tenNo, companyId));
        }

        //企业证件照
        List<String> licenseImages = requestDTO.getLicenseImages();
        for (int i = 0; i < licenseImages.size(); i++) {
            mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO, licenseImages.get(i), RnrFileType.ENTERPRISE_PIC.getCode(), "0", uuid, tenNo, companyId));
        }
        //企业照片
        //String enterprisePic = requestDTO.getLicenseImages();
        //mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO,enterprisePic,RnrFileType.ENTERPRISE_PIC.getCode(),"0",uuid,tenNo));
        //责任人证件照
        List<String> corporationPhotoPic = requestDTO.getCorporationPhotoPic();
        for (int i = 0; i < corporationPhotoPic.size(); i++) {
            int index = 0;
            if (i != 0) {
                index = 1;
            }
            //@皮豆 com.cusc.nirvana.user.rnr.enterprise.service.impl.EnterpriseRnrServiceImpl#getFileType
            //这个方法直接用 com.cusc.nirvana.user.rnr.enterprise.util.FileUtil#getFileType这个方法替换就好了
            mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO, corporationPhotoPic.get(i), FileUtil.getFileType(requestDTO.getCorporationCertType(), index), "0", uuid, tenNo, companyId));
        }
        //String corporationCertFrontPic = corporationPhotoPic.get(0);
        //mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO,corporationCertFrontPic,getFileType(requestDTO.getCorporationCertType(),1),"0",uuid,tenNo));
        //String corporationCertBackPic = corporationPhotoPic.get(1);
        //mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO,corporationCertBackPic,getFileType(requestDTO.getCorporationCertType(),0),"0",uuid,tenNo));
        //活体视频
        if (StringUtils.isNotBlank(requestDTO.getLiveVerificationVideo())) {
            mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO, requestDTO.getLiveVerificationVideo(), RnrFileType.LIVENESS_VIDEO.getCode(), "0", uuid, tenNo, companyId));
        }

        //责任告知书
        if (!CollectionUtils.isEmpty(requestDTO.getDutyPic())) {
            for (String duty : requestDTO.getDutyPic()) {
                mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO, duty, RnrFileType.DUTY_FILE.getCode(), "0", uuid, tenNo, companyId));
            }
        }

        //入网合同
        if (!CollectionUtils.isEmpty(requestDTO.getContractPic())) {
            for (String contact : requestDTO.getContractPic()) {
                mgRnrFileDTOS.add(newFileDTO(rnrRelationDTO, contact, RnrFileType.VEHUCLE_BIND.getCode(), "0", uuid, tenNo, companyId));
            }
        }


        return mgRnrFileDTOS;
    }

    private MgRnrFileDTO newFileDTO(RnrRelationDTO rnrRelationDTO, String pic, Integer fileType, String liasionId, String uuid, String tenNo, String companyId) {
        MgRnrFileDTO mgRnrFileDTO = new MgRnrFileDTO();
        mgRnrFileDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrFileDTO.setRnrId(uuid);
        mgRnrFileDTO.setFileType(fileType);
        mgRnrFileDTO.setFileSystemId(pic);
        mgRnrFileDTO.setTenantNo(tenNo);
        mgRnrFileDTO.setLiaisonId(liasionId);
        mgRnrFileDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrFileDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrFileDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
        mgRnrFileDTO.setTenantNo(tenNo);
        mgRnrFileDTO.setRnrCompanyId(companyId);
        return mgRnrFileDTO;
    }

    /**
     * 获取文件类型
     *
     * @param certType 证书类型
     * @param index    索引
     */
    private Integer getFileType(String certType, int index) {
        CertTypeEnum certTypeEnum = Arrays.stream(CertTypeEnum.values())
                .filter(c -> StringUtils.equalsIgnoreCase(c.getCode(), certType))
                .findFirst().get();
        switch (certTypeEnum) {
            case IDCARD:
                if (index == 0) return RnrFileType.IDENTITY_CARD_BACK.getCode();
                if (index == 1) return RnrFileType.IDENTITY_CARD_FRONT.getCode();
                return null;
            case PLA:
                if (index == 0) return RnrFileType.OFFICIAL_CARD_FRONT.getCode();
                if (index == 1) return RnrFileType.OFFICIAL_CARD_BACK.getCode();
            case HKIDCARD:
                if (index == 0) return RnrFileType.HK_MACAO_PASSPORT_FRONT.getCode();
                if (index == 1) return RnrFileType.HK_MACAO_PASSPORT_BACK.getCode();
            case TAIBAOZHENG:
                if (index == 0) return RnrFileType.TAIWAN_CARD_FRONT.getCode();
                if (index == 1) return RnrFileType.TAIWAN_CARD_BACK.getCode();
            case PASSPORT:
                if (index == 0) return RnrFileType.FOREIGN_NATIONAL_PASSPORT_FRONT.getCode();
                if (index == 1) return RnrFileType.FOREIGN_NATIONAL_PASSPORT_BACK.getCode();
            case POLICEPAPER:
                if (index == 0) return RnrFileType.POLICE_CARD_FRONT.getCode();
                if (index == 1) return RnrFileType.POLICE_CARD_BACK.getCode();
            case HKRESIDENCECARD:
                if (index == 0) return RnrFileType.HK_MACAO_RESIDENCE_PERMIT_FRONT.getCode();
                if (index == 1) return RnrFileType.HK_MACAO_RESIDENCE_PERMIT_BACK.getCode();
            case TWRESIDENCECARD:
                if (index == 0) return RnrFileType.TAIWAN_RESIDENCE_PERMIT_FRONT.getCode();
                if (index == 1) return RnrFileType.TAIWAN_RESIDENCE_PERMIT_BACK.getCode();
            case RESIDENCE:
                if (index == 0) return RnrFileType.RESIDENCE_BOOKLET_FRONT.getCode();
                if (index == 1) return RnrFileType.RESIDENCE_BOOKLET_BACK.getCode();
            default:
                return null;
        }
    }


    @Override
    public EnterpriseRnrH5RespDTO submitRnrH5(EnterpriseRnrH5RequestDTO dto) {
    	if (dto.getVinList().stream().distinct().count() != dto.getVinList().size()) {
    		throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"请不要输入重复的vin");
		}


        //检查短信验证码
         SmsRequestDTO smsRequestDTO = new SmsRequestDTO();
         smsRequestDTO.setPhone(dto.getCorporationPhone());
         smsRequestDTO.setTenantNo(UserSubjectUtil.getTenantNo());
         smsRequestDTO.setBizType(BizTypeEnum.RNR.getCode());
         smsRequestDTO.setCaptcha(dto.getVerificationCode());
         Response smsResponse = smsService.checkSmsCaptcha(smsRequestDTO);
         if (!smsResponse.isSuccess()) {
             throw new CuscUserException(smsResponse.getCode(), smsResponse.getMsg());
         }

        EnterpriseRnrCertificationInfoDTO certificationInfoDTO = new EnterpriseRnrCertificationInfoDTO();
        BeanUtils.copyProperties(dto, certificationInfoDTO);

        //构建relationDto
        String rnrId = CuscStringUtils.generateUuid();
        String tenantNo = UserSubjectUtil.getTenantNo();

        String orgId = userService.getOrganId(UserSubjectUtil.getUserId(), tenantNo);

        //公司信息
        MgRnrCompanyInfoDTO mgRnrCompanyInfoDTO = createCompanyInfoDTO(certificationInfoDTO, rnrId, tenantNo);

        //rnrInfo
        MgRnrInfoDTO rnrInfo = createRnrInfo(certificationInfoDTO, rnrId, tenantNo, mgRnrCompanyInfoDTO.getUuid());
        rnrInfo.setIsTrust(0);
        rnrInfo.setOrgId(orgId);

        //rnrOrder
        RnrOrderDTO rnrOrderDTO = createOrder(certificationInfoDTO, rnrInfo, rnrInfo.getUuid(), tenantNo, RnrOrderType.COMPANY_NEW_VEHICLE.getCode());
        rnrOrderDTO.setOrgId(orgId);

        //文件列表
        List<MgRnrFileDTO> fileDTOList = getMgRnrFileDTOs(certificationInfoDTO, null, rnrId, tenantNo, mgRnrCompanyInfoDTO.getUuid());

        //卡列表
        List<String> vinList = dto.getVinList();
        List<MgRnrCardInfoDTO> cardInfoDTOList = createCardInfoDtoByVinList(vinList, tenantNo, rnrId, rnrOrderDTO.getUuid());

        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();
        rnrRelationDTO.setCompanyInfo(mgRnrCompanyInfoDTO);
        rnrRelationDTO.setInfo(rnrInfo);
        rnrRelationDTO.setCardList(cardInfoDTOList);
        rnrRelationDTO.setOrder(rnrOrderDTO);
        rnrRelationDTO.setRnrFileList(fileDTOList);

        rnrRelationDTO.getOrder().setOrderSource(RnrOrderSourceEnum.H5.getCode());

        Response response = enterpriseClient.submitH5(rnrRelationDTO);
        if (!response.isSuccess()) {
            throw new CuscUserException(response.getCode(), response.getMsg());
        }

        EnterpriseRnrH5RespDTO rnrH5RespDTO = new EnterpriseRnrH5RespDTO();


        LiveLoginReqDTO loginUrlDto = new LiveLoginReqDTO();
        loginUrlDto.setOrderNo(rnrOrderDTO.getUuid());
        if ("PAD".equals(dto.getSource())) {
			loginUrlDto.setCallBackUrl(frontEnterprisePadCallBackUrl);
		} else {
			loginUrlDto.setCallBackUrl(frontEnterpriseH5CallBackUrl);
		}

        //调用云端接口
        Response<LiveLoginRespDTO> h5LiveLoginUrl = fpRnrClient.h5LiveLoginUrl(loginUrlDto);
        if (h5LiveLoginUrl.isSuccess()) {
            rnrH5RespDTO.setH5LivenessUrl(h5LiveLoginUrl.getData().getH5LiveLoginUrl());
        }
        rnrH5RespDTO.setRnrId(rnrId);
        rnrH5RespDTO.setOrderId(rnrOrderDTO.getUuid());
        return rnrH5RespDTO;
    }

    @Override
    public EnterpriseRnrH5CallBackRespDTO afreshLivenessUrl(LivenessCallbackReqDTO bean) {
        EnterpriseRnrH5CallBackRespDTO respDTO = new EnterpriseRnrH5CallBackRespDTO();
        boolean success = bean.getCode().equals(SUCCESS_CODE);
        if (success) {
            //todo 调用云端接口
            LiveLoginReqDTO liveLoginReqDTO = new LiveLoginReqDTO();
            liveLoginReqDTO.setOrderNo(bean.getOrderNo());
            Response<LivenessResultDTO> livenessResult = fpRnrClient.h5LivenessResult(liveLoginReqDTO);

            if (null == livenessResult || !livenessResult.isSuccess() || null == livenessResult.getData()) {
                //没有查询到 活体视屏 和 照片 再次拉起腾讯活体H5
                LiveLoginReqDTO loginUrlDto = new LiveLoginReqDTO();
                loginUrlDto.setOrderNo(bean.getOrderNo());
                loginUrlDto.setCallBackUrl(frontEnterpriseH5CallBackUrl);
                // 调用云端接口
                Response<LiveLoginRespDTO> h5LiveLoginUrl = fpRnrClient.h5LiveLoginUrl(loginUrlDto);
                respDTO.setStatus(2);
                if (h5LiveLoginUrl.isSuccess()) {
                    respDTO.setH5LivenessUrl(h5LiveLoginUrl.getData().getH5LiveLoginUrl());
                }
            } else {
                LivenessResultDTO livenessResultData = livenessResult.getData();
                EnterpriseH5CallBackRequestDTO callBackDto = new EnterpriseH5CallBackRequestDTO();
                callBackDto.setOrderNo(bean.getOrderNo());
                String[] livenessPhotos = new String[2];
                livenessPhotos[0] = livenessResultData.getPhotoId();
                livenessPhotos[1] = livenessResultData.getPhotoId();
                callBackDto.setLivenessPicFileId(livenessPhotos);
                callBackDto.setVideFileId(livenessResultData.getVideoFileId());
                Response<Integer> response = enterpriseClient.enterpriseRnrH5CallBack(callBackDto);
                if (!response.isSuccess() || null == response.getData()) {
                    throw new CuscUserException(response.getCode(), response.getMsg());
                }
                Integer data = response.getData();
                respDTO.setStatus(data);
            }
        } else {
            LiveLoginReqDTO loginUrlDto = new LiveLoginReqDTO();
            loginUrlDto.setOrderNo(bean.getOrderNo());
            loginUrlDto.setCallBackUrl(frontEnterpriseH5CallBackUrl);

            //调用云端接口
            Response<LiveLoginRespDTO> h5LiveLoginUrl = fpRnrClient.h5LiveLoginUrl(loginUrlDto);
            if (h5LiveLoginUrl.isSuccess()) {
                respDTO.setH5LivenessUrl(h5LiveLoginUrl.getData().getH5LiveLoginUrl());
            }
            respDTO.setStatus(2);

            //需要修改数据状态
            //RnrOrderDTO orderDTO = new RnrOrderDTO();
            //orderDTO.setUuid(bean.getOrderNo());
            //fpRnrClient.updateRnrInValid(orderDTO);
        }

        return respDTO;
    }


	private List<MgRnrCardInfoDTO> createCardInfoDtoByVinList(List<String> vinList, String tenantNo, String rnrId, String orderId) {
        List<MgRnrCardInfoDTO> cardInfoDTOList = Collections.synchronizedList(new ArrayList<>());
        String creator = UserSubjectUtil.getUserId();
        String operator = UserSubjectUtil.getUserId();
        vinList.parallelStream().forEach(vin -> {
            VinCardQueryDTO dto = new VinCardQueryDTO();
            dto.setVin(vin);
            Response<VinCardResultDTO> vinCardResultResponse = checkClient.queryUnBindCardByVin(dto);
            log.info("createCardInfoDtoByVinList vin = {},reponse = {}", vin, JSONObject.toJSONString(vinCardResultResponse));
            if (null == vinCardResultResponse || !vinCardResultResponse.isSuccess()
                    || null == vinCardResultResponse.getData()
                    || CollectionUtils.isEmpty(vinCardResultResponse.getData().getIccidList())) {
                throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(), "未查询到卡信息");
            }

            List<String> iccidList = vinCardResultResponse.getData().getIccidList();

            for (String iccid : iccidList) {
                MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
                mgRnrCardInfoDTO.setUuid(CuscStringUtils.generateUuid());
                mgRnrCardInfoDTO.setIccid(iccid);
                mgRnrCardInfoDTO.setTenantNo(tenantNo);
                mgRnrCardInfoDTO.setIotId(vin);
                mgRnrCardInfoDTO.setRnrId(rnrId);
                mgRnrCardInfoDTO.setOrderId(orderId);
                mgRnrCardInfoDTO.setCreator(creator);
                mgRnrCardInfoDTO.setOperator(operator);
                mgRnrCardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
                cardInfoDTOList.add(mgRnrCardInfoDTO);
            }
        });
        return cardInfoDTOList;
    }

	@Override
	public EnterpriseRnrH5RespDTO submitRnrWithFile(EnterpriseRnrH5FileRequestDTO dto) {
		//检查短信验证码
		SmsRequestDTO smsRequestDTO = new SmsRequestDTO();
		smsRequestDTO.setPhone(dto.getCorporationPhone());
		smsRequestDTO.setTenantNo(UserSubjectUtil.getTenantNo());
		smsRequestDTO.setBizType(BizTypeEnum.RNR.getCode());
		smsRequestDTO.setCaptcha(dto.getVerificationCode());
		Response smsResponse = smsService.checkSmsCaptcha(smsRequestDTO);
		if (!smsResponse.isSuccess()) {
			throw new CuscUserException(smsResponse.getCode(), smsResponse.getMsg());
		}

		//验证车卡关系
		Response<List<VinCardDTO>> listResponse = vehicleCardService.checkIccidList(dto.getFileId());
		log.info("验证车卡关系", dto);
		if (!listResponse.isSuccess()) {
			throw new CuscUserException(listResponse.getCode(),listResponse.getMsg());
		}

		EnterpriseRnrCertificationInfoDTO requestDto = new EnterpriseRnrCertificationInfoDTO();
		BeanUtils.copyProperties(dto,requestDto);
		RnrRelationDTO rnrRelationDTO = turnParams(requestDto, listResponse.getData());
		rnrRelationDTO.getOrder().setOrderSource(RnrOrderSourceEnum.H5.getCode());

		Response response = enterpriseClient.submitH5(rnrRelationDTO);
		if (!response.isSuccess()) {
			throw new CuscUserException(response.getCode(), response.getMsg());
		}

		EnterpriseRnrH5RespDTO rnrH5RespDTO = new EnterpriseRnrH5RespDTO();


		LiveLoginReqDTO loginUrlDto = new LiveLoginReqDTO();
		loginUrlDto.setOrderNo(rnrRelationDTO.getOrder().getUuid());
		if ("PAD".equals(dto.getSource())) {
			loginUrlDto.setCallBackUrl(frontEnterprisePadCallBackUrl);
		} else {
			loginUrlDto.setCallBackUrl(frontEnterpriseH5CallBackUrl);
		}

		//调用云端接口
		Response<LiveLoginRespDTO> h5LiveLoginUrl = fpRnrClient.h5LiveLoginUrl(loginUrlDto);
		if (h5LiveLoginUrl.isSuccess()) {
			rnrH5RespDTO.setH5LivenessUrl(h5LiveLoginUrl.getData().getH5LiveLoginUrl());
		}
		rnrH5RespDTO.setRnrId(rnrRelationDTO.getInfo().getUuid());
		rnrH5RespDTO.setOrderId(rnrRelationDTO.getOrder().getUuid());
		return rnrH5RespDTO;
	}
}
