package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.constants.RnrStatusEnum;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseUnBindDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VinCheckDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VinCheckResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.EnterpriseUnBindService;
import com.cusc.nirvana.user.rnr.fp.client.EnterpriseRnrClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.RnrOrderClient;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * @className: EnterpriseUnBindServiceImpl
 * @description: 企业解绑
 * @author: jk
 * @date: 2022/6/16 18:23
 * @version: 1.0
 **/
@Service
@Slf4j
public class EnterpriseUnBindServiceImpl implements EnterpriseUnBindService {

    @Resource
    private MgRnrCardInfoClient mgRnrCardInfoClient;

    @Resource
    private MgRnrInfoClient mgRnrInfoClient;

    @Resource
    private RnrOrderClient rnrOrderClient;

    @Resource
    private EnterpriseRnrClient enterpriseRnrClient;


    /**
     * @author: jk
     * @description: 参数解绑校验
     * @date: 2022/6/16 18:26
     */
    @Override
    public VinCheckResponseDTO enterpriseUnBindCheck(EnterpriseUnBindDTO dto) {

        VinCheckResponseDTO vinCheckResponseDTO = new VinCheckResponseDTO();
        if (!CollectionUtils.isEmpty(dto.getVinList()) && dto.getVinList().stream().distinct().count() != dto.getVinList().size()) {
            throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(), "请不要输入同样的Vin");
        }
        MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
        mgRnrCardInfoDTO.setVinList(dto.getVinList());
        //根据vin查询VIN是否存在且实名
        Response<List<MgRnrCardInfoDTO>> responseList = mgRnrCardInfoClient.queryBindListByVinsAndIccids(mgRnrCardInfoDTO);
        if (!responseList.isSuccess()) {
            throw new CuscUserException(responseList.getCode(), responseList.getMsg());
        }
        if(CollectionUtils.isEmpty(responseList.getData())){
            throw new CuscUserException("", "未查到实名数据!");

        }
        List<MgRnrCardInfoDTO> mgRnrCardInfoDTOList = responseList.getData();

		boolean hasInitCard = mgRnrCardInfoDTOList.stream().anyMatch(card -> RnrStatus.INIT.getCode() == card.getRnrStatus());
		if (hasInitCard) {
			throw new CuscUserException("", "存在未完成的工单信息");
		}

		//转为map
        Map<String, MgRnrCardInfoDTO> mgRnrCardInfoMap = mgRnrCardInfoDTOList.stream().collect(Collectors.toMap(MgRnrCardInfoDTO::getIotId, Function.identity(), (key1, key2) -> key2));

        List<String> vinList = dto.getVinList();

        //过滤出参数是否都存在
        List<VinCheckDTO> vinCheckDTOMap = vinList.stream().map(a -> {
            VinCheckDTO vinCheckDetailDTOs = new VinCheckDTO();
            if (null == mgRnrCardInfoMap.get(a)) {
                vinCheckDetailDTOs.setVin(a);
                vinCheckDetailDTOs.setCheckResult(false);
                vinCheckDetailDTOs.setErrorMsg("VIN不存在实名卡");
            }
            return vinCheckDetailDTOs;
        }).filter(b-> null !=b && !StringUtils.isEmpty(b.getVin())).collect(Collectors.toList());


        if (!CollectionUtils.isEmpty(vinCheckDTOMap)) {
            vinCheckResponseDTO.setList(vinCheckDTOMap);
            return vinCheckResponseDTO;
        }


//            List<MgRnrCardInfoDTO> distinctList = mgRnrCardInfoDTOList.stream().filter(distinctByKey(p->p.getRnrId())).collect(Collectors.toList());

//          //获取持卡人集合去重
        List<String> rnrList = mgRnrCardInfoDTOList.stream().distinct().map(MgRnrCardInfoDTO::getRnrId).collect(Collectors.toList());

        MgRnrInfoDTO mgRnrInfoDTO = new MgRnrInfoDTO();
        mgRnrInfoDTO.setUuidList(rnrList);
        //查询info信息
        Response<List<MgRnrInfoDTO>> mgRnrInfoList = mgRnrInfoClient.queryByList(mgRnrInfoDTO);

        if (!mgRnrInfoList.isSuccess()) {
            throw new CuscUserException(mgRnrInfoList.getCode(), mgRnrInfoList.getMsg());
        }
        List<MgRnrInfoDTO> mgRnrInfoDTOList = mgRnrInfoList.getData();

        //根据姓名+身份证号码分组
        Map<String, List<MgRnrInfoDTO>> rnrInfoMap = mgRnrInfoDTOList.stream().collect(Collectors.groupingBy(this::getGroupKey));


        //如果map大于1，说明VIN属于不同联系人
        if (null != rnrInfoMap && rnrInfoMap.size() > 1) {
            List<VinCheckDTO> vinCheckDTOS = vinList.stream().map(a -> {
                VinCheckDTO vinCheckDetailDTO = new VinCheckDTO();
                vinCheckDetailDTO.setVin(a);
                vinCheckDetailDTO.setCheckResult(false);
                vinCheckDetailDTO.setErrorMsg("VIN下卡不是同一个人的");
                return vinCheckDetailDTO;
            }).filter(a->!StringUtils.isEmpty(a.getVin())).collect(Collectors.toList());

            if (!CollectionUtils.isEmpty(vinCheckDTOS)) {
                vinCheckResponseDTO.setList(vinCheckDTOS);
                return vinCheckResponseDTO;
            }
        }


        //表示校验通过了
        List<VinCheckDTO> vinCheckDTOS = vinList.stream().map(a -> {
            VinCheckDTO vinCheckDetailDTO = new VinCheckDTO();
            vinCheckDetailDTO.setVin(a);
            vinCheckDetailDTO.setCheckResult(true);
            return vinCheckDetailDTO;
        }).filter(a->!StringUtils.isEmpty(a.getVin())).collect(Collectors.toList());

        vinCheckResponseDTO.setList(vinCheckDTOS);

        vinCheckResponseDTO.setPhone(mgRnrInfoList.getData().get(0).getPhone());
        vinCheckResponseDTO.setUserName(mgRnrInfoList.getData().get(0).getFullName());
        return vinCheckResponseDTO;
    }

    /**
     * @author: jk
     * @description: 提交解绑信息
     * @date: 2022/6/17 10:44
     * @version: 1.0
     */
    @Override
    public Response submitUnBind(EnterpriseUnBindDTO dto) {
        VinCheckResponseDTO response = this.enterpriseUnBindCheck(dto);
        List<VinCheckDTO> list = response.getList();
        if (!CollectionUtils.isEmpty(list)) {
            for (VinCheckDTO VinCheckDTO : list) {
                if (!VinCheckDTO.getCheckResult()) {
                    throw new CuscUserException("", VinCheckDTO.getErrorMsg());
                }
            }
        }
//        if(!response.isSuccess()){
//            throw new CuscUserException(response.getCode(),response.getMsg());
//        }

        Response<RnrRelationDTO> rnrRelationDTOResponse = convertToRelationDTO(dto);

        if (!rnrRelationDTOResponse.isSuccess()) {
            throw new CuscUserException(rnrRelationDTOResponse.getCode(), rnrRelationDTOResponse.getMsg());
        }
        Response enterpriseFpResponse = enterpriseRnrClient.enterpriseUnBind(rnrRelationDTOResponse.getData());
        if (!enterpriseFpResponse.isSuccess()) {
            throw new CuscUserException(enterpriseFpResponse.getCode(), enterpriseFpResponse.getMsg());
        }

        dto.setOrderId(rnrRelationDTOResponse.getData().getOrder().getUuid());
        return Response.createSuccess("",dto);
    }

    //组装信息
    private Response<RnrRelationDTO> convertToRelationDTO(EnterpriseUnBindDTO dto) {

        String rnrID = CuscStringUtils.generateUuid();

        String orderID = CuscStringUtils.generateUuid();

        String carID = CuscStringUtils.generateUuid();

        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();

        //查询卡信息
        MgRnrCardInfoDTO mgRnrCardInfo = new MgRnrCardInfoDTO();
        mgRnrCardInfo.setVinList(dto.getVinList());
        Response<List<MgRnrCardInfoDTO>> mgRnrCardInfoList = mgRnrCardInfoClient.queryBindListByVinsAndIccids(mgRnrCardInfo);
        if (!mgRnrCardInfoList.isSuccess()) {
            throw new CuscUserException(mgRnrCardInfoList.getCode(), mgRnrCardInfoList.getMsg());
        }

        if(CollectionUtils.isEmpty(mgRnrCardInfoList.getData())){
            throw new CuscUserException(com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode.RNR_CAR_INFO_IS_NULL.getCode(), com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode.RNR_CAR_INFO_IS_NULL.getMsg());
        }
        List<MgRnrCardInfoDTO> mgRnrCardInfoDTOList = mgRnrCardInfoList.getData();

        //获取rnrID集合
        List<String> rnrIdList = mgRnrCardInfoDTOList.stream().map(MgRnrCardInfoDTO::getRnrId).collect(Collectors.toList());
        //获取orderId 集合
        List<String> orderIdList = mgRnrCardInfoDTOList.stream().map(MgRnrCardInfoDTO::getOrderId).collect(Collectors.toList());

        //info信息
        MgRnrInfoDTO mgRnrInfoDTO = getMgRnrInfoDTO(rnrID, rnrIdList);

        rnrRelationDTO.setInfo(mgRnrInfoDTO);

        //工单信息
        RnrOrderDTO rnrOrderDTO = getMgRnrOrderDTO(orderID, rnrID, orderIdList);

        rnrRelationDTO.setOrder(rnrOrderDTO);

        //carInfo
        List<MgRnrCardInfoDTO> mgRnrCardInfoArray = getMgRnrCardDTOs(mgRnrCardInfoDTOList, rnrRelationDTO);

        rnrRelationDTO.setCardList(mgRnrCardInfoArray);

        rnrRelationDTO.setCarNumber(dto.getVinList().size());

        return Response.createSuccess(rnrRelationDTO);
    }

    /**
     * 获取实名主体信息
     *
     * @param rnrIdList 请求消息
     */
    private MgRnrInfoDTO getMgRnrInfoDTO(String rnrID, List<String> rnrIdList) {
        MgRnrInfoDTO mgRnrInfoDTO = new MgRnrInfoDTO();
        mgRnrInfoDTO.setUuidList(rnrIdList);
        mgRnrInfoDTO.setRnrStatus(RnrStatusEnum.REAL_NAME.getCode());
        //查询info
        Response<List<MgRnrInfoDTO>> mgRnrInfoList = mgRnrInfoClient.queryByList(mgRnrInfoDTO);
        if (!mgRnrInfoList.isSuccess()) {
            throw new CuscUserException(mgRnrInfoList.getCode(), mgRnrInfoList.getMsg());
        }
        if(CollectionUtils.isEmpty(mgRnrInfoList.getData())){
            throw new CuscUserException( com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode.RNR_INFO_IS_NULL.getCode(), com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode.RNR_INFO_IS_NULL.getMsg());
        }

        //获取info信息
        MgRnrInfoDTO MgRnrInfoDTOList = mgRnrInfoList.getData().get(0);

        MgRnrInfoDTOList.setUuid(rnrID);
        MgRnrInfoDTOList.setRnrStatus(RnrStatusEnum.NO_REAL_NAME.getCode());
        MgRnrInfoDTOList.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        MgRnrInfoDTOList.setCreator(UserSubjectUtil.getUserId());
        MgRnrInfoDTOList.setOperator(UserSubjectUtil.getUserId());
        MgRnrInfoDTOList.setCreateTime(null);
        MgRnrInfoDTOList.setUpdateTime(null);

        return MgRnrInfoDTOList;

    }

    /**
     * 创建工单信息
     *
     * @param orderID
     * @param orderIdList
     */
    private RnrOrderDTO getMgRnrOrderDTO(String orderID, String rnrId, List<String> orderIdList) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(orderIdList.get(0));
        Response<RnrOrderDTO> rnrOrderResponse = rnrOrderClient.getByUuid(rnrOrderDTO);
        if (!rnrOrderResponse.isSuccess()) {
            throw new CuscUserException(rnrOrderResponse.getCode(), rnrOrderResponse.getMsg());
        }
        if(null == rnrOrderResponse.getData()){
            throw new CuscUserException(com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode.RNR_ORDER_IS_NULL.getCode(), com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode.RNR_ORDER_IS_NULL.getMsg());
        }
        RnrOrderDTO rnrOrderResponseData = rnrOrderResponse.getData();
        rnrOrderResponseData.setUuid(orderID);
        rnrOrderResponseData.setOrderStatus(RnrOrderStatusEnum.TO_EXAMINE.getCode());
        //新增类型，企业解绑
        rnrOrderResponseData.setOrderType(RnrOrderType.ENTERPRISEUNBIND.getCode());
        rnrOrderResponseData.setCreator(UserSubjectUtil.getUserId());
        rnrOrderResponseData.setOperator(UserSubjectUtil.getUserId());
        rnrOrderResponseData.setRnrId(rnrId);
        rnrOrderResponseData.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        rnrOrderResponseData.setCreateTime(null);
        rnrOrderResponseData.setUpdateTime(null);
        return rnrOrderResponseData;
    }

    /**
     * 获取实名卡信息
     *
     * @param mgRnrCardInfoDTOList 请求消息
     */
    private List<MgRnrCardInfoDTO> getMgRnrCardDTOs(List<MgRnrCardInfoDTO> mgRnrCardInfoDTOList, RnrRelationDTO rnrRelationDTO) {

        String carID = CuscStringUtils.generateUuid();

        for (MgRnrCardInfoDTO mgRnrCardInfoDTO : mgRnrCardInfoDTOList) {
            String oldCarId = mgRnrCardInfoDTO.getUuid();
            mgRnrCardInfoDTO.setUuid(carID);
            mgRnrCardInfoDTO.setRnrStatus(Integer.valueOf(RnrStatusEnum.NO_REAL_NAME.getCode()));
            mgRnrCardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
            mgRnrCardInfoDTO.setOldCardId(oldCarId);
            mgRnrCardInfoDTO.setCreator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setRnrId(rnrRelationDTO.getInfo().getUuid());
            mgRnrCardInfoDTO.setOrderId(rnrRelationDTO.getOrder().getUuid());
            mgRnrCardInfoDTO.setCreateTime(null);
            mgRnrCardInfoDTO.setUpdateTime(null);
            mgRnrCardInfoDTO.setOrderId(UserSubjectUtil.getUserId());
        }
        return mgRnrCardInfoDTOList;
    }

    //私有方法
    private String getGroupKey(MgRnrInfoDTO vo) {
        return vo.getFullName() + vo.getCertNumber();
    }

    public static <T> Predicate<T> distinctByKey(Function<? super T, Object> keyExtractor) {
        Map<Object, Boolean> seen = new ConcurrentHashMap<>();
        System.out.println("这个函数将应用到每个item");
        return t -> seen.putIfAbsent(keyExtractor.apply(t), Boolean.TRUE) == null;
    }

}
