package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.client.UserClient;
import com.cusc.nirvana.user.eiam.dto.UserDTO;
import com.cusc.nirvana.user.eiam.dto.UserSimpleDTO;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.dto.LogCategoryDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LogInfoRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LogInfoResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.ILogService;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import com.cusc.nirvana.user.rnr.fp.client.FpUserOperationLogClient;
import com.cusc.nirvana.user.rnr.fp.dto.FpUserOperationLogDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author stayAnd
 * @date 2022/7/4
 */
@Service
public class LogServiceImpl implements ILogService {

	@Resource
	private FpUserOperationLogClient logClient;

	@Resource
	private UserClient userClient;

	@Override
	public LogCategoryDTO category() {
		LogCategoryDTO logCategoryDTO = new LogCategoryDTO();
		Map<String, List<SysModuleEnum>> sysLogMap = Arrays.stream(SysModuleEnum.values()).collect(Collectors.groupingBy(SysModuleEnum::getSysModule));

		List<LogCategoryDTO.LogCategoryDetailDTO> detailDTOList = new ArrayList<>();
		sysLogMap.forEach((key,vlaue)->{
			LogCategoryDTO.LogCategoryDetailDTO detailDTO = new LogCategoryDTO.LogCategoryDetailDTO();
			List<String> optList = vlaue.stream().map(SysModuleEnum::getSysAction).collect(Collectors.toList());
			detailDTO.setLogCategory(key);
			detailDTO.setLogOptList(optList);
			detailDTOList.add(detailDTO);
		});
		logCategoryDTO.setLogDetailList(detailDTOList);
		return logCategoryDTO;
	}

	@Override
	public PageResult<LogInfoResponseDTO> list(LogInfoRequestDTO requestDTO) {
		PageResult<LogInfoResponseDTO> result = new PageResult<>();

		FpUserOperationLogDTO queryDto = new FpUserOperationLogDTO();
		String accountUserId = null;
		if (CuscStringUtils.isNotEmpty(requestDTO.getAccount())) {
			UserDTO query = new UserDTO();
			query.setTenantNo(UserSubjectUtil.getTenantNo());
			query.setApplicationId(UserSubjectUtil.getAppId());
			query.setUserName(requestDTO.getAccount());
			UserDTO userDTO = userClient.getByUserName(query).getData();
			if (null == userDTO) {
				result.setList(Collections.emptyList());
				result.setTotalCount(0);
				return  result;
			}
			accountUserId = userDTO.getUuid();
			queryDto.setCreator(accountUserId);
		}

		String phoneUserId = null;
		if (CuscStringUtils.isNotEmpty(requestDTO.getPhone())) {
			UserDTO query = new UserDTO();
			query.setTenantNo(UserSubjectUtil.getTenantNo());
			query.setApplicationId(UserSubjectUtil.getAppId());
			query.setPhone(requestDTO.getPhone());
			UserDTO userDTO = userClient.getByPhone(query).getData();
			if (null == userDTO) {
				result.setList(Collections.emptyList());
				result.setTotalCount(0);
				return  result;
			}
			phoneUserId = userDTO.getUuid();
			queryDto.setCreator(phoneUserId);
		}


		if (CuscStringUtils.isNotEmpty(accountUserId) && CuscStringUtils.isNotEmpty(phoneUserId) && !accountUserId.equals(phoneUserId)) {
			result.setList(Collections.emptyList());
			result.setTotalCount(0);
			return  result;
		}

		if (CuscStringUtils.isEmpty(requestDTO.getOrgId())) {
			queryDto.setOrgId(UserSubjectUtil.getOrganId());
		} else {
			queryDto.setOrgId(requestDTO.getOrgId());
		}

		if (CuscStringUtils.isNotEmpty(requestDTO.getLogCategory())){
			queryDto.setOptModule(requestDTO.getLogCategory());
		}

		if (CuscStringUtils.isNotEmpty(requestDTO.getLogOpt())) {
			queryDto.setOptAction(requestDTO.getLogOpt());
		}

		if (null != requestDTO.getCreateTimeStart()) {
			queryDto.setCreateTimeQueryStart(requestDTO.getCreateTimeStart());
		}

		if (null != requestDTO.getCreateTimeEnd()) {
			queryDto.setCreateTimeQueryStartEnd(requestDTO.getCreateTimeEnd());
		}

		queryDto.setCurrPage(requestDTO.getCurrPage());
		queryDto.setPageSize(requestDTO.getPageSize());
		PageResult<FpUserOperationLogDTO> response = logClient.queryByPage(queryDto).getData();
		List<FpUserOperationLogDTO> dataList = response.getList();
		if (CollectionUtils.isEmpty(dataList)) {
			result.setList(Collections.emptyList());
			result.setTotalCount(0);
			return  result;
		}

		result.setTotalCount(response.getTotalCount());

		List<String> userIdList = dataList.stream().map(FpUserOperationLogDTO::getCreator).collect(Collectors.toList());
		UserDTO uuidQuery = new UserDTO();
		uuidQuery.setTenantNo(UserSubjectUtil.getTenantNo());
		uuidQuery.setApplicationId(UserSubjectUtil.getAppId());
		uuidQuery.setUuidList(userIdList);
		List<UserSimpleDTO> userList = userClient.getByUuids(uuidQuery).getData();
		Map<String, UserSimpleDTO> userMap = userList.stream().collect(Collectors.toMap(UserSimpleDTO::getUuid, Function.identity(), (o1, o2) -> o2));

		List<LogInfoResponseDTO> dtoList = dataList.stream().map(log -> {
			LogInfoResponseDTO logInfoResponseDTO = new LogInfoResponseDTO();
			logInfoResponseDTO.setLogCategory(log.getOptModule());
			logInfoResponseDTO.setLogOpt(log.getOptAction());
			logInfoResponseDTO.setLogContent(log.getOptContent());
			logInfoResponseDTO.setIp(log.getRequestIp());
			logInfoResponseDTO.setCreateTime(log.getCreateTime());
			UserSimpleDTO userSimpleDTO = userMap.get(log.getCreator());
			logInfoResponseDTO.setAccount(null != userSimpleDTO ? userSimpleDTO.getUserName() : "");
			logInfoResponseDTO.setPhone(null != userSimpleDTO ? DesensitizationUtil.desensitizePhone(userSimpleDTO.getPhone()) : "");
			return logInfoResponseDTO;
		}).collect(Collectors.toList());
		result.setList(dtoList);
		return result;
	}
}
