package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.convert.ManufacturerRnrConvert;
import com.cusc.nirvana.user.rnr.enterprise.dto.CompanyRnrResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseRnrFollowUpDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VehicleEnterpriseRnrEchoDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IManufacturerRnrService;
import com.cusc.nirvana.user.rnr.enterprise.service.IUserService;
import com.cusc.nirvana.user.rnr.enterprise.service.IVehicleCardService;
import com.cusc.nirvana.user.rnr.fp.client.EnterpriseRnrClient;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.fp.dto.ImageBase64DTO;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardDTO;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCompanyInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrFileClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgVehicleCompanyClient;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrFileType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderAuditTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderSourceEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCompanyInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrFileDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgVehicleCompanyDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
@Slf4j
public class ManufacturerRnrServiceImpl implements IManufacturerRnrService {

    @Autowired
    MgVehicleCompanyClient vehicleCompanyClient;
    @Resource
    private FileSystemClient fileSystemClient;

    @Autowired
    IVehicleCardService vehicleCardService;

    @Autowired
    private MgRnrCompanyInfoClient companyInfoClient;
    @Autowired
    private MgRnrFileClient rnrFileClient;
    @Autowired
    private MgRnrInfoClient rnrInfoClient;

    @Resource
    private EnterpriseRnrClient enterpriseClient;

    @Resource
    private IUserService userService;

    /**
     * 查询车企详情
     *
     * @return 实例对象
     */
    @Override
    public VehicleEnterpriseRnrEchoDTO findDetail(String tenantNo, String orgId) {
        //获取当前的用户车企信息
        MgVehicleCompanyDTO paramDTO = new MgVehicleCompanyDTO();
        paramDTO.setTenantNo(tenantNo);
        paramDTO.setOrgId(orgId);
        Response<MgVehicleCompanyDTO> response = vehicleCompanyClient.getVehicleCompany(paramDTO);
        log.info("查询企业信息返参:{}", JSON.toJSONString(response));
        if (response == null || !response.getSuccess() || response.getData() == null) {
            return null;
        }
        MgVehicleCompanyDTO vehicleCompanyDTO = response.getData();

        //人的信息
        MgRnrInfoDTO rnrInfo = new MgRnrInfoDTO();
        rnrInfo.setUuid(vehicleCompanyDTO.getFirstRnrId());
        rnrInfo.setTenantNo(tenantNo);
        Response<MgRnrInfoDTO> rnrInfoResp = rnrInfoClient.getByUuid(rnrInfo);
        if (rnrInfoResp == null || !rnrInfoResp.isSuccess() || rnrInfoResp.getData() == null) {
            throw new CuscUserException("", "实名责任人信息不能为空");
        }
        rnrInfo = rnrInfoResp.getData();
        //设置责任人信息
        vehicleCompanyDTO.setCorporationName(rnrInfo.getFullName());
        vehicleCompanyDTO.setCorporationPhone(rnrInfo.getPhone());


        VehicleEnterpriseRnrEchoDTO resultDTO =
                ManufacturerRnrConvert.INSTANCE.vehicleCompanyToEchoDTO(vehicleCompanyDTO);
        resultDTO.setVehicleEnterpriseId(vehicleCompanyDTO.getUuid());

     	//企业实名认证授权书
        resultDTO.setAuthorizationLetterPic(getFileBase64ByRnrIdType(vehicleCompanyDTO.getFirstRnrId(),tenantNo,RnrFileType.ENTERPRISE_AUTH_FILE));

        //入网合同
        resultDTO.setContractPic(getFileBase64ByRnrIdType(vehicleCompanyDTO.getFirstRnrId(),tenantNo,RnrFileType.VEHUCLE_BIND));

        //责任告知书
		resultDTO.setDutyPic(getFileBase64ByRnrIdType(vehicleCompanyDTO.getFirstRnrId(),tenantNo,RnrFileType.DUTY_FILE));
        
        return resultDTO;
    }

	private List<String> getFileBase64ByRnrIdType(String rnrId,String tenantNo, RnrFileType rnrFileType) {
		MgRnrFileDTO file = new MgRnrFileDTO();
		file.setRnrId(rnrId);
		file.setTenantNo(tenantNo);
		file.setFileType(rnrFileType.getCode());
		Response<List<MgRnrFileDTO>> fileListResp = rnrFileClient.queryByList(file);
		log.info("查询文件 rnrId = {},tenantNo = {}, fileType ={}, response = {}",rnrId,tenantNo,rnrFileType.getComment(),JSONObject.toJSONString(fileListResp));
		if (fileListResp == null || !fileListResp.isSuccess()) {
			return Collections.emptyList();
		}

		List<String> base64List = new ArrayList<>();
		for(MgRnrFileDTO fileTmp : fileListResp.getData()){
			FileDownloadDTO fileDownloadDTO = new FileDownloadDTO();
			fileDownloadDTO.setUuid(fileTmp.getFileSystemId());
			Response<ImageBase64DTO> fileRecordResponse = fileSystemClient.getBase64(fileDownloadDTO);
			if (fileRecordResponse != null && fileRecordResponse.isSuccess()) {
				//需要将文件信息转为base64
				base64List.add(fileRecordResponse.getData().getBase64());
			}
		}
		return base64List;
	}

	@Override
    public Response<CompanyRnrResponseDTO> submitRnrFollowUp(EnterpriseRnrFollowUpDTO bean) {
        log.info("submitRnrFollowUp 入参信息:{}", JSON.toJSONString(bean));

        //通过车企信息id查询车企信息
        MgVehicleCompanyDTO fvc = new MgVehicleCompanyDTO();
        fvc.setUuid(bean.getVehicleEnterpriseId());
        Response<MgVehicleCompanyDTO> fvcResp = vehicleCompanyClient.getByUuid(fvc);

        if (fvcResp != null && fvcResp.isSuccess()) {
            fvc = fvcResp.getData();
        }
        //验证车卡关系
        Response<List<VinCardDTO>> listResponse = vehicleCardService.checkIccidList(bean.getFileId());
        log.info("验证车卡关系", bean);
        if (!listResponse.isSuccess()) {
            return Response.createSuccess(false);
        }

        //调用fp接口 提交车企实名
        RnrRelationDTO rnrRelationDTO = turnParams(fvc, listResponse.getData());
        if (CollectionUtils.isEmpty(rnrRelationDTO.getCardList())) {
            return Response.createError("ICCID不能为空");
        }

        rnrRelationDTO.setSerialNumber(rnrRelationDTO.getOrder().getSerialNumber());
        Response<Boolean> enterpriseResponse =
                enterpriseClient.enterpriseRnrFollowUp(rnrRelationDTO);
        log.info("提交车企实名后续 :{}", JSON.toJSONString(enterpriseResponse));
        if (!enterpriseResponse.isSuccess()) {
            return Response.createError(enterpriseResponse.getMsg(), enterpriseResponse.getCode());
        } else {
            Response submitResponse = Response.createSuccess();
            CompanyRnrResponseDTO companyRnrResponseDTO = new CompanyRnrResponseDTO();
            companyRnrResponseDTO.setVerifyStatus(0);
            submitResponse.setData(companyRnrResponseDTO);
            return submitResponse;
        }
    }

    //转换成RelationDto
    private RnrRelationDTO turnParams(MgVehicleCompanyDTO vcDto, List<VinCardDTO> vinCardDTOList) {
        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();
        String rnrId = CuscStringUtils.generateUuid();
        String tenantNo = UserSubjectUtil.getTenantNo();
        //租户
        rnrRelationDTO.setTenantNo(tenantNo);
        rnrRelationDTO.setIsTrust(0);


        //查询当前用户的组织id
        String orgId = userService.getOrganId(UserSubjectUtil.getUserId(),tenantNo);

        //人的信息
        MgRnrInfoDTO rnrInfo = new MgRnrInfoDTO();
        rnrInfo.setUuid(vcDto.getFirstRnrId());
        rnrInfo.setTenantNo(tenantNo);
        Response<MgRnrInfoDTO> rnrInfoResp = rnrInfoClient.getByUuid(rnrInfo);
        if (rnrInfoResp == null || !rnrInfoResp.isSuccess() || rnrInfoResp.getData() == null) {
            throw new CuscUserException("", "实名责任人信息不能为空");
        }
        rnrInfo = rnrInfoResp.getData();
        rnrInfo.setUuid(rnrId);
        rnrInfo.setCreator(UserSubjectUtil.getUserId());
        rnrInfo.setCreateTime(null);
        rnrInfo.setOperator(null);
        rnrInfo.setUpdateTime(null);
        rnrInfo.setId(null);
        rnrInfo.setSerial_number(rnrId);
        rnrInfo.setOrgId(orgId);

        //公司信息 ,通过rnrId查询公司信息
        MgRnrCompanyInfoDTO companyInfo = new MgRnrCompanyInfoDTO();
        companyInfo.setRnrId(vcDto.getFirstRnrId());
        companyInfo.setTenantNo(tenantNo);
        Response<MgRnrCompanyInfoDTO> companyInfoResp = companyInfoClient.getByRnrid(companyInfo);
        if (companyInfoResp == null || !companyInfoResp.isSuccess() || companyInfoResp.getData() == null) {
            throw new CuscUserException("", "实名车企信息不能为空");
        }
        companyInfo = companyInfoResp.getData();
        companyInfo.setUuid(CuscStringUtils.generateUuid());
        companyInfo.setRnrId(rnrId);
        companyInfo.setCreator(UserSubjectUtil.getUserId());
        companyInfo.setCreateTime(null);
        companyInfo.setOperator(null);
        companyInfo.setUpdateTime(null);
        companyInfo.setId(null);
        companyInfo.setSerial_number(rnrId);

        int orderType = RnrOrderType.CARMAKER_NEW_VEHICLE.getCode();

        //订单信息
        RnrOrderDTO rnrOrderDTO = createOrder(rnrId, tenantNo, orderType);
        rnrOrderDTO.setOrgId(orgId);
        //卡信息
        List<MgRnrCardInfoDTO> cardInfoDTOS =
                createCardInfoDTOs(tenantNo, rnrId, rnrOrderDTO.getUuid(), vinCardDTOList);
        //文件列表
        //公司信息 ,通过rnrId查询公司信息
        MgRnrFileDTO fileInfo = new MgRnrFileDTO();
        fileInfo.setRnrId(vcDto.getFirstRnrId());
        fileInfo.setTenantNo(tenantNo);
        Response<List<MgRnrFileDTO>> fileInfoResp = rnrFileClient.queryByList(fileInfo);
        if (fileInfoResp == null || !fileInfoResp.isSuccess() || CollectionUtils.isEmpty(fileInfoResp.getData())) {
            throw new CuscUserException("", "实名图片信息不能为空");
        }
        List<MgRnrFileDTO> fileInfoList = fileInfoResp.getData();
        for(MgRnrFileDTO file : fileInfoList){
            file.setRnrId(rnrId);
            file.setRnrCompanyId(companyInfo.getUuid());
            file.setCreator(UserSubjectUtil.getUserId());
            file.setCreateTime(null);
            file.setOperator(null);
            file.setUpdateTime(null);
            file.setId(null);
            file.setSerial_number(rnrId);
        }
        //实名联系人信息
        rnrRelationDTO.setRnrLiaisonList(new ArrayList<>());
        rnrRelationDTO.setCompanyInfo(companyInfo);
        rnrRelationDTO.setInfo(rnrInfo);
        rnrRelationDTO.setCardList(cardInfoDTOS);
        rnrRelationDTO.setOrder(rnrOrderDTO);
        rnrRelationDTO.setRnrFileList(fileInfoList);
        rnrRelationDTO.setRnrTagList(new ArrayList<>());
        return rnrRelationDTO;
    }

    //创建工单
    private RnrOrderDTO createOrder(String uuid, String tenNo, int orderType) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderType(orderType);
        rnrOrderDTO.setRnrId(uuid);
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setIsBatchOrder(1);
        rnrOrderDTO.setSerialNumber(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.PORTAL.getCode());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setTenantNo(tenNo);
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.PASS.getCode());
        rnrOrderDTO.setSendWorkOrder(false);
        return rnrOrderDTO;
    }

    //卡信息
    private List<MgRnrCardInfoDTO> createCardInfoDTOs(String tenantNo, String rnrId, String uuid,
                                                      List<VinCardDTO> vinCardDTOS) {
        List<MgRnrCardInfoDTO> cardInfoDTOS = new ArrayList<>();
        for (VinCardDTO cardDTO : vinCardDTOS) {
            MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
            mgRnrCardInfoDTO.setUuid(CuscStringUtils.generateUuid());
            mgRnrCardInfoDTO.setIccid(cardDTO.getIccid());
            mgRnrCardInfoDTO.setTenantNo(tenantNo);
            mgRnrCardInfoDTO.setIotId(cardDTO.getVin());
            mgRnrCardInfoDTO.setRnrId(rnrId);
            mgRnrCardInfoDTO.setOrderId(uuid);
            mgRnrCardInfoDTO.setCreator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setOperator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
            cardInfoDTOS.add(mgRnrCardInfoDTO);
        }
        return cardInfoDTOS;
    }
}
