package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.service.INewVinCardService;
import com.cusc.nirvana.user.rnr.fp.client.NewVinCardClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;

/**
 * @author stayAnd
 * @date 2022/5/18
 */
@Service
public class NewVinCardServiceImpl implements INewVinCardService {

	@Resource
	private NewVinCardClient newVinCardClient;

	@Override
	public Response<VinCardResultDTO> queryUnBindCardByVin(VinCardQueryDTO queryDTO) {
		return newVinCardClient.queryUnBindCardByVin(queryDTO);
	}

	@Override
	public VinCardResultDTO queryBindCardByVin(VinCardQueryDTO queryDTO) {
		return newVinCardClient.queryBindCardByVin(queryDTO).getData();
	}

	@Override
	public VinCardCheckResultDTO checkVinCard(VinCardCheckDTO vinCardCheckDTO) {
		//paramCheck(vinCardCheckDTO);
		if (!CollectionUtils.isEmpty(vinCardCheckDTO.getIccidList()) && vinCardCheckDTO.getIccidList().stream().distinct().count() != vinCardCheckDTO.getIccidList().size()) {
			throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"请不要输入同样的iccid");
		}
		Response<VinCardCheckResultDTO> checkResponse = newVinCardClient.checkVinCard(vinCardCheckDTO);
		if (!checkResponse.isSuccess()) {
			throw new CuscUserException(checkResponse.getCode(),checkResponse.getMsg());
		}
		return checkResponse.getData();
	}

	@Override
	public VinCheckResultDTO checkVin(VinCheckRequestDTO requestDTO) {
		if (requestDTO.getVinList().stream().distinct().count() != requestDTO.getVinList().size()) {
			throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"请不要输入同样的iccid");
		}
		Response<VinCheckResultDTO> response = newVinCardClient.checkVin(requestDTO);
		if (!response.isSuccess()) {
			throw new CuscUserException(response.getCode(),response.getMsg());
		}
		return response.getData();
	}


}
