package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.rnr.enterprise.common.BizConstants;
import com.cusc.nirvana.user.rnr.enterprise.common.CompanyTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.common.IndustryTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.dto.AuthInfo;
import com.cusc.nirvana.user.rnr.enterprise.dto.AuthItemInfo;
import com.cusc.nirvana.user.rnr.enterprise.dto.CardInfo;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseInfo;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileInfo;
import com.cusc.nirvana.user.rnr.enterprise.dto.NatualPersonInfo;
import com.cusc.nirvana.user.rnr.enterprise.dto.OrderDetailsResponse;
import com.cusc.nirvana.user.rnr.enterprise.dto.OrderInfo;
import com.cusc.nirvana.user.rnr.enterprise.service.IOrderMgrService;
import com.cusc.nirvana.user.rnr.enterprise.util.DateExtentUtils;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FileRecordDTO;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrAuthenticationResultClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCompanyInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrFileClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrLiaisonInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.RnrOrderClient;
import com.cusc.nirvana.user.rnr.mg.constants.AuthWayTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.CertTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.GenderEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrFileType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrLiaisonType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrAuthenticationResultDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCompanyInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrFileDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrLiaisonInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Service
@Slf4j
public class OrderMgrServiceImpl implements IOrderMgrService {
    public static final String DEFALUT_VALUE_LIAISONID = "0";
    public static final String AUTH_RESULT_SUCCESS = "1";
    @Resource
    private RnrOrderClient rnrOrderClient;
    @Resource
    private MgRnrInfoClient mgRnrInfoClient;
    @Resource
    private MgRnrCompanyInfoClient mgRnrCompanyInfoClient;
    @Resource
    private MgRnrFileClient mgRnrFileClient;
    @Resource
    private MgRnrCardInfoClient mgRnrCardInfoClient;
    @Resource
    private MgRnrLiaisonInfoClient mgRnrLiaisonInfoClient;
    @Resource
    private MgRnrAuthenticationResultClient mgRnrAuthenticationResultClient;
    @Resource
    @Qualifier("nonLoadBalancedFile")
    private RestTemplate restTemplateNoLB;

    @Resource
	private FileSystemClient fileSystemClient;



    /**
     * 自然人关联文件类型集合
     */
    private static Set<Integer> NPFileTypeSet = new HashSet<>();
    /**
     * 企业关联文件类型
     */
    private static Set<Integer> EntFileTypeSet = new HashSet<>();
    /**
     * 认证信息关联文件类型
     */
    private static Set<Integer> authFileTypeSet = new HashSet<>();

    static {
        NPFileTypeSet.add(RnrFileType.IDENTITY_CARD_BACK.getCode());
        NPFileTypeSet.add(RnrFileType.IDENTITY_CARD_FRONT.getCode());
        NPFileTypeSet.add(RnrFileType.POLICE_CARD_FRONT.getCode());
        NPFileTypeSet.add(RnrFileType.POLICE_CARD_BACK.getCode());
        NPFileTypeSet.add(RnrFileType.HK_MACAO_PASSPORT_FRONT.getCode());
        NPFileTypeSet.add(RnrFileType.HK_MACAO_PASSPORT_BACK.getCode());
        NPFileTypeSet.add(RnrFileType.TAIWAN_CARD_FRONT.getCode());
        NPFileTypeSet.add(RnrFileType.TAIWAN_CARD_BACK.getCode());
        NPFileTypeSet.add(RnrFileType.FOREIGN_NATIONAL_PASSPORT_FRONT.getCode());
        NPFileTypeSet.add(RnrFileType.FOREIGN_NATIONAL_PASSPORT_BACK.getCode());
        NPFileTypeSet.add(RnrFileType.HK_MACAO_RESIDENCE_PERMIT_FRONT.getCode());
        NPFileTypeSet.add(RnrFileType.HK_MACAO_RESIDENCE_PERMIT_BACK.getCode());
        NPFileTypeSet.add(RnrFileType.TAIWAN_RESIDENCE_PERMIT_FRONT.getCode());
        NPFileTypeSet.add(RnrFileType.TAIWAN_RESIDENCE_PERMIT_BACK.getCode());
        NPFileTypeSet.add(RnrFileType.RESIDENCE_BOOKLET_FRONT.getCode());
        NPFileTypeSet.add(RnrFileType.RESIDENCE_BOOKLET_BACK.getCode());
		NPFileTypeSet.add(RnrFileType.OFFICIAL_CARD_FRONT.getCode());
		NPFileTypeSet.add(RnrFileType.OFFICIAL_CARD_BACK.getCode());
		NPFileTypeSet.add(RnrFileType.OTHER.getCode());

        EntFileTypeSet.add(RnrFileType.ENTERPRISE_PIC.getCode());
		EntFileTypeSet.add(RnrFileType.ENTERPRISE_AUTH_FILE.getCode());

        authFileTypeSet.add(RnrFileType.LIVENESS_SCREEN_FIRST.getCode());
        authFileTypeSet.add(RnrFileType.LIVENESS_SCREEN_TWO.getCode());
        authFileTypeSet.add(RnrFileType.LIVENESS_VIDEO.getCode());
    }

    @Override
    public OrderDetailsResponse queryOrderDetail(String rnrOrderUuid) {
        OrderDetailsResponse rsp = new OrderDetailsResponse();
        RnrOrderDTO rnrOrderDto = new RnrOrderDTO();
        rnrOrderDto.setUuid(rnrOrderUuid);
        Response<RnrOrderDTO> rnrOrderRsp = rnrOrderClient.getByUuid(rnrOrderDto);
        if (rnrOrderRsp != null && rnrOrderRsp.isSuccess() && rnrOrderRsp.getData() != null) {
            RnrOrderDTO rnrOrder = wrapOrderInfo(rsp, rnrOrderRsp);//包装工单信息
            String rnrId = rnrOrder.getRnrId();
            String tenantNo = rnrOrder.getTenantNo();
            MgRnrInfoDTO mgRnrInfoQueryBean = new MgRnrInfoDTO();
            mgRnrInfoQueryBean.setUuid(rnrId);
            mgRnrInfoQueryBean.setTenantNo(tenantNo);
            Response<MgRnrInfoDTO> mgRnrInfoRsp = mgRnrInfoClient.getByUuid(mgRnrInfoQueryBean);
            if (mgRnrInfoRsp != null && mgRnrInfoRsp.isSuccess() && mgRnrInfoRsp.getData() != null) {
                MgRnrInfoDTO mgRnrInfo = mgRnrInfoRsp.getData();
                if (mgRnrInfo.getIsCompany() == 0) {
                    //个人实名
                    deal4NaturalPerson(rsp, rnrId, mgRnrInfo);
                    rsp.getOrderInfo().setIsCompany(false);
                } else if (mgRnrInfo.getIsCompany() == 1) {
                    //企业实名

                    dear4Company(rsp, mgRnrInfo);
                    rsp.getOrderInfo().setIsCompany(true);

                }
                //车卡关系
                wrapCardInfo(rsp, rnrId, tenantNo);
                //处理认证信息
//                wrapAuthInfo(rsp, rnrId, tenantNo);
                wrapAuthInfoNew(rsp, rnrId, tenantNo);
                //处理文件
                wrapFileInfo(rsp, rnrId, tenantNo);
                //////////

                //企业变更责任人
                if (RnrOrderType.COMPANY_CORPORATION_CHANGE.getCode() == rnrOrder.getOrderType()) {
                    rsp.setOldCompanyInfo(rsp.getEnterpriseInfo());
                    rsp.setEnterpriseInfo(null);
                    wrapNewOwnerInfo(rsp, mgRnrInfo);
                }
            }
        }
        return rsp;
    }

    @Resource
    private OrganizationClient organizationClient;

    private RnrOrderDTO wrapOrderInfo(OrderDetailsResponse rsp, Response<RnrOrderDTO> rnrOrderRsp) {
        RnrOrderDTO rnrOrder = rnrOrderRsp.getData();
        Integer orderType = rnrOrder.getOrderType();

        OrderInfo orderInfo = new OrderInfo();
        orderInfo.setRnrCreateDate(DateUtils.formatDatetime(rnrOrder.getCreateTime()));
        orderInfo.setBizTypeName(RnrOrderType.getDescription(orderType).getComment());

        if (StringUtils.isNotBlank(rnrOrder.getOrgId())) {
            OrganizationDTO queryOrg = new OrganizationDTO();
            queryOrg.setUuid(rnrOrder.getOrgId());
            Response<OrganizationDTO> orgRsp = organizationClient.getByUuid(queryOrg);
            if (orgRsp != null && orgRsp.isSuccess() && orgRsp.getData() != null) {
                OrganizationDTO orgDetail = orgRsp.getData();
                if (orgDetail.getBizType() == BizConstants.ORG_BIZ_TYPE_VEHICLE_COMPANY) {
                    orderInfo.setVehicleCompanyName(orgDetail.getOrganName());
                } else if (orgDetail.getBizType() == BizConstants.ORG_BIZ_TYPE_DISTRIBUTOR) {
                    orderInfo.setDistributorName(orgDetail.getOrganName());
                }
            }
        }

        rsp.setOrderInfo(orderInfo);
        return rnrOrder;
    }

    private void deal4NaturalPerson(OrderDetailsResponse rsp, String rnrId, MgRnrInfoDTO mgRnrInfo) {
        try {
            //车主、责任人信息
            wrapOwnerInfo(rsp, mgRnrInfo);
            //委托实名
            if (mgRnrInfo.getIsTrust() == 1) {
                wrapAgentInfo(rsp, rnrId, mgRnrInfo.getTenantNo());
            }
        } catch (Exception e) {
            log.error("deal4NaturalPerson error", e);
        }
    }

    private void wrapOwnerInfo(OrderDetailsResponse rsp, MgRnrInfoDTO mgRnrInfo) {
        NatualPersonInfo ownerInfo = wrapNatualPersonInfoByInfoDto(mgRnrInfo);
        rsp.setOwnerInfo(ownerInfo);//自然人-车主、企业实名-责任人
    }

    private void dear4Company(OrderDetailsResponse rsp, MgRnrInfoDTO mgRnrInfo) {
        String companyUuid = mgRnrInfo.getRnrCompanyId();
        MgRnrCompanyInfoDTO queryBean4CompanyUuid = new MgRnrCompanyInfoDTO();
        queryBean4CompanyUuid.setUuid(companyUuid);
        queryBean4CompanyUuid.setTenantNo(mgRnrInfo.getTenantNo());
        Response<MgRnrCompanyInfoDTO> rnrCompanyRsp = mgRnrCompanyInfoClient.getByUuid(queryBean4CompanyUuid);
        if (rnrCompanyRsp != null && rnrCompanyRsp.isSuccess() && rnrCompanyRsp.getData() != null) {
            MgRnrCompanyInfoDTO mgRnrCompanyInfo = rnrCompanyRsp.getData();
            EnterpriseInfo entInfo = new EnterpriseInfo();
            entInfo.setCompanyName(mgRnrCompanyInfo.getCompanyName());
            entInfo.setIndustryType(IndustryTypeEnum.getEnumByCode(mgRnrCompanyInfo.getIndustryType()).getValue());
            entInfo.setCompanyType(CompanyTypeEnum.getEnumByCode(mgRnrCompanyInfo.getCompanyType()).getValue());
            entInfo.setCompanyContactAddress(mgRnrCompanyInfo.getCompanyContactAddress());
            entInfo.setCompanyCertNumber(mgRnrCompanyInfo.getCompanyCertNumber());
            entInfo.setCompanyCertAddress(mgRnrCompanyInfo.getCompanyCertAddress());
            entInfo.setCompanyCertType(CertTypeEnum.getEnumByCode(mgRnrCompanyInfo.getCompanyCertType()).getName());
            rsp.setEnterpriseInfo(entInfo);
        }
        wrapOwnerInfo(rsp, mgRnrInfo);
    }

    //企业责任人变更后的信息
    private void wrapNewOwnerInfo(OrderDetailsResponse rsp, MgRnrInfoDTO mgRnrInfo) {
        NatualPersonInfo ownerInfo = wrapNatualPersonInfoByInfoDto(mgRnrInfo);
        ownerInfo.setFileMap(rsp.getOwnerInfo().getFileMap());
        rsp.setOwnerInfo(null);
        rsp.setNewOwnerInfo(ownerInfo);//自然人-车主、企业实名-责任人
    }

    private NatualPersonInfo wrapNatualPersonInfoByInfoDto(MgRnrInfoDTO mgRnrInfo) {
        NatualPersonInfo ownerInfo = new NatualPersonInfo();
        if (mgRnrInfo.getExpiredDate() != null) {
            ownerInfo.setExpiredDate(mgRnrInfo.getExpiredDate());
        }
        if (mgRnrInfo.getEffectiveDate() != null) {
            ownerInfo.setEffectiveDate(DateUtils.formatDate(DateExtentUtils.dateToLocalDate(mgRnrInfo.getEffectiveDate()), "yyyy-MM-dd"));
        }
        ownerInfo.setPhone(mgRnrInfo.getPhone());
        ownerInfo.setGender(GenderEnum.findByCode(mgRnrInfo.getGender()).getName());
        ownerInfo.setContactAddress(mgRnrInfo.getContactAddress());
        ownerInfo.setCertType(CertTypeEnum.getEnumByCode(mgRnrInfo.getCertType()).getName());
        ownerInfo.setCertNumber(mgRnrInfo.getCertNumber());
        ownerInfo.setFullName(mgRnrInfo.getFullName());
        ownerInfo.setIsCompany(mgRnrInfo.getIsCompany() == 1);
        ownerInfo.setCertAddress(mgRnrInfo.getCertAddress());
        ownerInfo.setUuid(mgRnrInfo.getUuid());
        return ownerInfo;
    }

    private void wrapCardInfo(OrderDetailsResponse rsp, String rnrId, String tenantNo) {
        try {
            MgRnrCardInfoDTO cardInfoQBean = new MgRnrCardInfoDTO();
            cardInfoQBean.setRnrId(rnrId);
            cardInfoQBean.setTenantNo(tenantNo);
            Response<List<MgRnrCardInfoDTO>> cardInfoListRsp = mgRnrCardInfoClient.queryByList(cardInfoQBean);
            if (cardInfoListRsp != null && cardInfoListRsp.isSuccess() && cardInfoListRsp.getData() != null) {
                List<MgRnrCardInfoDTO> rnrCardInfoList = cardInfoListRsp.getData();
                if (rnrCardInfoList.size() > 0) {
                    List<CardInfo> cardInfoList = new ArrayList<>();
                    for (MgRnrCardInfoDTO rnrCardInfo : rnrCardInfoList) {
                        CardInfo cardInfo = new CardInfo();
                        cardInfo.setIccid(rnrCardInfo.getIccid());
                        cardInfo.setVin(rnrCardInfo.getIotId());
                        cardInfo.setCheckInfo("待确认信息");
                        cardInfoList.add(cardInfo);
                    }
                    rsp.setCardInfoList(cardInfoList);
                }
            }
        } catch (Exception e) {
            log.error("wrapCardInfo error", e);
        }
    }

    private void wrapAgentInfo(OrderDetailsResponse rsp, String rnrId, String tenantNo) {
        try {
            NatualPersonInfo agentInfo = new NatualPersonInfo();
            MgRnrLiaisonInfoDTO queryBean = new MgRnrLiaisonInfoDTO();
            queryBean.setRnrId(rnrId);
            queryBean.setTenantNo(tenantNo);
            queryBean.setLiaisonType(RnrLiaisonType.CONSIGNEE.getCode());
            Response<List<MgRnrLiaisonInfoDTO>> mgRnrLiaisonInfoRsp = mgRnrLiaisonInfoClient.queryByList(queryBean);
            if (mgRnrLiaisonInfoRsp != null && mgRnrLiaisonInfoRsp.isSuccess() && mgRnrLiaisonInfoRsp.getData() != null) {
                List<MgRnrLiaisonInfoDTO> mgRnrLiaisonInfoList = mgRnrLiaisonInfoRsp.getData();
                //代理人只且只有一个
                if (mgRnrLiaisonInfoList.size() > 0) {
                    MgRnrLiaisonInfoDTO mgRnrLiaisonInfo = mgRnrLiaisonInfoList.get(0);
                    agentInfo.setFullName(mgRnrLiaisonInfo.getLiaisonName());
                    agentInfo.setCertType(CertTypeEnum.getEnumByCode(mgRnrLiaisonInfo.getLiaisonCertType()).getName());
                    agentInfo.setCertNumber(mgRnrLiaisonInfo.getLiaisonCertNumber());
                    agentInfo.setCertAddress(mgRnrLiaisonInfo.getLiaisonCertAddress());
                    agentInfo.setPhone(mgRnrLiaisonInfo.getLiaisonPhone());
                    agentInfo.setContactAddress(mgRnrLiaisonInfo.getLiaisonContactAddress());
                    agentInfo.setUuid(mgRnrLiaisonInfo.getUuid());
                    agentInfo.setGender("TODO");
                    agentInfo.setExpiredDate("TODO");//FIXME

                    rsp.setAgentInfo(agentInfo);
                }

            }
        } catch (Exception e) {
            log.error("wrapAgentInfo error", e);
        }
    }

    private void wrapFileInfo(OrderDetailsResponse rsp, String rnrId, String tenantNo) {
        try {
            MgRnrFileDTO queryBean = new MgRnrFileDTO();
            queryBean.setRnrId(rnrId);
            queryBean.setTenantNo(tenantNo);
            Response<List<MgRnrFileDTO>> fileInfoListRsp = mgRnrFileClient.queryByList(queryBean);
            if (fileInfoListRsp != null && fileInfoListRsp.isSuccess() && fileInfoListRsp.getData() != null) {
                List<MgRnrFileDTO> fileInfoList = fileInfoListRsp.getData();
                for (MgRnrFileDTO fileInfo : fileInfoList) {
                    String fileType = RnrFileType.getDescription(fileInfo.getFileType()).name();
                    if (NPFileTypeSet.contains(fileInfo.getFileType())) {
                        if (StringUtils.isNotBlank(fileInfo.getLiaisonId()) && !DEFALUT_VALUE_LIAISONID.equals(fileInfo.getLiaisonId()) && rsp.getAgentInfo() != null) {
                            Map<String, List<FileInfo>> fileMap = rsp.getAgentInfo().getFileMap();
                            fileMap.put(fileType, getFileInfoList(fileMap.get(fileType), fileInfo));
                        } else if (rsp.getOwnerInfo() != null) {
                            Map<String, List<FileInfo>> fileMap = rsp.getOwnerInfo().getFileMap();
                            fileMap.put(fileType, getFileInfoList(fileMap.get(fileType), fileInfo));
                        }
                    } else if (EntFileTypeSet.contains(fileInfo.getFileType()) && rsp.getEnterpriseInfo() != null) {
                        //企业信息模块中展示文件
                        EnterpriseInfo enterpriseInfo = rsp.getEnterpriseInfo();
                        Map<String, List<FileInfo>> fileMap = enterpriseInfo.getFileMap();
                        fileMap.put(fileType, getFileInfoList(fileMap.get(fileType), fileInfo));
                        continue;
                    } else if (authFileTypeSet.contains(fileInfo.getFileType()) && rsp.getAuthInfo() != null) {
                        Map<String, List<FileInfo>> fileMap = rsp.getAuthInfo().getFileMap();
                        fileMap.put(fileType, getFileInfoList(fileMap.get(fileType), fileInfo));
                        continue;
                    } else {
                        Map<String, List<FileInfo>> fileMap = rsp.getFileMap();
                        fileMap.put(fileType, getFileInfoList(fileMap.get(fileType), fileInfo));
                        continue;
                    }
                }
            }
        } catch (Exception e) {
            log.error("wrapFileInfo error", e);
        }

    }


    private void wrapAuthInfoNew(OrderDetailsResponse rsp, String rnrId, String tenantNo) {
        MgRnrAuthenticationResultDTO baseAuthQBean = new MgRnrAuthenticationResultDTO();
        baseAuthQBean.setRnrId(rnrId);
        baseAuthQBean.setTenantNo(tenantNo);
        try {
            Response<List<MgRnrAuthenticationResultDTO>> authResultRsp = mgRnrAuthenticationResultClient.queryByList(baseAuthQBean);
            if (authResultRsp != null && authResultRsp.isSuccess() && authResultRsp.getData() != null) {
                List<MgRnrAuthenticationResultDTO> authResultList = authResultRsp.getData();
                if (authResultList.size() > 0) {
                    AuthInfo authInfo = new AuthInfo();
                    List<AuthItemInfo> authItemInfoList = new ArrayList<>();
                    for (MgRnrAuthenticationResultDTO authResultItem : authResultList) {
                        AuthItemInfo authItem = new AuthItemInfo();
                        authItem.setMessage(authResultItem.getAuthResultMsg());
                        authItem.setSuccess(AUTH_RESULT_SUCCESS.equals(authResultItem.getAuthResult()));
                        authItem.setTpActionCode(authResultItem.getAuthWayType());
                        AuthWayTypeEnum authWayEnum = AuthWayTypeEnum.getEnumByCode(authResultItem.getAuthWayType());
                        authItem.setTpActionName(authWayEnum == null ? "" : authWayEnum.getName());
                        authItemInfoList.add(authItem);
                    }
                    authInfo.setAuthItemInfoList(authItemInfoList);
                    rsp.setAuthInfo(authInfo);
                }
            }
        } catch (Exception e) {
            log.error("wrapAuthInfo error", e);
        }

    }

    private List<FileInfo> getFileInfoList(List<FileInfo> fileInfos, MgRnrFileDTO fileInfo) {
        List<FileInfo> fileList = null;
        if (fileInfos == null) {
            fileList = new ArrayList<>();
            fileList.add(wrapFileInfo(fileInfo));
        } else {
            fileInfos.add(wrapFileInfo(fileInfo));
            fileList = fileInfos;
        }
        return fileList;
    }

    public String getFileUrl(String fileUuid) {
        try {
            //String url = fileServerUrl + "/file/url/" + fileUuid;
            //RnrResponse<String> fileUrlRsp = RnrBaseRestTemplateUtils.getForResponse(this.restTemplateNoLB, url, String.class);
            //return fileUrlRsp != null && fileUrlRsp.isSuccess() && fileUrlRsp.getData() != null ? fileUrlRsp.getData() : null;

			FileDownloadDTO dto = new FileDownloadDTO();
			dto.setUuid(fileUuid);
			Response<FileRecordDTO> response = fileSystemClient.getUrl(dto);
			return  response.getData().getAccessUrl();
		} catch (Exception e) {
            log.error("getFileUrl error", e);
            return null;
        }
    }

    private FileInfo wrapFileInfo(MgRnrFileDTO rnrFileInfo) {
        FileInfo fileInfo = new FileInfo();
        fileInfo.setFileBase64(rnrFileInfo.getFileBase64());
        fileInfo.setFileName(rnrFileInfo.getFileName());
        fileInfo.setFileTypeComment(RnrFileType.getDescription(rnrFileInfo.getFileType()).name());
        String fileUuid = rnrFileInfo.getFileSystemId();
        String fileUrl = getFileUrl(fileUuid);
        fileInfo.setFileUrl(fileUrl);
        return fileInfo;
    }


}
