package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cache.CacheFactory;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.client.UserClient;
import com.cusc.nirvana.user.eiam.client.UserOrganClient;
import com.cusc.nirvana.user.eiam.client.UserRoleClient;
import com.cusc.nirvana.user.eiam.dto.*;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.common.RedisConstants;
import com.cusc.nirvana.user.rnr.enterprise.constants.*;
import com.cusc.nirvana.user.rnr.enterprise.convert.OrganConvert;
import com.cusc.nirvana.user.rnr.enterprise.dto.*;
import com.cusc.nirvana.user.rnr.enterprise.service.IOrganService;
import com.cusc.nirvana.user.rnr.enterprise.service.IUserService;
import com.cusc.nirvana.user.rnr.fp.constants.BizTypeEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Description: 车企组织管理service
 * <br />
 * CreateDate 2022-05-20 13:37:40
 *
 * @author yuyi
 **/
@Service
@Slf4j
public class OrganServiceImpl implements IOrganService {

	@Resource
	private IUserService userService;

	@Resource
	private OrganizationClient organizationClient;

	@Resource
	private UserClient userClient;

	@Resource
	private CacheFactory cacheFactory;

	@Resource
	private UserRoleClient userRoleClient;

	@Value("${user.eiam.applicationId:4}")
	private String applicationId;

	@Resource
	private UserOrganClient userOrganClient;


	@Override
	public void add(OrganDTO bean) {

		//查询用户是否存在
		UserDTO queryUserDTO = userService.queryUserInInfoByPhone(bean.getOrganAdminPhone());
		if (null != queryUserDTO) {
			throw new CuscUserException(ResponseCode.USER_PHONE_EXISTS.getCode(),ResponseCode.USER_PHONE_EXISTS.getMsg());
		}

		//如果父类id为空表示创建在当前登录人下
		String organId = "";
		if(!StringUtils.isEmpty(bean.getParentId())){
			organId = bean.getParentId();
		}else {
			organId = userService.getOrganId(UserSubjectUtil.getUserId(),UserSubjectUtil.getTenantNo());
		}

		//获取当前登录人的上级id
		OrganizationDTO organs = new OrganizationDTO();
		organs.setUuid(organId);
		Response<OrganizationDTO> organizationResponse = organizationClient.getByUuid(organs);

		if (!organizationResponse.isSuccess()) {
			throw new CuscUserException(organizationResponse.getCode(),organizationResponse.getMsg());
		}

//		OrganizationDTO organization = new OrganizationDTO();
//		organization.setUuid(StringUtils.isEmpty(bean.getParentId())?organizationResponse.getData().getParentId():bean.getParentId());
//		organization.setTenantNo(UserSubjectUtil.getTenantNo());
//
//		Response<OrganizationDTO> organizationDTOResponse = organizationClient.getByUuid(organization);

//		if (!organizationDTOResponse.isSuccess()) {
//			throw new CuscUserException(organizationDTOResponse.getCode(),organizationDTOResponse.getMsg());
//		}

		//如果是经销商，不允许选择别的子组织
		if(OrganBizzTypeEnum.DISTRIBUTOR.getCode()==organizationResponse.getData().getBizType() && OrganBizzTypeEnum.DISTRIBUTOR.getCode() !=bean.getBizType()){
			throw new CuscUserException(ResponseCode.BIZTYPE_ERROR.getCode(),
					ResponseCode.BIZTYPE_ERROR.getMsg());
		}

//		//获取当前的用户的组织
//		UserOrganDTO userOrganDTO = new UserOrganDTO();
//		userOrganDTO.setUserId(UserSubjectUtil.getUserId());
//		userOrganDTO.setTenantNo(UserSubjectUtil.getTenantNo());
//
//		Response<List<OrganizationDTO>> organResp = organizationClient.getOrganListByUserId(userOrganDTO);
//
//		if (!organResp.isSuccess()) {
//			throw new CuscUserException(organResp.getCode(),organResp.getMsg());
//		}
//
//		if(CollectionUtils.isEmpty(organResp.getData())){
//			throw new CuscUserException(ResponseCode.USER_ORGAN_NOT_FOUND.getCode(),
//					ResponseCode.USER_ORGAN_NOT_FOUND.getMsg());
//		}
//
//		OrganizationDTO organ = organResp.getData().get(0);



		UserDTO userDto = bean.toUserDTO();
		userDto.setTenantNo(UserSubjectUtil.getTenantNo());
		userDto.setApplicationId(applicationId);
		//如果是车厂管理员为0，如果经销商为1，如果是子组织为2
		userDto.setOrdinaryAdmin(bean.getBizType().equals(OrganBizzTypeEnum.CAR_FACTORY.getCode())?OrdinaryAdminEnum.CAR_FACTORY.getCode():bean.getBizType().equals(OrganBizzTypeEnum.DISTRIBUTOR.getCode())
				?OrdinaryAdminEnum.DISTRIBUTOR.getCode():OrdinaryAdminEnum.ORGAN.getCode());
		UserDTO addUserDto = userService.addUser(userDto);

		//添加车企组织
		OrganizationDTO organDto = bean.toOrganDTO();
		organDto.setTenantNo(UserSubjectUtil.getTenantNo());
		organDto.setOrganCode(generateOrganCode());
		//如果没有传值上级id，获取当前用户登录的组织uuid
//		if(StringUtils.isEmpty(bean.getParentId())){
//			organDto.setParentId(organ.getUuid());
//		}
		organDto.setParentId(organizationResponse.getData().getUuid());
		Response<OrganizationDTO> organizationResponses = organizationClient.add(organDto);
		OrganizationDTO organizationDTO = organizationResponses.getData();

		//新增车企组织管理员及其关系
		userService.addUserRelationRole(addUserDto.getUuid(),organizationDTO.getUuid(),OrganBizzTypeEnum.CAR_FACTORY.getCode() == bean.getBizType() || OrganBizzTypeEnum.SON_ORGAN.getCode() == bean.getBizType()? RoleTyeEnum.SON_FACTORY.getCode():RoleTyeEnum.DISTRIBUTOR.getCode());
	}

	@Override
	public PageResult<OrganDTO> page(OrganPageDTO bean) {
		//获取当前的用户
		String userId = UserSubjectUtil.getUserId();
		//获取当前的用户的组织
		UserOrganDTO userOrganDTO = new UserOrganDTO();
		userOrganDTO.setUserId(UserSubjectUtil.getUserId());
		userOrganDTO.setTenantNo(UserSubjectUtil.getTenantNo());
		Response<List<OrganizationDTO>> organResp = organizationClient.getOrganListByUserId(userOrganDTO);
		if (!organResp.isSuccess()) {
			throw new CuscUserException(organResp.getCode(),organResp.getMsg());
		}
		if(CollectionUtils.isEmpty(organResp.getData())){
			throw new CuscUserException(ResponseCode.USER_ORGAN_NOT_FOUND.getCode(),
					ResponseCode.USER_ORGAN_NOT_FOUND.getMsg());
		}
		OrganizationDTO organ = organResp.getData().get(0);

		//查询当前用户，下一级数据。不查所有
		OrganizationDTO queryDto = bean.toOrganPageDTO();
//		queryDto.setQueryCode(organ.getQueryCode());
		if(StringUtils.isEmpty(bean.getParentId())) {
			queryDto.setParentId(organ.getUuid());
		}
		queryDto.setOrganType(null);
		queryDto.setType("1");
		Response<PageResult<OrganizationDTO>> organResponse = organizationClient.queryByPage(queryDto);
		if(!organResponse.isSuccess()){
			throw new CuscUserException(organResponse.getCode(),organResponse.getMsg());
		}
		PageResult<OrganizationDTO> page = organResponse.getData();
		//查询父类名称
		List<String> parentList=page.getList().stream().map(OrganizationDTO::getParentId).collect(Collectors.toList());
		OrganizationDTO organizationDTOs = new OrganizationDTO();
		organizationDTOs.setUuidList(parentList);
		Response<List<OrganizationDTO>> response=organizationClient.queryByList(organizationDTOs);
		if(!response.isSuccess()){
			throw new CuscUserException(response.getCode(),response.getMsg());
		}
		List<OrganizationDTO> organizationDTOList = response.getData();
		Map<String,String> map = organizationDTOList.stream().collect(Collectors.toMap(OrganizationDTO::getUuid,OrganizationDTO::getOrganName,(k, v)->v));

		List<OrganDTO> dtoList = page.getList().stream().map(organizationDTO -> {
			OrganDTO organDTO = OrganConvert.INSTANCE.eiamOrganToLocalOrgan(organizationDTO);
			organDTO.setOrganAdminName(organizationDTO.getAdminName());
			organDTO.setOrganAdminAccount(organizationDTO.getAdminAccount());
			organDTO.setUserId(organizationDTO.getAdminUserId());
			organDTO.setOrganAdminPhone(organizationDTO.getAdminPhone());
			organDTO.setCreateTime(organizationDTO.getCreateTime());
			organDTO.setParentOrganName(map.get(organizationDTO.getParentId()));
			organDTO.setAdminUserId(organizationDTO.getAdminUserId());
			return organDTO;
		}).collect(Collectors.toList());
		return new PageResult<OrganDTO>(dtoList,page.getTotalCount(),page.getPageSize(),page.getCurrPage());
	}

	@Override
	public void update(UpdateOrganDTO bean) {
		OrganizationDTO updateDto = new OrganizationDTO();
		updateDto.setUuid(bean.getUuid());
		updateDto.setOrganName(bean.getOrganName());
		updateDto.setComment(bean.getComment());
		updateDto.setTenantNo(UserSubjectUtil.getTenantNo());
		updateDto.setOperator(UserSubjectUtil.getUserId());
		updateDto.setBizType(bean.getBizType());
		updateDto.setStatus(bean.getStatus());
		organizationClient.update(updateDto);
	}

	 /**
	   * @author: jk
	   * @description: 变更管理员
	   * @date: 2022/5/27 9:13
	   * @version: 1.0
	   * @Param:
	   * @Return:
	   */
	@Override
	public void changeAdmin(OrganChangeAdminDto dto) {
		//如果新管理id为空表示是用新账户替换
		if(null == dto.getNewAdminUserId()){
			//查询新账号是否存在
			UserDTO userDTO = userService.queryUserInInfoByPhone(dto.getCreateAdminPhone());
			if (null != userDTO) {
				throw new CuscUserException(ResponseCode.USER_PHONE_EXISTS.getCode(),ResponseCode.USER_PHONE_EXISTS.getMsg());
			}
		}

		dto.setOrdinaryAdmin(OrganBizzTypeEnum.CAR_FACTORY.getCode()==dto.getBizType()?OrdinaryAdminEnum.CAR_FACTORY.getCode():OrganBizzTypeEnum.DISTRIBUTOR.getCode()==dto.getBizType()?OrdinaryAdminEnum.DISTRIBUTOR.getCode():OrdinaryAdminEnum.ORGAN.getCode());
		if (null == dto.getNewAdminUserId()) {
			this.checkNewAdminIsNull(dto);
			//创建用户
			UserDTO userAdminDto = new UserDTO();
			userAdminDto.setOrdinaryAdmin(dto.getOrdinaryAdmin());
			userAdminDto.setUserName(dto.getCreateAdminAccount());
			userAdminDto.setFullName(dto.getCreateAdminUserName());
			userAdminDto.setPhone(dto.getCreateAdminPhone());
			userAdminDto.setPassword(dto.getCreateAdminPassword());
			userAdminDto.setTenantNo(UserSubjectUtil.getTenantNo());
			userAdminDto.setApplicationId(applicationId);
			//新增用户 eiam_user
			UserDTO userDTO = userService.addUser(userAdminDto);
			//新增用户组织eiam_user_organ 和 用户和角色关系 eiam_user_role
			userService.addUserRelationRole(userDTO.getUuid(),dto.getUuid(),OrdinaryAdminEnum.CAR_FACTORY.getCode() == dto.getOrdinaryAdmin() || OrdinaryAdminEnum.ORGAN.getCode() == dto.getOrdinaryAdmin()? RoleTyeEnum.SON_FACTORY.getCode():RoleTyeEnum.DISTRIBUTOR.getCode());
		}else {
			if(StringUtils.isEmpty(dto.getNewAdminUserId())){
				throw new CuscUserException(CheckChangeAdminEnum.NEW_ADMIN_USER_ID.getCode(),CheckChangeAdminEnum.NEW_ADMIN_USER_ID.getMsg());
			}
			//选中的员工 赋值新的角色管理员
			UserDTO newAdminDto = new UserDTO();
			newAdminDto.setUuid(dto.getNewAdminUserId());
			newAdminDto.setTenantNo(UserSubjectUtil.getTenantNo());
			newAdminDto.setApplicationId(applicationId);
			newAdminDto.setOrdinaryAdmin(dto.getOrdinaryAdmin());
			newAdminDto.setIsTenantAdmin(1);
			//更新用户表数据 eiam_user
			Response updateResponse = userClient.update(newAdminDto);
			if (!updateResponse.isSuccess()) {
				throw new CuscUserException(updateResponse.getCode(),updateResponse.getMsg());
			}

			UserRoleDTO newUserRoleAdminDto = new UserRoleDTO();
			newUserRoleAdminDto.setUserId(dto.getNewAdminUserId());
			newUserRoleAdminDto.setRoleId(userService.getRoleId(OrdinaryAdminEnum.CAR_FACTORY.getCode() == dto.getOrdinaryAdmin() || OrdinaryAdminEnum.ORGAN.getCode() == dto.getOrdinaryAdmin()? RoleTyeEnum.SON_FACTORY.getCode():RoleTyeEnum.DISTRIBUTOR.getCode()));
			//更新用户和角色关系 eiam_user_role
			userRoleClient.updateByUserId(newUserRoleAdminDto);
		}

		UserOrganDTO userOrganDTO = new UserOrganDTO();
		userOrganDTO.setOrganId(dto.getUuid());
		Response<List<UserOrganDTO>> queryByList = userOrganClient.queryByList(userOrganDTO);

		if(!queryByList.isSuccess()){
			throw new CuscUserException(queryByList.getCode(),queryByList.getMsg());
		}
		List<UserOrganDTO> organizationDTOList = queryByList.getData();
		List<String> uuidList = organizationDTOList.stream().map(UserOrganDTO::getUserId).collect(Collectors.toList());
		UserDTO userDTO = new UserDTO();
		userDTO.setUuidList(uuidList);
		userDTO.setApplicationId(applicationId);
		userDTO.setTenantNo(UserSubjectUtil.getTenantNo());
		Response<List<UserSimpleDTO>> userSimpleList=userClient.query(userDTO);
		if(!userSimpleList.isSuccess()){
			throw new CuscUserException(userSimpleList.getCode(),userSimpleList.getMsg());
		}
		List<UserSimpleDTO> userSimpleDTOList = userSimpleList.getData();

		//查询之前的管理员
		List<UserSimpleDTO> oldAdminUserList = userSimpleDTOList.stream().filter(a -> a.getOrdinaryAdmin() > 0
				&& !a.getUuid().equals(dto.getNewAdminUserId())
		).collect(Collectors.toList());

		if (CollectionUtils.isEmpty(oldAdminUserList)) {
			log.info("未查询到其他管理员");
			return;
		}

		//取消当前账号管理员身份
		UserDTO cancelAdminDto = new UserDTO();
		cancelAdminDto.setUuid(oldAdminUserList.get(0).getUuid());
		cancelAdminDto.setTenantNo(oldAdminUserList.get(0).getTenantNo());
		cancelAdminDto.setApplicationId(applicationId);
		cancelAdminDto.setOrdinaryAdmin(0);
		cancelAdminDto.setIsTenantAdmin(0);
		userClient.update(cancelAdminDto);

		UserRoleDTO cancelRoleAdminDto = new UserRoleDTO();
		cancelRoleAdminDto.setTenantNo(UserSubjectUtil.getTenantNo());
		cancelRoleAdminDto.setApplicationId(applicationId);
		cancelRoleAdminDto.setUserId(oldAdminUserList.get(0).getUuid());
		//角色降级 车厂、车企子组织降级为102 ， 经销商降级为202
//		cancelRoleAdminDto.setRoleId(userService.getRoleId(OrdinaryAdminEnum.CAR_FACTORY.getCode() == dto.getOrdinaryAdmin() || OrdinaryAdminEnum.ORGAN.getCode() == dto.getOrdinaryAdmin()? RoleTyeEnum.UPDATE_CAR_FACTORY.getCode():RoleTyeEnum.UPDATE_DISTRIBUTOR.getCode()));

		userRoleClient.delBatchUser(cancelRoleAdminDto);

		//新增用户角色关系
		UserRoleDTO userRoleDto = new UserRoleDTO();
		userRoleDto.setUserId(oldAdminUserList.get(0).getUuid());
		userRoleDto.setRoleId(userService.getRoleId(OrdinaryAdminEnum.CAR_FACTORY.getCode() == dto.getOrdinaryAdmin() || OrdinaryAdminEnum.ORGAN.getCode() == dto.getOrdinaryAdmin()? RoleTyeEnum.UPDATE_CAR_FACTORY.getCode():RoleTyeEnum.UPDATE_DISTRIBUTOR.getCode()));
		userRoleDto.setApplicationId(applicationId);
		userRoleDto.setTenantNo(UserSubjectUtil.getTenantNo());
		userRoleClient.add(userRoleDto);

//		userRoleClient.updateByUserId(cancelRoleAdminDto);
	}
	public static Response checkNewAdminIsNull(OrganChangeAdminDto dto){
		if(StringUtils.isEmpty(dto.getCreateAdminAccount())){
			throw new CuscUserException(CheckChangeAdminEnum.CREATE_ADMIN_ACCOUNT.getCode(),CheckChangeAdminEnum.CREATE_ADMIN_ACCOUNT.getMsg());
		}
		if(StringUtils.isEmpty(dto.getCreateAdminUserName())){
			throw new CuscUserException(CheckChangeAdminEnum.CREATE_ADMIN_NAME.getCode(),CheckChangeAdminEnum.CREATE_ADMIN_NAME.getMsg());
		}
		if(StringUtils.isEmpty(dto.getCreateAdminPassword())){
			throw new CuscUserException(CheckChangeAdminEnum.CREATE_ADMIN_PASSWORD.getCode(),CheckChangeAdminEnum.CREATE_ADMIN_PASSWORD.getMsg());
		}
		if(StringUtils.isEmpty(dto.getCreateAdminPhone())){
			throw new CuscUserException(CheckChangeAdminEnum.CREATE_ADMIN_PHONE.getCode(),CheckChangeAdminEnum.CREATE_ADMIN_PHONE.getMsg());
		}
		return null;
	}
	@Override
	public List<UserAccountDTO> queryUserAccount(OrganUserAccountDTO dto) {
		UserOrganDTO userOrganDTO = new UserOrganDTO();
		userOrganDTO.setOrganId(dto.getOrganId());
		Response<List<UserOrganDTO>> userOrganDTOResponse = userOrganClient.queryByList(userOrganDTO);
		if (!userOrganDTOResponse.isSuccess()) {
			throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(),ResponseCode.SYS_BUSY.getMsg());
		}
		List<UserOrganDTO> userOrganDTOList = userOrganDTOResponse.getData();

		List<String> userIdList=userOrganDTOList.stream().map(UserOrganDTO::getUserId).collect(Collectors.toList());
		UserDTO userDTO =new UserDTO();
		userDTO.setUuidList(userIdList);
		userDTO.setApplicationId(applicationId);
		userDTO.setTenantNo(dto.getTenantNo());

		Response<List<UserSimpleDTO>> userSimpleList= userClient.getByUuids(userDTO);

		if (!userSimpleList.isSuccess()) {
			throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(),ResponseCode.SYS_BUSY.getMsg());
		}

		List<UserAccountDTO> list= null;
		list =userSimpleList.getData().stream().map(a->{
			UserAccountDTO userAccountDTO = new UserAccountDTO();
			userAccountDTO.setUuid(a.getUuid());
			userAccountDTO.setFullName(a.getFullName());
			userAccountDTO.setUserName(a.getUserName());
			userAccountDTO.setTenantNo(a.getTenantNo());
			return userAccountDTO;
			}
		).collect(Collectors.toList());
		return list;
	}

	@Override
	public List<OrganDTO> queryCurrentSubOrgList() {
		String organId = UserSubjectUtil.getOrganId();
		OrganizationDTO dto = new OrganizationDTO();
		dto.setUuid(organId);
		dto.setTenantNo(UserSubjectUtil.getTenantNo());
		OrganizationDTO data = organizationClient.getByUuid(dto).getData();
		if (null == data) {
			return Collections.emptyList();
		}
		OrganizationDTO query = new OrganizationDTO();
		query.setQueryCode(data.getQueryCode());
		List<OrganizationDTO> organizationList = organizationClient.queryByList(query).getData();
		if (CollectionUtils.isEmpty(organizationList)){
			return Collections.emptyList();
		}

		return organizationList.stream().map(o->{
			OrganDTO organDTO = new OrganDTO();
			BeanUtils.copyProperties(o,organDTO);
			return organDTO;
		}).collect(Collectors.toList());
	}


	public String generateOrganCode() {
		try {
			Calendar calendar = Calendar.getInstance();
			int year = calendar.get(Calendar.YEAR);
			Long aLong = cacheFactory.getStringService().setValueIncr(RedisConstants.ORG_ORGAN_CODE_PREFIX + year, 1);
			return String.valueOf(year * 10000 + aLong);
		} catch (Exception e) {
			log.error("generateOrganCode has error",e);
		}
		return "";
	}

	//------------------私有方法区域
}
