package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.constants.CustomerTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.convert.PersonalRnrRequestConvert;
import com.cusc.nirvana.user.rnr.enterprise.dto.ConsignerInfoDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.LivenessCallbackReqDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrH5CallBackRespDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrH5ReqDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrH5RespDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrRequestDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.PersonalRnrResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VinDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IPersonalRnrService;
import com.cusc.nirvana.user.rnr.enterprise.service.IUserService;
import com.cusc.nirvana.user.rnr.enterprise.service.IVehicleCardService;
import com.cusc.nirvana.user.rnr.enterprise.util.FileUtil;
import com.cusc.nirvana.user.rnr.enterprise.util.SpringValidationUtil;
import com.cusc.nirvana.user.rnr.enterprise.util.ValidationUtil;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrClient;
import com.cusc.nirvana.user.rnr.fp.client.VinCardClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.constants.BizTypeEnum;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.constants.CertTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.GenderEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrFileType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrLiaisonType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderAuditTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderSourceEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrFileDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrLiaisonInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import com.cusc.nirvana.user.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 个人实名信息
 *
 * @author **
 */
@Service
@Slf4j
public class PersonalRnrServiceImpl implements IPersonalRnrService {

    @Autowired
    IVehicleCardService vehicleCardService;

    @Autowired
    SmsServiceImpl smsService;

    @Autowired
    FpRnrClient fpRnrClient;

    @Autowired
    VinCardClient vinCardClient;

    @Autowired
    CardUnBindClient cardUnBindClient;

    @Value("${rnr.h5.url:wwww}")
    private String rnrH5Url;

    @Resource
    private IUserService userService;

	@Value("${rnr.frontPersonH5CallBackUrl}")
	private String frontPersonH5CallBackUrl;


	@Value("${rnr.frontPersonPadCallBackUrl}")
	private String frontPersonPadCallBackUrl;

    @Override
    public Response<PersonalRnrResponseDTO> verifyVinCards(VinCardInfoDTO vinCardListDTO) {
        //调用公共验证接口
        Response<VerifyVinCardResponseDTO> response = vehicleCardService.verifyVinCard(vinCardListDTO);
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }

        //拼接消息体
        PersonalRnrResponseDTO personalRnrResponseDTO = new PersonalRnrResponseDTO();


        if (CollectionUtils.isEmpty(response.getData().getIccidVerifyResults())) {
            return Response.createError("ICCID和VIN不能为空");
        }

        List<String> msg = response.getData().getIccidVerifyResults().stream()
                .filter(ret -> !ret.isExist())
                .map(ret -> ret.getMsg())
                .collect(Collectors.toList());
        personalRnrResponseDTO.setResponseMsg(msg);

        if (msg.isEmpty()) {
            return Response.createSuccess(personalRnrResponseDTO);
        } else {
            return Response.createError(ResponseCode.INVALID_DATA.getMsg(), personalRnrResponseDTO);

        }
    }

    @Override
    public Response getIccidListByVin(VinDTO vinDTO) {
        IccidListDTO iccidListDTO = new IccidListDTO();
        Response<VinCardInfoDTO> response = vinCardClient.queryVinCard(vinDTO.getVin());
        if (!response.isSuccess()) {
            return response;
        }

        if (response.getData() == null || CollectionUtils.isEmpty(response.getData().getIccidList())) {
            iccidListDTO.setIccidList(Collections.emptyList());
        } else {
            iccidListDTO.setIccidList(response.getData().getIccidList());
        }

        return Response.createSuccess(iccidListDTO);
    }

    @Override
    public Response<PersonalRnrDTO> submitRnr(@RequestBody PersonalRnrRequestDTO requestDTO) {
        requestDTO.setRnrBizzTypeEnum(RnrBizzTypeEnum.Bind.getCode());
        if (requestDTO.getIccidList().stream().distinct().count() != requestDTO.getIccidList().size()) {
        	throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"请不要输入同样的iccid");
		}
        Response<RnrRelationDTO> rnrRelationDTOResponse = checkAndConvertToRelation(requestDTO, false);
        if (!rnrRelationDTOResponse.isSuccess()) {
            return Response.createError(rnrRelationDTOResponse.getMsg(), rnrRelationDTOResponse.getCode());
        }
        RnrRelationDTO rnrRelationDTO = rnrRelationDTOResponse.getData();
        Response rnrResponse;
        if (requestDTO.getCustomerType() == CustomerTypeEnum.USED_CAR_OWNER.getCode()) {
            rnrResponse = fpRnrClient.personSecondHandRnr(rnrRelationDTO.getOrder().getSerialNumber(), requestDTO.getRequestId(), rnrRelationDTO);
        } else {
            //新车车主实名，如果vin已经有实名的卡，走一车多卡绑定模式
            /*Response<List<MgRnrCardInfoDTO>> cardListResponse = cardUnBindClient.getCardList(new FpVehicleCardRnrDTO().setVin(requestDTO.getVin()));
            if (!cardListResponse.isSuccess()) {
                return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
            }

            if(!CollectionUtils.isEmpty(cardListResponse.getData())){
                return Response.createError("VIN【" + requestDTO.getVin() + "】存在已实名的卡，请走一车多卡进行绑定",ResponseCode.INVALID_DATA.getCode());
            }*/

            rnrResponse = fpRnrClient.personRnr(rnrRelationDTO.getOrder().getSerialNumber(), requestDTO.getRequestId(), rnrRelationDTO);
        }

        if (rnrResponse.isSuccess()) {
            return Response.createSuccess(new PersonalRnrDTO((Integer) rnrResponse.getData()));
        }

        return rnrResponse;
    }

    /**
     * 检查参数并转换
     *
     * @param requestDTO
     * @return
     */
    @Override
    public Response<RnrRelationDTO> checkAndConvertToRelation(PersonalRnrRequestDTO requestDTO, boolean unbind) {
        log.info("Param checkAndConvertToRelation:{}", JSON.toJSONString(requestDTO));

        //手机验证码验证
        Response response;

        //手机验证码验证去掉
       /* SmsRequestDTO smsRequestDTO = new SmsRequestDTO();
        if (requestDTO.getIsConsigner()) {
            smsRequestDTO.setPhone(requestDTO.getConsignerInfo().getPhone());
            smsRequestDTO.setCaptcha(requestDTO.getConsignerInfo().getVerificationCode());
        } else {
            smsRequestDTO.setPhone(requestDTO.getPhone());
            smsRequestDTO.setCaptcha(requestDTO.getVerificationCode());
        }
        smsRequestDTO.setBizType(BizTypeEnum.RNR.getCode());
        smsRequestDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        response = smsService.checkSmsCaptcha(smsRequestDTO);
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }*/


        //参数必填校验
        response = validRnrInfo(requestDTO, unbind);
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }

//        //车卡信息验证
        VinCardInfoDTO vinCardListDTO = new VinCardInfoDTO();
        vinCardListDTO.setVin(requestDTO.getVin());
        vinCardListDTO.setIccidList(requestDTO.getIccidList());


        response = verifyVinCards(vinCardListDTO);


        if (!response.isSuccess()) {
            if (StringUtils.equals(response.getMsg(), ResponseCode.INVALID_DATA.getMsg())) {
                //返回第一个错误
                return Response.createError(((PersonalRnrResponseDTO) response.getData()).getResponseMsg().get(0), response.getCode());
            }
            return Response.createError(response.getMsg(), response.getCode());
        }

        return Response.createSuccess(convertToRelationDTO(requestDTO));
    }

    /**
     * 获取活体认证code接口
     *
     * @return
     */
    @Override
    public Response<LivenessCodeResponseDTO> getLivenessCode() {
        LivenessCodeRequestDTO livenessCodeRequestDTO = new LivenessCodeRequestDTO();
        livenessCodeRequestDTO.setSerialNumber(CuscStringUtils.generateUuid());
        String tenantNo = UserSubjectUtil.getTenantNo();
        livenessCodeRequestDTO.setTenantNo(tenantNo);
        return fpRnrClient.getLivenessCode(livenessCodeRequestDTO);
    }

    @Override
    public PersonalRnrH5RespDTO submitRnrH5(PersonalRnrH5ReqDTO bean) {
        bean.checkParam();
		if (bean.getIccidList().stream().distinct().count() != bean.getIccidList().size()) {
			throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"请不要输入同样的iccid");
		}

		SmsRequestDTO smsRequestDTO = new SmsRequestDTO();
		if (bean.getIsConsigner()) {
			smsRequestDTO.setPhone(bean.getConsignerInfo().getPhone());
			smsRequestDTO.setCaptcha(bean.getConsignerInfo().getVerificationCode());
		} else {
			smsRequestDTO.setPhone(bean.getPhone());
			smsRequestDTO.setCaptcha(bean.getVerificationCode());
		}
		smsRequestDTO.setBizType(BizTypeEnum.RNR.getCode());
		smsRequestDTO.setTenantNo(UserSubjectUtil.getTenantNo());
		Response smsResponse = smsService.checkSmsCaptcha(smsRequestDTO);
		if (!smsResponse.isSuccess()) {
			throw new CuscUserException(smsResponse.getCode(), smsResponse.getMsg());
		}


        PersonalRnrRequestDTO personalRnrRequestDTO = PersonalRnrRequestConvert.INSTANCE.h5PersonRnrRequestToPersonRequstDTO(bean);
        personalRnrRequestDTO.setRnrBizzTypeEnum(RnrBizzTypeEnum.Bind.getCode());
        Response<RnrRelationDTO> rnrRelationDTOResponse = checkAndConvertToRelation(personalRnrRequestDTO, false);

        if (!rnrRelationDTOResponse.isSuccess()) {
            throw new CuscUserException(rnrRelationDTOResponse.getCode(), rnrRelationDTOResponse.getMsg());
        }

        RnrRelationDTO data = rnrRelationDTOResponse.getData();
        data.getOrder().setOrderSource(RnrOrderSourceEnum.H5.getCode());

        log.info("PersonalRnrServiceImpl h5提交参数 data = {}", JSONObject.toJSONString(data));

        //先不存工单系统
        Response response = fpRnrClient.personRnrH5(data.getOrder().getSerialNumber(), data);
        if (!response.isSuccess()) {
            throw new CuscUserException(response.getCode(), response.getMsg());
        }

        PersonalRnrH5RespDTO h5RespDTO = new PersonalRnrH5RespDTO();
        h5RespDTO.setOrderId(data.getOrder().getUuid());
        LiveLoginReqDTO loginUrlDto = new LiveLoginReqDTO();
        loginUrlDto.setOrderNo(data.getOrder().getUuid());

        if ("PAD".equals(bean.getSource())) {
			loginUrlDto.setCallBackUrl(frontPersonPadCallBackUrl);
		} else {
			loginUrlDto.setCallBackUrl(frontPersonH5CallBackUrl);
		}

        //todo 调用云端接口
        Response<LiveLoginRespDTO> h5LiveLoginUrl = fpRnrClient.h5LiveLoginUrl(loginUrlDto);
        if (h5LiveLoginUrl.isSuccess()) {
            h5RespDTO.setH5LivenessUrl(h5LiveLoginUrl.getData().getH5LiveLoginUrl());
        }
        h5RespDTO.setRnrId(data.getInfo().getUuid());
        return h5RespDTO;
    }

    @Override
    public PersonalRnrH5CallBackRespDTO afreshLivenessUrl(LivenessCallbackReqDTO bean) {
        boolean success = bean.getCode().equals("0");

        PersonalRnrH5CallBackRespDTO reponse = new PersonalRnrH5CallBackRespDTO();
        if (success) {
            //活体成功
            PersonH5CallBackRequestDTO callBackDto = new PersonH5CallBackRequestDTO();
            callBackDto.setOrderNo(bean.getOrderNo());
            callBackDto.setTenantNo(UserSubjectUtil.getTenantNo());

            //调用云端接口
            LiveLoginReqDTO liveLoginReqDTO = new LiveLoginReqDTO();
            liveLoginReqDTO.setOrderNo(bean.getOrderNo());
            Response<LivenessResultDTO> livenessResult = fpRnrClient.h5LivenessResult(liveLoginReqDTO);
            if (null == livenessResult || !livenessResult.isSuccess() || null == livenessResult.getData()) {
                //没有查询到 活体视屏 和 照片 再次拉起腾讯活体H5
                reponse.setStatus(2);
                LiveLoginReqDTO loginUrlDto = new LiveLoginReqDTO();
                loginUrlDto.setOrderNo(bean.getOrderNo());

				if ("PAD".equals(bean.getSource())) {
					loginUrlDto.setCallBackUrl(frontPersonPadCallBackUrl);
				} else {
					loginUrlDto.setCallBackUrl(frontPersonH5CallBackUrl);
				}

                //todo 调用云端接口
                Response<LiveLoginRespDTO> h5LiveLoginUrl = fpRnrClient.h5LiveLoginUrl(loginUrlDto);
                if (h5LiveLoginUrl.isSuccess()) {
                    reponse.setH5LivenessUrl(h5LiveLoginUrl.getData().getH5LiveLoginUrl());
                }
                return reponse;
            }

            LivenessResultDTO livenessResultData = livenessResult.getData();
            String[] livenessPhotos = new String[2];
            livenessPhotos[0] = livenessResultData.getPhotoId();
            livenessPhotos[1] = livenessResultData.getPhotoId();
            callBackDto.setLivenessPicFileId(livenessPhotos);
            callBackDto.setVideFileId(livenessResultData.getVideoFileId());
            Response<Integer> response = fpRnrClient.personH5CallBack(callBackDto);
            if (!response.isSuccess() || null == response.getData()) {
                throw new CuscUserException(response.getCode(), response.getMsg());
            }
            Integer data = response.getData();
            reponse.setStatus(data);
        } else {
            //活体失败
            reponse.setStatus(2);
            LiveLoginReqDTO loginUrlDto = new LiveLoginReqDTO();
            loginUrlDto.setOrderNo(bean.getOrderNo());
			if ("PAD".equals(bean.getSource())) {
				loginUrlDto.setCallBackUrl(frontPersonPadCallBackUrl);
			} else {
				loginUrlDto.setCallBackUrl(frontPersonH5CallBackUrl);
			}

            Response<LiveLoginRespDTO> h5LiveLoginUrl = fpRnrClient.h5LiveLoginUrl(loginUrlDto);
            if (h5LiveLoginUrl.isSuccess()) {
                reponse.setH5LivenessUrl(h5LiveLoginUrl.getData().getH5LiveLoginUrl());
            }

            //需要修改数据状态
            //RnrOrderDTO orderDTO = new RnrOrderDTO();
            //orderDTO.setUuid(bean.getOrderNo());
            //fpRnrClient.updateRnrInValid(orderDTO);
        }
        return reponse;
    }

    //-------------私有方法区----------------------


    /**
     * 转成实名信息
     *
     * @param requestDTO 请求消息
     */
    private RnrRelationDTO convertToRelationDTO(PersonalRnrRequestDTO requestDTO) {
        String rnrID = CuscStringUtils.generateUuid();

        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();
        String tenantNo = UserSubjectUtil.getTenantNo();
        //租户
        rnrRelationDTO.setTenantNo(tenantNo);
        rnrRelationDTO.setIsTrust(requestDTO.getIsConsigner() ? 1 : 0);
        rnrRelationDTO.setIsSecondHandCar(requestDTO.getCustomerType() == 1 ? 1 : 0);

        //查询当前用户的组织id
        String orgId = userService.getOrganId(UserSubjectUtil.getUserId(),tenantNo);

        //实名实体信息
        MgRnrInfoDTO rnrInfoDTO = getMgRnrInfoDTO(requestDTO, rnrID);
        rnrInfoDTO.setOrgId(orgId);
        rnrRelationDTO.setInfo(rnrInfoDTO);

        //实名工单信息
        RnrOrderDTO orderDTO = getMgRnrOrderDTO(requestDTO, rnrRelationDTO);
        orderDTO.setOrgId(orgId);
        orderDTO.setTenantNo(tenantNo);
        rnrRelationDTO.setOrder(orderDTO);

        //实名卡信息
        rnrRelationDTO.setCardList(getMgRnrCardDTOs(requestDTO, rnrRelationDTO));
        //联系人信息
        if (requestDTO.getIsConsigner()) {
            rnrRelationDTO.setRnrLiaisonList(getMgRnrLiaisonDTOs(requestDTO, rnrRelationDTO));
        }
        //实名文件信息
        rnrRelationDTO.setRnrFileList(getMgRnrFileDTOs(requestDTO, rnrRelationDTO));
        return rnrRelationDTO;
    }

    /**
     * 创建工单信息
     *
     * @param requestDTO     请求消息
     * @param rnrRelationDTO 实名消息
     */
    private RnrOrderDTO getMgRnrOrderDTO(PersonalRnrRequestDTO requestDTO, RnrRelationDTO rnrRelationDTO) {
        //二手车、或者证书类型不是身份证都是手工
        boolean sendWorkOrder = isSendWorkOrder(requestDTO);
        boolean isSecondHandCar = rnrRelationDTO.getIsSecondHandCar() == 1;

        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderType(isSecondHandCar ? RnrOrderType.SEC_VEHICLE.getCode() : RnrOrderType.NEW_VEHICLE.getCode());
        rnrOrderDTO.setRnrId(rnrRelationDTO.getInfo().getUuid());
        rnrOrderDTO.setAuditType(sendWorkOrder ? RnrOrderAuditTypeEnum.MANUAL.getCode() : RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        rnrOrderDTO.setAutoRnr(!sendWorkOrder);
        rnrOrderDTO.setIsBatchOrder(0);
        rnrOrderDTO.setSerialNumber(rnrRelationDTO.getInfo().getSerial_number());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.PORTAL.getCode());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setSendWorkOrder(sendWorkOrder);
        rnrOrderDTO.setOrderStatus(sendWorkOrder ? RnrOrderStatusEnum.ASSIGNMENT.getCode() : RnrOrderStatusEnum.PASS.getCode());
        rnrOrderDTO.setRnrBizzType(requestDTO.getRnrBizzTypeEnum());
        return rnrOrderDTO;
    }

    /**
     * 判断是否是是否需要发送工单
     *
     * @param requestDTO
     * @return
     */
    private boolean isSendWorkOrder(PersonalRnrRequestDTO requestDTO) {
        //二手车实名，委托人模式，新车车主非身份证模式都需要发送工单
        return requestDTO.getCustomerType() == CustomerTypeEnum.USED_CAR_OWNER.getCode()
                || requestDTO.getIsConsigner()
                || !StringUtils.equalsIgnoreCase(requestDTO.getCertType(), CertTypeEnum.IDCARD.getCode());
    }

    /**
     * 获取联系人信息
     *
     * @param requestDTO 请求消息
     */
    private List<MgRnrLiaisonInfoDTO> getMgRnrLiaisonDTOs(PersonalRnrRequestDTO requestDTO, RnrRelationDTO rnrRelationDTO) {
        List<MgRnrLiaisonInfoDTO> liaisonInfoDTOS = new ArrayList<>();
        ConsignerInfoDTO consignerInfo = requestDTO.getConsignerInfo();
        MgRnrLiaisonInfoDTO mgRnrLiaisonInfoDTO = new MgRnrLiaisonInfoDTO();
        mgRnrLiaisonInfoDTO.setLiaisonName(consignerInfo.getFullName());
        mgRnrLiaisonInfoDTO.setLiaisonPhone(consignerInfo.getPhone());
        mgRnrLiaisonInfoDTO.setLiaisonCertAddress(consignerInfo.getCertAddress());
        mgRnrLiaisonInfoDTO.setLiaisonCertNumber(consignerInfo.getCertNumber());
        mgRnrLiaisonInfoDTO.setLiaisonCertType(consignerInfo.getCertType());
        mgRnrLiaisonInfoDTO.setLiaisonContactAddress(consignerInfo.getContactAddress());
        mgRnrLiaisonInfoDTO.setLiaisonType(RnrLiaisonType.CONSIGNEE.getCode());
        mgRnrLiaisonInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        mgRnrLiaisonInfoDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrLiaisonInfoDTO.setRnrId(rnrRelationDTO.getInfo().getUuid());
        mgRnrLiaisonInfoDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrLiaisonInfoDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrLiaisonInfoDTO.setRnrBizzType(requestDTO.getRnrBizzTypeEnum());
        mgRnrLiaisonInfoDTO.setLiaisonExpiredDate(requestDTO.getCertExpirationDate());
		mgRnrLiaisonInfoDTO.setLiaisonGender(consignerInfo.getGender());
		mgRnrLiaisonInfoDTO.setLiaisonExpiredDate(consignerInfo.getCertExpirationDate());
        liaisonInfoDTOS.add(mgRnrLiaisonInfoDTO);
        return liaisonInfoDTOS;
    }

    /**
     * 获取实名文件信息
     *
     * @param requestDTO 请求消息
     */
    private List<MgRnrFileDTO> getMgRnrFileDTOs(PersonalRnrRequestDTO requestDTO, RnrRelationDTO rnrRelationDTO) {
        List<MgRnrFileDTO> mgRnrFileDTOS = new ArrayList<>();

        String rnrId = rnrRelationDTO.getInfo().getUuid();

        int rnrBizzTypeEnum = requestDTO.getRnrBizzTypeEnum();

        //证件照片
        List<String> certPic = requestDTO.getCertPic();
        for (int i = 0; i < certPic.size(); i++) {
            String pic = certPic.get(i);
            Integer fileType = FileUtil.getFileType(requestDTO.getCertType(), i);
            if (fileType != null) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, pic, fileType, rnrBizzTypeEnum));
            }
        }

        if (requestDTO.getIsConsigner()) {
            String consignerId = rnrRelationDTO.getRnrLiaisonList().get(0).getUuid();
            //委托人照片
            List<String> consignerPic = requestDTO.getConsignerInfo().getCertPic();
            for (int i = 0; i < consignerPic.size(); i++) {
                String pic = consignerPic.get(i);
                Integer fileType = FileUtil.getFileType(requestDTO.getConsignerInfo().getCertType(), i);
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, pic, fileType, consignerId, rnrBizzTypeEnum));
            }
            //委托书
            for (String attorney : requestDTO.getConsignerInfo().getAttorneyLetterPic()) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, attorney, RnrFileType.LETTER_ATTORNEY.getCode(), consignerId, rnrBizzTypeEnum));
            }
            //活体视频
            if (StringUtils.isNotBlank(requestDTO.getLiveVerificationVideo())) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, requestDTO.getLiveVerificationVideo(), RnrFileType.LIVENESS_VIDEO.getCode(), consignerId, rnrBizzTypeEnum));
            }
        } else {
            //活体视频
            if (StringUtils.isNotBlank(requestDTO.getLiveVerificationVideo())) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, requestDTO.getLiveVerificationVideo(), RnrFileType.LIVENESS_VIDEO.getCode(), rnrBizzTypeEnum));
            }
        }

        //二手车，购车合同，购车发票，过户合同
        if (requestDTO.getCustomerType() == CustomerTypeEnum.USED_CAR_OWNER.getCode()) {
            if (!CollectionUtils.isEmpty(requestDTO.getPurchaseInvoicePic())) {
                for (String invoice : requestDTO.getPurchaseInvoicePic()) {
                    mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, invoice, RnrFileType.CAR_PURCHASE_INVOICE.getCode(), rnrBizzTypeEnum));
                }
            }

            for (String contract : requestDTO.getPurchaseContractPic()) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, contract, RnrFileType.CAR_PURCHASE_CONTRACT.getCode(), rnrBizzTypeEnum));
            }

            for (String transfer : requestDTO.getTransferCertificatePic()) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, transfer, RnrFileType.CAR_TRANSFER_CERTIFICATE.getCode(), rnrBizzTypeEnum));
            }
        }

        //入网合同
        if (!CollectionUtils.isEmpty(requestDTO.getContractPic())) {
            for (String contractPic : requestDTO.getContractPic()) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, contractPic, RnrFileType.VEHUCLE_BIND.getCode(), rnrBizzTypeEnum));
            }
        }

        //责任告知书
		if (!CollectionUtils.isEmpty(requestDTO.getDutyPic())) {
        	for (String duty:requestDTO.getDutyPic()) {
				mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId,duty, RnrFileType.DUTY_FILE.getCode(), rnrBizzTypeEnum));
			}
		}

        return mgRnrFileDTOS;
    }

    /**
     * 获取实名卡信息
     *
     * @param requestDTO 请求消息
     */
    private List<MgRnrCardInfoDTO> getMgRnrCardDTOs(PersonalRnrRequestDTO requestDTO, RnrRelationDTO rnrRelationDTO) {
        List<String> iccidList = requestDTO.getIccidList();
        List<MgRnrCardInfoDTO> mgRnrCardInfoDTOS = new ArrayList<>();
        MgRnrCardInfoDTO mgRnrCardInfoDTO;
        for (String iccid : iccidList) {
            mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
            mgRnrCardInfoDTO.setUuid(CuscStringUtils.generateUuid());
            mgRnrCardInfoDTO.setIccid(iccid);
            mgRnrCardInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
            mgRnrCardInfoDTO.setIotId(requestDTO.getVin());
            mgRnrCardInfoDTO.setRnrId(rnrRelationDTO.getInfo().getUuid());
            mgRnrCardInfoDTO.setOrderId(rnrRelationDTO.getOrder().getUuid());
            mgRnrCardInfoDTO.setCreator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setOperator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setRnrBizzType(requestDTO.getRnrBizzTypeEnum());
            mgRnrCardInfoDTOS.add(mgRnrCardInfoDTO);
        }
        return mgRnrCardInfoDTOS;
    }

    /**
     * 获取实名主体信息
     *
     * @param requestDTO 请求消息
     */
    private MgRnrInfoDTO getMgRnrInfoDTO(PersonalRnrRequestDTO requestDTO, String rnrID) {
        MgRnrInfoDTO mgRnrInfoDTO = PersonalRnrRequestConvert.INSTANCE.requestDTOToMgRnrDTO(requestDTO);
        mgRnrInfoDTO.setIsCompany(0);
        mgRnrInfoDTO.setIsTrust(requestDTO.getIsConsigner() ? 1 : 0);
        mgRnrInfoDTO.setIsSecondHandCar(Objects.equals(requestDTO.getCustomerType(), CustomerTypeEnum.NEW_CAR_OWNER.getCode()) ? 0 : 1);
        mgRnrInfoDTO.setUuid(rnrID);
        mgRnrInfoDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setSerial_number(CuscStringUtils.generateUuid());
        mgRnrInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        mgRnrInfoDTO.setRnrStatus(RnrStatus.INIT.getCode());
        mgRnrInfoDTO.setRnrBizzType(requestDTO.getRnrBizzTypeEnum());
		mgRnrInfoDTO.setEffectiveDate(DateUtils.parseDate(requestDTO.getCertEffectiveDate(),"yyyy-MM-dd"));
        mgRnrInfoDTO.setExpiredDate(requestDTO.getCertExpirationDate());
        return mgRnrInfoDTO;
    }

    /**
     * 进行参数验证
     *
     * @param requestDTO 请求消息
     */
    private Response validRnrInfo(PersonalRnrRequestDTO requestDTO, boolean unbind) {
//        if (StringUtils.isBlank(requestDTO.getPurchaseContract())) {
//            return Response.createError("购车合同不能为空", ResponseCode.INVALID_DATA.getCode());
//        }
//        if (StringUtils.isBlank(requestDTO.getPurchaseInvoice())) {
//            return Response.createError("购车证明不能为空", ResponseCode.INVALID_DATA.getCode());
//        }
        //检查手机
        Response response = ValidationUtil.checkPhone(requestDTO.getPhone());
        if (!response.isSuccess()) {
            return response;
        }

        //性别校验
        response = validGender(requestDTO.getGender());
        if (!response.isSuccess()) {
            return response;
        }

        //校验证件号码
        response = ValidationUtil.checkIdentifyNumber(requestDTO.getCertType(), requestDTO.getCertNumber());
        if (!response.isSuccess()) {
            return response;
        }

        //证件照数量验证
        response = ValidationUtil.checkIdentifyPics(requestDTO.getCertType(), requestDTO.getCertPic().size());
        if (!response.isSuccess()) {
            return response;
        }

        //二手车车主，需要有过户证明
        if (requestDTO.getCustomerType() == CustomerTypeEnum.USED_CAR_OWNER.getCode()) {
            if (CollectionUtils.isEmpty(requestDTO.getTransferCertificatePic())) {
                return Response.createError("过户证明不能为空", ResponseCode.INVALID_DATA.getCode());
            }
            if (CollectionUtils.isEmpty(requestDTO.getPurchaseContractPic())) {
                return Response.createError("购车合同不能为空", ResponseCode.INVALID_DATA.getCode());
            }
            if (!unbind && CollectionUtils.isEmpty(requestDTO.getPurchaseInvoicePic())) {
                return Response.createError("购车发票不能为空", ResponseCode.INVALID_DATA.getCode());
            }
        }

        if (requestDTO.getIsConsigner()) {
            //验证委托人关系
            ConsignerInfoDTO consignerInfo = requestDTO.getConsignerInfo();

            Set<ConstraintViolation<ConsignerInfoDTO>> violations = SpringValidationUtil.groupVerificationParameters(consignerInfo);
            if (!violations.isEmpty()) {
                return Response.createError(violations.stream().findFirst().get().getMessage());
            }

            //性别校验
            response = validGender(consignerInfo.getGender());
            if (!response.isSuccess()) {
                return response;
            }

            //校验证件号码
            response = ValidationUtil.checkIdentifyNumber(consignerInfo.getCertType(), consignerInfo.getCertNumber());
            if (!response.isSuccess()) {
                return response;
            }

            //证件照数量验证
            response = ValidationUtil.checkIdentifyPics(consignerInfo.getCertType(), consignerInfo.getCertPic().size());
            if (!response.isSuccess()) {
                return response;
            }

            //检查手机
            response = ValidationUtil.checkPhone(consignerInfo.getPhone());
            if (!response.isSuccess()) {
                return response;
            }
        } else {
            //没有委托人，车主验证码必填
            if (StringUtils.isBlank(requestDTO.getVerificationCode())) {
                return Response.createError("手机验证码不能为空", ResponseCode.INVALID_DATA.getCode());
            }
        }

        return Response.createSuccess();
    }

    /**
     * 校验性别
     *
     */
    private Response validGender(Integer gender) {
        if (gender == null || (gender != GenderEnum.FEMALE.getCode()
                && gender != GenderEnum.MALE.getCode())) {
            return Response.createError("性别格式错误", ResponseCode.INVALID_DATA.getCode());
        }
        return Response.createSuccess();
    }

    @Override
    public Response<RnrResponseDTO> h5ValidationOCR(H5ValidationOCRReqDTO bean) {
        bean.setIsIdentityCard(true);
        bean.setTenantNo(UserSubjectUtil.getTenantNo());
        bean.setBizUuid(RnrBizzTypeEnum.Bind.getCode().toString());
        bean.setSerialNumber(CuscStringUtils.generateUuid());
        log.info("h5ValidationOCR request :" + JSONObject.toJSONString(bean));
        Response<RnrResponseDTO> response = fpRnrClient.h5ValidationOCR(bean);
        log.info("h5ValidationOCR response :" + JSONObject.toJSONString(response));
        return response;
    }
}
