package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.client.UserClient;
import com.cusc.nirvana.user.eiam.client.UserOrganClient;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.eiam.dto.UserOrganDTO;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.config.EnterpriseConfig;
import com.cusc.nirvana.user.rnr.enterprise.constants.FileMouldEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.enterprise.dto.*;
import com.cusc.nirvana.user.rnr.enterprise.service.IProtocolManageService;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrProtocolManageClient;
import com.cusc.nirvana.user.rnr.fp.client.OrganClient;
import com.cusc.nirvana.user.rnr.fp.dto.FileRecordDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpRnrProtocolManageDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.checkerframework.checker.units.qual.K;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

/**
 * @className: ProtocolManageServiceImpl
 * @description: 协议管理
 * @author: jk
 * @date: 2022/6/1 13:55
 * @version: 1.0
 **/
@Service
@Slf4j
@RefreshScope
public class ProtocolManageServiceImpl implements IProtocolManageService {

    @Resource
    private FpRnrProtocolManageClient fpRnrProtocolManageClient;

    @Resource
    private UserClient userClient;

    @Value("${user.eiam.applicationId:4}")
    private String applicationId;

    @Value("${rnr.fileMouldUrl}")
    private String fileMouldUrl;

    @Resource
    private UserOrganClient userOrganClient;

    @Resource
    private FileSystemClient fileSystemClient;

    @Resource
    private OrganClient organClient;

    @Resource
    private EnterpriseConfig enterpriseConfig;

    @Resource
    private OrganizationClient organizationClient;

    @Override
    public void add(ProtocolManageDTO dto) {
        //根据当前登录人获取组织id
        UserOrganDTO userOrganDTO= this.queryUserOrgan();

        if(StringUtils.isEmpty(UserSubjectUtil.getUserId())){
            throw new CuscUserException(ResponseCode.DATA_NULL.getCode(), ResponseCode.DATA_NULL.getMsg());
        }
        //通过组织id出协议文件
        FpRnrProtocolManageDTO fpRnrProtocolManageDTO = new FpRnrProtocolManageDTO();
        fpRnrProtocolManageDTO.setOrgId(dto.getOrgId());
        Response<List<FpRnrProtocolManageDTO>> responseList = fpRnrProtocolManageClient.queryByList(fpRnrProtocolManageDTO);

        if (!responseList.isSuccess()) {
            throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(), ResponseCode.SYS_BUSY.getMsg());
        }
        if(!CollectionUtils.isEmpty(responseList.getData())){
            throw new CuscUserException(ResponseCode.ProtocolManage_NOT_NULL.getCode(), ResponseCode.ProtocolManage_NOT_NULL.getMsg());
        }
        FpRnrProtocolManageDTO fpRnrProtocolManage = new FpRnrProtocolManageDTO();
        BeanUtils.copyProperties(dto,fpRnrProtocolManage);
        fpRnrProtocolManage.setCreator(UserSubjectUtil.getUserId());
        fpRnrProtocolManage.setTenantNo(UserSubjectUtil.getTenantNo());
        fpRnrProtocolManage.setOrgId(dto.getOrgId());
        log.warn("fpRnrProtocolManage入参{}",JSONObject.toJSONString(fpRnrProtocolManage));
        fpRnrProtocolManageClient.add(fpRnrProtocolManage);
    }

    @Override
    public void update(ProtocolManageUpdateDTO dto) {
        if(StringUtils.isEmpty(dto.getUuid())){
            throw new CuscUserException(ResponseCode.UUID_NULL.getCode(), ResponseCode.UUID_NULL.getMsg());
        }
        //获取当前登录人数据
        UserOrganDTO userOrganDTO= this.queryUserOrgan();
        FpRnrProtocolManageDTO fpRnrProtocolManageDTO =new FpRnrProtocolManageDTO();
        BeanUtils.copyProperties(dto,fpRnrProtocolManageDTO);
        fpRnrProtocolManageDTO.setOperator(UserSubjectUtil.getUserId());
        fpRnrProtocolManageDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        fpRnrProtocolManageDTO.setOrgId(dto.getOrgId());
        fpRnrProtocolManageClient.update(fpRnrProtocolManageDTO);
    }

    @Override
    public Response<List<ProtocolManageDTO>> query(ProtocolManageUpdateDTO dto) {

        UserOrganDTO userOrganDTO = this.queryUserOrgan();
        //没有登录直接展示最上级LOGO

        OrganizationDTO organizationDTO = new OrganizationDTO();
        organizationDTO.setUuid(dto.getOrgId());
        organizationDTO.setTenantNo(UserSubjectUtil.getTenantNo());

        Response<OrganizationDTO> organizationDTOResponse = organClient.getCarSubOrganByOrganId(organizationDTO);
        if (!organizationDTOResponse.isSuccess()) {
            throw new CuscUserException(organizationDTOResponse.getCode(), organizationDTOResponse.getMsg());

        }
        //通过组织id出协议文件
        FpRnrProtocolManageDTO fpRnrProtocolManageDTO = new FpRnrProtocolManageDTO();
        fpRnrProtocolManageDTO.setOrgId(organizationDTOResponse.getData().getUuid());
        fpRnrProtocolManageDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<List<FpRnrProtocolManageDTO>> responseList = fpRnrProtocolManageClient.queryByList(fpRnrProtocolManageDTO);


        if (!responseList.isSuccess()) {
            throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(), ResponseCode.SYS_BUSY.getMsg());
        }

        List<FpRnrProtocolManageDTO> rnrProtocolManageDTOList = responseList.getData();

        if (CollectionUtils.isEmpty(rnrProtocolManageDTOList)) {
            return Response.createSuccess(rnrProtocolManageDTOList);
        }

        FpRnrProtocolManageDTO fpRnrProtocolManage = rnrProtocolManageDTOList.get(0);

        ProtocolManageDTO protocolManageDTO = new ProtocolManageDTO();
        String orgId = fpRnrProtocolManage.getOrgId();

        ProtocolManageOneDTO protocolManageOneDTO = new ProtocolManageOneDTO();

        BeanUtils.copyProperties(fpRnrProtocolManage, protocolManageOneDTO);
        //将对象属性转为map
        Map<String, String> map = convertToMap(protocolManageOneDTO);

        Map<String, Object> mapReturn = new HashMap<>();
        //并行调用查询文件路径
        map.entrySet().stream().map(entry -> {

            if (!StringUtils.isEmpty(entry.getValue())) {
                FileDownloadDTO fileRecordDTO = new FileDownloadDTO();
                fileRecordDTO.setUuid(entry.getValue());
                Response<FileRecordDTO> fileRecordDTOResponse = fileSystemClient.getUrl(fileRecordDTO);
                if (!fileRecordDTOResponse.isSuccess()) {
                    throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(), ResponseCode.SYS_BUSY.getMsg());
                }
                ProtocolManageUrlDTO protocolManageUrlDTO = new ProtocolManageUrlDTO();
                protocolManageUrlDTO.setFileUrl(fileRecordDTOResponse.getData().getAccessUrl());
                //protocolManageUrlDTO.setFileName(fileRecordDTOResponse.getData().getFileName());
                protocolManageUrlDTO.setUuid(fileRecordDTOResponse.getData().getUuid());

                mapReturn.put(entry.getKey(), protocolManageUrlDTO);

            } else {
                mapReturn.put(entry.getKey(), "");
            }

            return mapReturn;
            //将路径塞进map对应的key中
//                entry.setValue(fileRecordDTOResponse.getData().getPath());
        }).collect(Collectors.toList());
        mapReturn.put("id", fpRnrProtocolManage.getId());
        mapReturn.put("uuid", fpRnrProtocolManage.getUuid());
        mapReturn.put("orgId", fpRnrProtocolManage.getOrgId());
        //通过反射将map key对应value塞进对应bean属性中
//            Object obj= toBean(map1,ProtocolManageDTO.class);
//            protocolManageDTO = (ProtocolManageDTO)obj;
//            protocolManageDTO.setOrgId(orgId);

        return Response.createSuccess(mapReturn);
    }

     /**
       * @author: jk
       * @description: 查询文件模板
       * @date: 2022/6/2 14:36
       * @version: 1.0
       * @Param:
       * @Return:
       */
    @Override
    public Response<Map> queryFileMould() {

        String[] fileMouldList = fileMouldUrl.split(",");

        Map<String,String> map = new HashMap<>();
        FileMouldEnum[]  fileMouldEnums = FileMouldEnum.getFileMouldMap();

        for(FileMouldEnum fileMouldEnum:fileMouldEnums){
            FileDownloadDTO fileRecordDTO = new FileDownloadDTO();
            fileRecordDTO.setUuid(fileMouldList[fileMouldEnum.getCode()]);
            Response<FileRecordDTO> fileRecordDTOResponse = fileSystemClient.getUrl(fileRecordDTO);
            if (!fileRecordDTOResponse.isSuccess()) {
                throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(), ResponseCode.SYS_BUSY.getMsg());
            }
            map.put(fileMouldEnum.getMsg(),fileRecordDTOResponse.getData().getAccessUrl());
        }
        return Response.createSuccess(map);
    }

    @Override
    public Response<List<ProtocolManageDTO>> noLogin() {
        UserOrganDTO userOrganDTO = this.queryUserOrgan();
        //没有登录直接展示最上级LOGO
        //通过组织id出协议文件
        OrganizationDTO organizationDTO = new OrganizationDTO();
        organizationDTO.setTenantNo(enterpriseConfig.getTenantNo());
        organizationDTO.setParentId("0");
        organizationDTO.setBizType(1);
        Response<List<OrganizationDTO>> re = organizationClient.queryByList(organizationDTO);
        if(!re.isSuccess()){
            throw new CuscUserException("","获取组织信息失败");
        }

        FpRnrProtocolManageDTO fpRnrProtocolManageDTO = new FpRnrProtocolManageDTO();
        fpRnrProtocolManageDTO.setTenantNo(enterpriseConfig.getTenantNo());
        fpRnrProtocolManageDTO.setType("1");

        Response<List<FpRnrProtocolManageDTO>> responseList = fpRnrProtocolManageClient.queryByList(fpRnrProtocolManageDTO);

        if (!responseList.isSuccess()) {
            throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(), ResponseCode.SYS_BUSY.getMsg());
        }

        List<FpRnrProtocolManageDTO> rnrProtocolManageDTOList = responseList.getData();

        if (CollectionUtils.isEmpty(rnrProtocolManageDTOList)) {
            return Response.createSuccess(rnrProtocolManageDTOList);
        }

        FpRnrProtocolManageDTO fpRnrProtocolManage = rnrProtocolManageDTOList.get(0);

        ProtocolManageDTO protocolManageDTO = new ProtocolManageDTO();
        String orgId = fpRnrProtocolManage.getOrgId();

        ProtocolManageOneDTO protocolManageOneDTO = new ProtocolManageOneDTO();

        BeanUtils.copyProperties(fpRnrProtocolManage, protocolManageOneDTO);
        //将对象属性转为map
        Map<String, String> map = convertToMap(protocolManageOneDTO);


        Map<String, Object> loginOutReturn = new HashMap<>();
        //并行调用查询文件路径
        map.entrySet().stream().map(entry -> {

            if (!StringUtils.isEmpty(entry.getValue())) {
                FileDownloadDTO fileRecordDTO = new FileDownloadDTO();
                fileRecordDTO.setUuid(entry.getValue());
                Response<FileRecordDTO> fileRecordDTOResponse = fileSystemClient.getUrl(fileRecordDTO);
                if (!fileRecordDTOResponse.isSuccess()) {
                    throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(), ResponseCode.SYS_BUSY.getMsg());
                }
                ProtocolManageUrlDTO protocolManageUrlDTO = new ProtocolManageUrlDTO();
                protocolManageUrlDTO.setFileUrl(fileRecordDTOResponse.getData().getAccessUrl());
                //protocolManageUrlDTO.setFileName(fileRecordDTOResponse.getData().getFileName());
                protocolManageUrlDTO.setUuid(fileRecordDTOResponse.getData().getUuid());

                if (StringUtils.isEmpty(UserSubjectUtil.getUserId()) && ("logoPc".equals(entry.getKey()) || "logoH5".equals(entry.getKey()))) {
                    loginOutReturn.put(entry.getKey(), protocolManageUrlDTO);
                }
            } else {
                loginOutReturn.put(entry.getKey(), "");
            }

            return loginOutReturn;
        }).collect(Collectors.toList());
        loginOutReturn.put("id", fpRnrProtocolManage.getId());
        loginOutReturn.put("uuid", fpRnrProtocolManage.getUuid());
        loginOutReturn.put("orgId", fpRnrProtocolManage.getOrgId());
        loginOutReturn.put("organName",re.getData().get(0).getOrganName());

        return Response.createSuccess(loginOutReturn);
    }

    //通过反射 将key映射到对象的bean中
    public static T toBean(Map beanPropMap, Class type) {
        try {
            T beanInstance = (T) type.getConstructor().newInstance();
            for (Object k : beanPropMap.keySet()) {
                String key = (String) k;
                Object value = beanPropMap.get(k);
                if (value != null) {
                    try {
                        Field field = type.getDeclaredField(key);
                        field.setAccessible(true);
                        field.set(beanInstance, value);
                        field.setAccessible(false);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
            return beanInstance;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void main(String[] args) {
        FpRnrProtocolManageDTO fpRnrProtocolManageDTO = new  FpRnrProtocolManageDTO();
        fpRnrProtocolManageDTO.setOrgId("11");
        fpRnrProtocolManageDTO.setTenantNo("222");
        fpRnrProtocolManageDTO.setCompanyAgreementHorizontal("1");
        fpRnrProtocolManageDTO.setCompanyAgreementVertical("2");
        fpRnrProtocolManageDTO.setCompanyAgreementWord("3");
        fpRnrProtocolManageDTO.setOtherHorizontal("4");

        ProtocolManageOneDTO protocolManageOneDTO =new ProtocolManageOneDTO();
        BeanUtils.copyProperties(fpRnrProtocolManageDTO,protocolManageOneDTO);

        Map<String, String> map = convertToMap(protocolManageOneDTO);

        Map<String,ProtocolManageUrlDTO> map1 = new HashMap<>();
        List<Map> list = new ArrayList<>();
        map.entrySet().stream().map(entry->{
            if(!StringUtils.isEmpty(entry.getValue())) {
                FileRecordDTO fileRecordDTO = new FileRecordDTO();
                fileRecordDTO.setUuid(entry.getValue());
                ProtocolManageUrlDTO protocolManageUrlDTO = new ProtocolManageUrlDTO();
                protocolManageUrlDTO.setFileUrl("fileRecordDTOResponse.getData().getPath()");
                protocolManageUrlDTO.setFileName("fileRecordDTOResponse.getData().getFileName()");
                map1.put(entry.getKey(),protocolManageUrlDTO);
                System.out.println(entry.getValue());
                System.out.println(entry.getKey());
                System.out.println(JSONObject.toJSONString(fileRecordDTO));
            }
            return map1;
        }).collect(Collectors.toList());
        list.add(map1);
        Object fpRnrProtocolManageDTO1=toBean(map, QueryProtocolManageDTO.class);
        fpRnrProtocolManageDTO = (FpRnrProtocolManageDTO)fpRnrProtocolManageDTO1;
//        mapToObject(map,fpRnrProtocolManageDTO);
        System.out.println(JSONObject.toJSONString(map));

    }

    //将java对象转为map为了方便调用文件查询接口
    public static Map<String, String> convertToMap(Object obj) {
        try {
            if (obj instanceof Map) {
                return (Map)obj;
            }
            Map<String, String> returnMap = org.apache.commons.beanutils.BeanUtils.describe(obj);

            returnMap.remove("class");
            return returnMap;
        } catch (IllegalAccessException e1) {
            e1.getMessage();
        } catch (InvocationTargetException e2) {
            e2.getMessage();
        } catch (NoSuchMethodException e3) {
            e3.getMessage();
        }
        return new HashMap();
    }

    public UserOrganDTO queryUserOrgan(){
        //根据当前登录人获取组织id
        UserOrganDTO userOrganDTO = new UserOrganDTO();
        userOrganDTO.setUserId(UserSubjectUtil.getUserId());

        Response<List<UserOrganDTO>> response = userOrganClient.queryByList(userOrganDTO);
        if (!response.isSuccess()) {
            throw new CuscUserException(ResponseCode.SYS_BUSY.getCode(),ResponseCode.SYS_BUSY.getMsg());
        }
        List<UserOrganDTO> userOrganDTOList = response.getData();
        if(CollectionUtils.isEmpty(userOrganDTOList)){
            throw new CuscUserException(ResponseCode.USER_ORGAN_NOT_FOUND.getCode(),ResponseCode.USER_ORGAN_NOT_FOUND.getMsg());
        }
        return userOrganDTOList.get(0);
    }



}
