package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.client.RoleClient;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.eiam.dto.UserOrganDTO;
import com.cusc.nirvana.user.eiam.dto.UserRoleDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.SearchCardAuthDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.ISearchCardAuthService;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import com.cusc.nirvana.user.rnr.fp.client.FpSearchCardAuthClient;
import com.cusc.nirvana.user.rnr.fp.client.OrganClient;
import com.cusc.nirvana.user.rnr.mg.client.MgCheckProgressClient;
import com.cusc.nirvana.user.rnr.mg.dto.*;
import com.cusc.nirvana.user.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 查询车辆认证进度
 * @modify 单车进度查询封装返回结果 2022-05-18
 */
@Service
@Slf4j
public class SearchCardAuthServiceImpl implements ISearchCardAuthService {

    @Resource
    private FpSearchCardAuthClient fpSearchCardAuthClient;

    @Resource
    private MgCheckProgressClient mgCheckProgressClient;

    @Resource
    private OrganClient organClient;

    @Resource
    private OrganizationClient organizationClient;

    @Resource
    private RoleClient roleClient;
    /**
     * 单车进度
     * @modify 查脱敏配置进行数据脱敏
     * @param bean
     * @return
     */
    @Override
    public Response<List<MgSearchCardAuthDTO>> searchAuth1(SearchCardAuthDTO bean) {
        MgRnrCardInfoDTO cardInfoDTO = new MgRnrCardInfoDTO();
        cardInfoDTO.setIotId(bean.getVin());
        Response<List<MgSearchCardAuthDTO>> response = fpSearchCardAuthClient.getMgSearchCardAuthDTO(cardInfoDTO);
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }
        List<MgSearchCardAuthDTO> cardAuthDTOList = response.getData();

        //查询脱敏配置 false 脱敏，  true 不脱敏
        Boolean hideSensitive = getHideSensitiveByUserId();

        List<MgCheckProgressDTO> collect = cardAuthDTOList.stream().map(cardAuthDTO -> {
            MgCheckProgressDTO checkProgressDTO = new MgCheckProgressDTO();

            if(hideSensitive){
                checkProgressDTO.setIotId(cardAuthDTO.getCardInfoDTO().getIotId());
                checkProgressDTO.setFullName(cardAuthDTO.getRnrInfoDTO().getFullName());
                checkProgressDTO.setPhone(cardAuthDTO.getRnrInfoDTO().getPhone());
            }else{
                checkProgressDTO.setIotId(DesensitizationUtil.desensitizeVin(cardAuthDTO.getCardInfoDTO().getIotId()));
                checkProgressDTO.setFullName(DesensitizationUtil.desensitizeName(cardAuthDTO.getRnrInfoDTO().getFullName()));
                checkProgressDTO.setPhone(DesensitizationUtil.desensitizePhone(cardAuthDTO.getRnrInfoDTO().getPhone()));
            }

            checkProgressDTO.setIccid(cardAuthDTO.getCardInfoDTO().getIccid());
            checkProgressDTO.setOrderType(String.valueOf(cardAuthDTO.getRnrOrderDTO().getOrderType()));
            checkProgressDTO.setOrderStatus(String.valueOf(cardAuthDTO.getRnrOrderDTO().getOrderStatus()));
            checkProgressDTO.setStartCheckDate(DateUtils.formatDatetime(cardAuthDTO.getRnrOrderDTO().getCreateTime()));
            checkProgressDTO.setCompanyName(cardAuthDTO.getCompanyInfoDTO()!=null?cardAuthDTO.getCompanyInfoDTO().getCompanyName():"");
            checkProgressDTO.setOrgName(cardAuthDTO.getOrgName());
            return checkProgressDTO;
        }).collect(Collectors.toList());
        return Response.createSuccess("查询成功！",collect);

    }

    /**
     * 审核列表
     * @param bean
     * @return
     */
    @Override
    public Response searchAuth2(SearchCardAuthDTO bean) {

        MgCheckProgressDTO checkProgressDTO = new MgCheckProgressDTO();

        //查询dto转换
        checkProgressDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        checkProgressDTO.setIotId(bean.getVin());
        checkProgressDTO.setIccid(bean.getIccid());
        checkProgressDTO.setFullName(bean.getUser());
        checkProgressDTO.setPhone(bean.getUserPhone());
        checkProgressDTO.setCompanyName(bean.getCompanyName());
        checkProgressDTO.setIsVehicleCompany(bean.getIsVehicleCompany());
        checkProgressDTO.setStartCheckBegin(bean.getStartSendCheck());
        checkProgressDTO.setStartCheckEnd(bean.getEndSendCheck());
        checkProgressDTO.setEndCheckBegin(bean.getStartCheck());
        checkProgressDTO.setEndCheckEnd(bean.getEndCheck());
        checkProgressDTO.setOrderType(bean.getOrderType());
        checkProgressDTO.setOrderStatus(bean.getCheckStatus());
        checkProgressDTO.setPageSize(bean.getPageSize());
        checkProgressDTO.setCurrPage(bean.getCurrPage());

        List<String> userOrgIdList = this.getUserOrgIdList();
        checkProgressDTO.setOrgIdList(userOrgIdList);

        Response<PageResult<MgCheckProgressDTO>> response = fpSearchCardAuthClient.getCheckProgressList(checkProgressDTO);

        //查询脱敏配置 false 脱敏，  true 不脱敏
        Boolean hideSensitive = getHideSensitiveByUserId();
        List<MgCheckProgressDTO> list = response.getData().getList();
        if(!hideSensitive){
            for (MgCheckProgressDTO mgCheckProgressDTO : list){
                mgCheckProgressDTO.setIotId(DesensitizationUtil.desensitizeVin(mgCheckProgressDTO.getIotId()));
                mgCheckProgressDTO.setFullName(DesensitizationUtil.desensitizeName(mgCheckProgressDTO.getFullName()));
                mgCheckProgressDTO.setPhone(DesensitizationUtil.desensitizePhone(mgCheckProgressDTO.getPhone()));
            }
        }

        return response;
    }

    /**
     * 审核看板统计
     * @return
     * @modify 2022-05-31 根据登录用户的所属车企的所有下属组织做筛选 注意带上租户编号
     */
    @Override
    public Response queryCheckStatistics() {

        //查询到当前用户的车企组织
        UserOrganDTO param = new UserOrganDTO();
        param.setUserId(UserSubjectUtil.getUserId());
        param.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<OrganizationDTO> organizationDTOResponse = organClient.getCarSubOrganByUserId(param);

        //再用上面查到的组织查询所有下级 queryByList接口可以查询到所有下级组织，参数传queryCode
        OrganizationDTO organizationDTO = new OrganizationDTO();
        organizationDTO.setQueryCode(organizationDTOResponse.getData().getQueryCode());
        Response<List<OrganizationDTO>> listResponse = organizationClient.queryByList(organizationDTO);
        List<OrganizationDTO> orgList = listResponse.getData();
        List<String> orgIdList = orgList.stream().map(OrganizationDTO::getUuid).collect(Collectors.toList());

        MgCheckStatisticsQueryDTO queryDTO = new MgCheckStatisticsQueryDTO();
        queryDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        queryDTO.setUserOrgId(organizationDTOResponse.getData().getUuid());
        queryDTO.setOrgIdList(orgIdList);
        Response<MgCheckStatisticsDTO> response = mgCheckProgressClient.queryCheckStatistics(queryDTO);
        return response;
    }

    private List<String> getUserOrgIdList(){
        //查询到当前用户的车企组织
        UserOrganDTO param = new UserOrganDTO();
        param.setUserId(UserSubjectUtil.getUserId());
        param.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<OrganizationDTO> organizationDTOResponse = organClient.getCarSubOrganByUserId(param);

        //再用上面查到的组织查询所有下级 queryByList接口可以查询到所有下级组织，参数传queryCode
        OrganizationDTO organizationDTO = new OrganizationDTO();
        organizationDTO.setQueryCode(organizationDTOResponse.getData().getQueryCode());
        Response<List<OrganizationDTO>> listResponse = organizationClient.queryByList(organizationDTO);
        List<OrganizationDTO> orgList = listResponse.getData();
        List<String> orgIdList = orgList.stream().map(OrganizationDTO::getUuid).collect(Collectors.toList());
        return orgIdList;
    }


    private Boolean getHideSensitiveByUserId(){
        UserRoleDTO userRoleDTO = new UserRoleDTO();
        userRoleDTO.setUserId(UserSubjectUtil.getUserId());
        userRoleDTO.setApplicationId(UserSubjectUtil.getAppId());
        userRoleDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<Boolean> booleanResponse = roleClient.queryHideSensitiveByUserId(userRoleDTO);
        Boolean data = booleanResponse.getData();
        return data;
    }
}
