package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.exception.ExcelAnalysisException;
import com.alibaba.excel.read.metadata.ReadSheet;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.rnr.enterprise.common.FileTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.FIleSystemType;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileUploadDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.ImportSimDTO;
import com.cusc.nirvana.user.rnr.enterprise.excel.*;
import com.cusc.nirvana.user.rnr.enterprise.service.ISimVehicleRelImportService;
import com.cusc.nirvana.user.rnr.enterprise.service.ISimVehicleService;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.client.SimVehicleRelClient;
import com.cusc.nirvana.user.rnr.fp.dto.FileRecordDTO;
import com.cusc.nirvana.user.rnr.mg.client.OrgSimRelClient;
import com.cusc.nirvana.user.rnr.mg.client.SimFileHistoryClient;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.util.Base64;
import java.util.List;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @author stayand
 */
@Service
@Slf4j
public class SimVehicleRelImportImportServiceImpl implements ISimVehicleRelImportService {

    private final static Logger LOGGER = LoggerFactory.getLogger(SimVehicleRelImportImportServiceImpl.class);

    /**核心线程数*/
    private final static int CORE_POOL_SIZE = Runtime.getRuntime().availableProcessors() * 2;

    private final static int MAX_POOL_SIZE = CORE_POOL_SIZE * 2;

    private final static long KEEP_ALIVE_TIME = 60L;

    @Value("${import.simrel.upload.maxFilesInQueue:100}")
    private int maxQueuedImportFiles;

    @Value("${import.simrel.upload.tmp-dir:}")
    private String tmpDir;

    @Value("${import.simrel.upload.error-excel-path:sim-export:}")
    private String errorExcelPath;

    @Value("${import.simrel.upload.max-record-count:5000}")
    private Integer maxRecordCount;

    @Autowired
    private SimFileHistoryClient simFileHistoryClient;

    @Autowired
    private SimVehicleRelClient simVehicleClient;

    @Autowired
    private ISimVehicleService simVehicleService;

    @Autowired
    private OrgSimRelClient orgSimRelClient;

    /**导入文件线程池*/
    private ThreadPoolExecutor executor;

    @PostConstruct
    public void init() {
        executor = new ThreadPoolExecutor(CORE_POOL_SIZE, MAX_POOL_SIZE, KEEP_ALIVE_TIME, TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(maxQueuedImportFiles));
    }

	@Resource
	private FileSystemClient fileSystemClient;

    @Override
    public Response excelProcess(ImportSimDTO importSimDTO) {
        if (executor.getQueue().size() >= maxQueuedImportFiles) {
            return Response.createError("处理文件失败，当前排队上传任务太多，请稍后重试...");
        }

        FileDownloadDTO fileDownloadDTO = new FileDownloadDTO();
        fileDownloadDTO.setUuid(importSimDTO.getFileUuid());
        fileDownloadDTO.setFileName(importSimDTO.getFileName());
        try {
                String base64 = fileSystemClient.getBase64(fileDownloadDTO).getData().getBase64();
			byte[] result = Base64.getDecoder().decode(base64);
			if (result == null || result.length == 0) {
				return Response.createError("没有找到车卡关系Excel文件");
			}
            SimFileHistoryDTO tmpHistoryDto = new SimFileHistoryDTO();
            if (importSimDTO.getFileType() == 1) {
                tmpHistoryDto.setTotalCount(0);
                //获取总行数
               getExcelTotalCount(result, tmpHistoryDto);
            }
            //如果是启用停用
            if(importSimDTO.getFileType()== FileTypeEnum.CARCLOSE.getCode()||importSimDTO.getFileType()== FileTypeEnum.CARSTARTUP.getCode()){
                tmpHistoryDto.setTotalCount(0);
                getExcelTotalCountStatus(result, tmpHistoryDto);
            }
            if (tmpHistoryDto.getTotalCount() > maxRecordCount) {
                return Response.createError("上传记录数不能大于["+maxRecordCount+"]");
            }
            SimFileHistoryDTO simFileHistoryDTO = insertImportFile(importSimDTO);
            simFileHistoryDTO.setTotalCount(tmpHistoryDto.getTotalCount());
            simFileHistoryDTO.setCreator(UserSubjectUtil.getUserId());

            executor.submit(new Runnable() {
                @Override
                public void run() {
                    asyncProcessExcel(result, simFileHistoryDTO, importSimDTO.getTagUuid());
                }
            });
            return Response.createSuccess("文件正在处理中...");
        }catch (Exception ex) {
            log.error("导入车卡关系失败,原因：" + ex.getMessage());
            return Response.createError("导入车卡关系失败，原因：" + ex.getMessage());
        }
    }

    /**
     * 获取excel的总行数
     * @param fileBytes
     * @param simFileHistoryDTO
     */
    private void getExcelTotalCount(byte[] fileBytes, SimFileHistoryDTO simFileHistoryDTO) throws ExcelAnalysisException {
        try {
            ByteArrayInputStream is = new ByteArrayInputStream(fileBytes);
            ExcelReader excelReader = EasyExcel.read(is, VehicleSimRelRow.class,
                    new VehicleSimRowCountListener(simFileHistoryDTO)).build();
            if (excelReader != null) {
                ReadSheet readSheet = EasyExcel.readSheet(0).build();
                excelReader.read(readSheet);
                excelReader.finish();
            }
        }catch (Exception ex) {
            throw new ExcelAnalysisException("读取车卡关系Excel文件失败");
        }
    }

    /**
     * 获取excel的总行数
     * @param fileBytes
     * @param simFileHistoryDTO
     */
    private void getExcelTotalCountStatus(byte[] fileBytes, SimFileHistoryDTO simFileHistoryDTO) throws ExcelAnalysisException {
        try {
            ByteArrayInputStream is = new ByteArrayInputStream(fileBytes);
            ExcelReader excelReader = EasyExcel.read(is, VehicleStatusRelRow.class,
                    new VehicleStatusRowCountListener(simFileHistoryDTO)).build();
            if (excelReader != null) {
                ReadSheet readSheet = EasyExcel.readSheet(0).build();
                excelReader.read(readSheet);
                excelReader.finish();
            }
        }catch (Exception ex) {
            throw new ExcelAnalysisException("读取车卡关系Excel文件失败");
        }
    }

    /**
     * 异步处理导入文件
     * @param simFileHistoryDTO
     */
    private boolean asyncProcessExcel(byte[] fileBytes, SimFileHistoryDTO simFileHistoryDTO, String tagUuid) {

        // 从文件服务下载需导入的文件
        ByteArrayInputStream is = new ByteArrayInputStream(fileBytes);
        ExcelReader excelReader = null;
        String errorExcelFile = tmpDir + "/" + CuscStringUtils.generateUuid() + ".xlsx";
        // 读取sim卡Excel文件
        try {
            if (simFileHistoryDTO.getFileType() == 1) {
                // 读取车卡关系Excel文件
                excelReader = EasyExcel.read(is, VehicleSimRelRow.class,
                        new VehicleSimListener(simVehicleClient, simFileHistoryDTO, errorExcelFile, tagUuid)).build();
            }
            if (simFileHistoryDTO.getFileType() == FileTypeEnum.CARSTARTUP.getCode()||simFileHistoryDTO.getFileType() == FileTypeEnum.CARCLOSE.getCode()) {
                // 读取车卡关系Excel文件
                excelReader = EasyExcel.read(is, VehicleStatusRelRow.class,
                        new VehicleStatusListener(simVehicleClient, simFileHistoryDTO, errorExcelFile, tagUuid,orgSimRelClient)).build();
            }
        } catch (Exception e) {
            LOGGER.error("导入组织和SIM卡关系错误，原因：", e);
            return false;
        } finally {

            if (is != null){
                try {
                    is.close();
                } catch (IOException e) {
                    log.warn("关闭流失败", e);
                }
            }
            try {
                if (excelReader != null) {
                    ReadSheet readSheet = EasyExcel.readSheet(0).build();
                    excelReader.read(readSheet);
                    excelReader.finish();
                }
                File excelFile = new File(errorExcelFile);
                if (excelFile.exists()) {
					com.cusc.nirvana.user.rnr.fp.dto.FileUploadDTO fileUploadDTO = new com.cusc.nirvana.user.rnr.fp.dto.FileUploadDTO();
                    fileUploadDTO.setPath(errorExcelPath);
                    fileUploadDTO.setUuid(CuscStringUtils.generateUuid());
                    MultipartFile multipartFile = getMultipartFile(excelFile, simFileHistoryDTO);
                    if (multipartFile != null) {
						fileUploadDTO.setFile(multipartFile);
						Response<FileRecordDTO> response = fileSystemClient.uploadFile(fileUploadDTO);
						if (response.getSuccess()) {
							simFileHistoryDTO.setErrorFileAddr(response.getData().getUuid());
						}
                    }
                }
            }finally {
                simFileHistoryDTO.setStatus(1);
                simFileHistoryClient.update(simFileHistoryDTO);
            }
        }
        return true;
    }

    private MultipartFile getMultipartFile(File file, SimFileHistoryDTO simFileHistoryDTO) {
        try {
            FileItem fileItem = new DiskFileItem("excelFile", Files.probeContentType(file.toPath()),
                    false, simFileHistoryDTO.getFileName(), (int) file.length(), file.getParentFile());
            byte[] buffer = new byte[4096];
            int n;
            try (InputStream inputStream = new FileInputStream(file);OutputStream os = fileItem.getOutputStream();) {
                while ((n = inputStream.read(buffer, 0, 4096)) != -1) {
                    os.write(buffer, 0, n);
                }
                MultipartFile multipartFile = new CommonsMultipartFile(fileItem);
                return multipartFile;
            }
        } catch (IOException e) {
            LOGGER.error("上传SIM卡错误文件至文件服务器失败，原因：", e);
        }
        return null;
    }

    /**
     * 新增导入文件信息
     * @param importSimDTO
     * @return
     */
    private SimFileHistoryDTO insertImportFile(ImportSimDTO importSimDTO) {
        SimFileHistoryDTO simFileHistoryDTO = new SimFileHistoryDTO();
        simFileHistoryDTO.setOrgUuid(importSimDTO.getOrgUuid());
        simFileHistoryDTO.setFileName(importSimDTO.getFileName());
        simFileHistoryDTO.setVerification(0);
        simFileHistoryDTO.setErrorInfo("");
        simFileHistoryDTO.setBatchNo("");
        simFileHistoryDTO.setTotalCount(0);
        simFileHistoryDTO.setErrorCount(0);
        simFileHistoryDTO.setStatus(0);
        simFileHistoryDTO.setFileType(importSimDTO.getFileType());
        simFileHistoryDTO.setFileAddr(importSimDTO.getFileUuid());
        simFileHistoryDTO.setImportType(0);
        simFileHistoryDTO.setIsDelete(0);
        simFileHistoryDTO.setCreator(UserSubjectUtil.getUserId());
        Response<SimFileHistoryDTO> response = simFileHistoryClient.add(simFileHistoryDTO);
        return response.getData();
    }


}
