package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.DeleteByUuidDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.ImportSimDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.ISimVehicleRelImportService;
import com.cusc.nirvana.user.rnr.enterprise.service.ISimVehicleService;
import com.cusc.nirvana.user.rnr.fp.client.SimVehicleRelClient;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.client.OrgBusinessTagClient;
import com.cusc.nirvana.user.rnr.mg.client.OrgSimRelClient;
import com.cusc.nirvana.user.rnr.mg.client.SimFileHistoryClient;
import com.cusc.nirvana.user.rnr.mg.dto.OrgBusinessTagDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgBusinessTagQueryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelQueryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryQueryDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Service
public class SimVehicleRelServiceImpl implements ISimVehicleService {

    @Autowired
    private SimFileHistoryClient simFileHistoryClient;

    @Autowired
    private ISimVehicleRelImportService simVehicleRelService;

    @Autowired
    private OrgSimRelClient simRelClient;

    @Autowired
    private SimVehicleRelClient simVehicleRelClient;

    @Autowired
    private OrgBusinessTagClient businessTagClient;

    @Autowired
    private OrganizationClient organizationClient;

    @Override
    public Response<List<OrganizationDTO>> queryOrgByList(OrganizationDTO organizationDTO) {
        return organizationClient.queryByList(organizationDTO);
    }

    @Override
    public Response<?> importSim(ImportSimDTO importSimDTO) {
        return simVehicleRelService.excelProcess(importSimDTO);
    }

    @Override
    public Response<SimVehicleDTO> unbind(BindRequestDTO requestDTO) {
        return simVehicleRelClient.unbind(requestDTO);
    }

    @Override
    public Response<IccidDetailDTO> getSimByUuid(String uuid) {
        Response<OrgSimRelDTO> orgSimResponse = simRelClient.getByUuid(uuid);
        IccidDetailDTO iccidDetailDTO = null;
        if (orgSimResponse.isSuccess() && orgSimResponse.getData() != null) {
            OrgSimRelDTO orgSimRelDTO = orgSimResponse.getData();
            List<String> iccids = new ArrayList<>();
            iccids.add(orgSimRelDTO.getIccid());
            IccidListDTO listDTO = new IccidListDTO();
            listDTO.setIccidList(iccids);
            Response<List<IccidDetailDTO>> simVehicleResponse = simVehicleRelClient.queryByIccids(listDTO);
            if (simVehicleResponse.isSuccess() && !CollectionUtils.isEmpty(simVehicleResponse.getData())) {
                iccidDetailDTO = simVehicleResponse.getData().get(0);
                iccidDetailDTO.setCreateTime(orgSimRelDTO.getCreateTime());
                iccidDetailDTO.setCreator(orgSimRelDTO.getCreator());
            }
        }
        return Response.createSuccess(iccidDetailDTO);
    }

    @Override
    public Response<PageResult<IccidDetailDTO>> querySimByPage(OrgSimRelQueryDTO queryDTO) {
        Response<PageResult<OrgSimRelDTO>> response = simRelClient.queryByPage(queryDTO);
        Map<String, OrgSimRelDTO> map = new LinkedHashMap<>();
        List<IccidDetailDTO> iccidDetailDTOS = new ArrayList<>();
        List<String> iccids = new ArrayList<>();
        if (response.isSuccess() && response.getData() != null) {
            response.getData().getList().forEach(orgSimRelDTO -> {
                map.put(orgSimRelDTO.getIccid(), orgSimRelDTO);
                iccids.add(orgSimRelDTO.getIccid());
            });
        }
        if (!CollectionUtils.isEmpty(iccids)) {
            IccidListDTO listDTO = new IccidListDTO();
            listDTO.setIccidList(iccids);
            Response<List<IccidDetailDTO>> detailResponse = simVehicleRelClient.queryByIccids(listDTO);
            detailResponse.getData().forEach(iccidDetailDTO -> {
                OrgSimRelDTO orgSimRelDTO = map.get(iccidDetailDTO.getIccid());
                iccidDetailDTO.setCreateTime(orgSimRelDTO.getCreateTime());
                iccidDetailDTO.setCreator(orgSimRelDTO.getCreator());
                iccidDetailDTOS.add(iccidDetailDTO);
            });
        }
        PageResult<VinIccidDTO> pageResult = new PageResult(iccidDetailDTOS, response.getData().getTotalCount(),
                response.getData().getPageSize(), response.getData().getCurrPage());
        return Response.createSuccess(pageResult);
    }

    @Override
    public Response<List<VinIccidDTO>> querySimVehicleByVin(String orgUuid, String vin) {
        Response<List<VinIccidDTO>> response = simVehicleRelClient.getIccidByVin(vin);
        if (response.isSuccess() && !CollectionUtils.isEmpty(response.getData())) {
            Iterator<VinIccidDTO> iter = response.getData().iterator();
            while (iter.hasNext()) {
                VinIccidDTO vinIccidDTO = iter.next();
                Response<OrgSimRelDTO> simResponse = simRelClient.getByIccid(vinIccidDTO.getIccid());
                if (simResponse.isSuccess() && simResponse.getData() != null
                        && !simResponse.getData().getOrgUuid().equals(orgUuid)) {
                    iter.remove();
                }
            }
        }
        return response;
    }

    @Override
    public Response<PageResult<SimVehicleResultDTO>> querySimVehiclePage(SimVehicleQueryDTO queryDTO) {
        return simVehicleRelClient.querySimVehiclePage(queryDTO);
    }

    @Override
    public Response deleteImportHisotry(DeleteByUuidDTO uuidDTO) {
        SimFileHistoryDTO simFileHistoryDTO = new SimFileHistoryDTO();
        simFileHistoryDTO.setUuid(uuidDTO.getUuid());
        return simFileHistoryClient.delete(simFileHistoryDTO);
    }

    @Override
    public Response<SimFileHistoryDTO> getImportHistory(String uuid) {
        return simFileHistoryClient.getByUuid(uuid);
    }

    @Override
    public Response<PageResult<SimFileHistoryDTO>> queryImportHistory(SimFileHistoryQueryDTO queryDTO) {
        return simFileHistoryClient.queryByPage(queryDTO);
    }

    @Override
    public Response<OrgBusinessTagDTO> getTagByUuid(String uuid) {
        return businessTagClient.getByUuid(uuid);
    }

    @Override
    public Response<List<OrgBusinessTagDTO>> queryTags(OrgBusinessTagQueryDTO queryDto) {
        return businessTagClient.query(queryDto);
    }

    private Response<List<IccidDetailDTO>> getIccidDetail(List<String> iccids) {
        IccidListDTO listDTO = new IccidListDTO();
        listDTO.setIccidList(iccids);
        Response<List<IccidDetailDTO>> detailResponse = simVehicleRelClient.queryByIccids(listDTO);
        return detailResponse;
    }


}
