package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.dto.T1CommonResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.T1CompletionReqDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.T1InstructionService;
import com.cusc.nirvana.user.rnr.fp.client.IFpT1UploadStatusClient;
import com.cusc.nirvana.user.rnr.fp.client.T1DataUploadClient;
import com.cusc.nirvana.user.rnr.fp.dto.FpT1UploadStatusDTO;
import com.cusc.nirvana.user.rnr.fp.dto.InstructionDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @className: T1InstructionServiceImpl
 * @description: 指令下发
 * @author: jk
 * @date: 2022/7/26 16:57
 * @version: 1.0
 **/
@Slf4j
@Service
public class T1InstructionServiceImpl implements T1InstructionService {
    @Autowired
    private T1DataUploadClient t1DataUploadClient;

    @Autowired
    private IFpT1UploadStatusClient iFpT1UploadStatusClient;

    @Override
    public Response<InstructionDTO> instructionsIssue(InstructionDTO instructionDTO) {
        return t1DataUploadClient.instructions(instructionDTO);
    }


    @Override
    public T1CommonResponseDTO callbackResult(T1CompletionReqDTO t1CompletionReqDTO) {
        FpT1UploadStatusDTO fpT1UploadStatusDTO = new FpT1UploadStatusDTO();

        fpT1UploadStatusDTO.setIccid(t1CompletionReqDTO.getICCID());
        Response<List<FpT1UploadStatusDTO>> response = iFpT1UploadStatusClient.queryByList(fpT1UploadStatusDTO);
        if(!response.isSuccess()|| CollectionUtils.isEmpty(response.getData())){
            return T1CommonResponseDTO.createError(t1CompletionReqDTO.getRequestID(),"没有查询到相关卡信息");
        }
        FpT1UploadStatusDTO fpT1UploadStatus = response.getData().get(0);

        fpT1UploadStatus.setReportCompletion(t1CompletionReqDTO.getIsAuth()?1:2);
        fpT1UploadStatus.setReportCompletionResp(t1CompletionReqDTO.getMessage());

        Response<FpT1UploadStatusDTO> fpT1UploadStatusDTOResponse = iFpT1UploadStatusClient.update(fpT1UploadStatus);
        if(!fpT1UploadStatusDTOResponse.isSuccess()){
            return T1CommonResponseDTO.createError(t1CompletionReqDTO.getRequestID(),"更新数据失败");
        }
        return T1CommonResponseDTO.createSuccess(t1CompletionReqDTO.getRequestID());
    }

}
