package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.client.*;
import com.cusc.nirvana.user.eiam.dto.*;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.service.IUserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author stayAnd
 * @date 2022/4/14
 * 用户相关
 */
@Service
@Slf4j
public class UserServiceImpl implements IUserService {


    @Resource
    private UserOrganClient userOrganClient;

    @Resource
    private UserClient userClient;

    @Resource
    private UserRoleClient userRoleClient;

    @Value("${user.eiam.applicationId:4}")
    private String applicationId;

    @Resource
    private RoleClient roleClient;


    @Resource
    private OrganizationClient organizationClient;


    @Override
    public String getOrganId(String userId,String tenantNo) {
        UserOrganDTO dto = new UserOrganDTO();
        dto.setUserId(userId);
        dto.setTenantNo(tenantNo);
        log.info("getOrganId param = {}",JSONObject.toJSONString(dto));
        Response<List<UserOrganDTO>> listResponse = userOrganClient.queryByList(dto);
        return listResponse.getData().stream().filter(userOrganDTO -> userId.equals(userOrganDTO.getUserId()))
                .findFirst().map(UserOrganDTO::getOrganId).orElse(null);
    }

    @Override
    public void addUserRelationRole(String userId, String organId, String roleCode) {

        //新增经销商和管理员关系
        UserOrganDTO userOrganDto = new UserOrganDTO();
        userOrganDto.setUserId(userId);
        userOrganDto.setTenantNo(UserSubjectUtil.getTenantNo());
        userOrganDto.setOrganId(organId);
        userOrganClient.add(userOrganDto);

        //新增用户角色关系
        UserRoleDTO userRoleDto = new UserRoleDTO();
        userRoleDto.setUserId(userId);
        userRoleDto.setRoleId(getRoleId(roleCode));
        userRoleDto.setApplicationId(applicationId);
        userRoleDto.setTenantNo(UserSubjectUtil.getTenantNo());
        userRoleClient.add(userRoleDto);
    }

    @Override
    public String getRoleId(String roleCode) {
        RoleDTO dto = new RoleDTO();
        dto.setApplicationId(applicationId);
        dto.setTenantNo(UserSubjectUtil.getTenantNo());
        dto.setRoleCode(roleCode);
        Response<RoleDTO> roleDTOResponse = roleClient.get(dto);
        if (roleDTOResponse != null && roleDTOResponse.isSuccess() && null != roleDTOResponse.getData()) {
            return roleDTOResponse.getData().getUuid();
        }
        return "";
    }

    @Override
    public String getOrganName(String organId,String tenantNo) {
        OrganizationDTO query = new OrganizationDTO();
        query.setTenantNo(tenantNo);
        query.setUuid(organId);
        Response<OrganizationDTO> response = organizationClient.getByUuid(query);
        if (response != null && response.isSuccess() && response.getData() != null) {
            return response.getData().getOrganName();
        }
        return null;
    }

    @Override
    public UserDTO queryUserInInfoByPhone(String phone) {
        UserDTO queryDto = new UserDTO();
        queryDto.setTenantNo(UserSubjectUtil.getTenantNo());
        queryDto.setApplicationId(applicationId);
        queryDto.setPhone(phone);

        Response<UserDTO> response = userClient.getByPhone(queryDto);
        if (!response.isSuccess()) {
            throw new CuscUserException(response.getCode(), response.getMsg());
        }
        log.info("queryUserInInfoByPhone request = {} | response = {}", JSONObject.toJSONString(queryDto),
                JSONObject.toJSONString(response));
        return response.getData();
    }

    @Override
    public UserDTO addUser(UserDTO userDTO) {
        Response userResponse = userClient.add(userDTO);
        if (null == userResponse) {
            throw new CuscUserException("", "新增用户失败");
        }

        if (!userResponse.isSuccess() || userResponse.getData() == null) {
            throw new CuscUserException(userResponse.getCode(), userResponse.getMsg());
        }

        return JSONObject.parseObject(JSONObject.toJSONString(userResponse.getData()), UserDTO.class);
    }

    @Override
    public void updateUserRole(String roleCode, String userId) {
        String roleId = this.getRoleId(roleCode);
        if (StringUtils.isNotBlank(roleId)) {
            UserRoleDTO updateDto = new UserRoleDTO();
            updateDto.setUserId(userId);
            updateDto.setRoleId(roleId);
            userRoleClient.updateByUserId(updateDto);
        }

    }
}
