package com.cusc.nirvana.user.rnr.enterprise.util;

import org.apache.commons.lang3.StringUtils;

/**
 * 脱敏工具类
 *
 * @author yubo
 * @since 2022-05-03 10:26
 */
public class DesensitizationUtil {
	/**
	 * 不脱敏
	 */
	public static final int DESENSITIZATION_NONE = 0;

	/**
	 * 对手机号脱敏
	 */
	public static final int DESENSITIZATION_PHONE = 1;


	/**
	 * 对姓名脱敏
	 */
	public static final int DESENSITIZATION_NAME = 2;


	/**
	 * 对姓名脱敏
	 */
	public static final int DESENSITIZATION_VIN = 3;

    /**
     * 手机号脱敏
     *
     * @param phone
     * @return
     */
    public static String desensitizePhone(String phone) {
        if (StringUtils.isNotBlank(phone)) {
            return phone.replaceAll("(\\w{3})\\w{4}(\\w{4})", "$1****$2");
        }
        return phone;
    }

    /**
     * 姓名脱敏
     *
     * @param name
     * @return
     */
    public static String desensitizeName(String name) {
        if (StringUtils.isNotBlank(name)) {
            String left = StringUtils.left(name, 1);
            return StringUtils.rightPad(left, name.length(), "*");
        }
        return name;
    }


	public static String desensitizeByRule(int rule,String content){
		if (rule == DESENSITIZATION_PHONE){
			return desensitizePhone(content);
		}
		if (rule == DESENSITIZATION_NAME) {
			return desensitizeName(content);
		}
		if (rule == DESENSITIZATION_VIN) {
			return desensitizeVin(content);
		}
		return content;
	}

	public static String desensitizeVin(String vin){
		//vin 17 位
		if (StringUtils.isNotBlank(vin)) {
			//vin.replace(3,13)
			String substring = vin.substring(4, 13);
			return vin.replace(substring,"**********");
		}
		return vin;
	}

	public static String desensitizeIccid(String iccid){
		//iccid 20 位
		if (StringUtils.isNotBlank(iccid)) {
			String substring = iccid.substring(5, 15);
			return iccid.replace(substring,"**********");
		}
		return iccid;
	}

	public static String desensitizeCertNumber(String certNumber) {

		if (StringUtils.isNotBlank(certNumber)) {
			if (certNumber.length() >= 10) {
				//展示前4后4
				String substring = certNumber.substring(4, certNumber.length()-4);
				return certNumber.replace(substring,"**********");
			} else if (certNumber.length() >= 3){
				//展示前1后1
				String substring = certNumber.substring(1, certNumber.length()-1);
				return certNumber.replace(substring,"**********");
			} else {
				return certNumber;
			}
		}
		return certNumber;
	}
}
